unit AngleBitmap;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  System.Actions, System.Math.vectors, System.UIConsts, System.IOUtils, System.iniFiles,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Objects, FMX.Layouts,
  FMX.Platform, FMX.clipBoard, FMX.StdCtrls, FMX.ActnList,
  FMX.Memo, FMX.ListBox, FMX.Graphics, FMX.Utils,
  FMX.Controls.Presentation, FMX.Edit, FMX.EditBox, FMX.SpinBox, FMX.Colors,
  avProbe, FMX.TMSToolBar, FMX.TMSBitmapContainer,
  images, maths, FMX.TMSBaseControl, FMX.TMSBaseGroup, FMX.TMSRadioGroup,
  grapheOptique, uniteker;

type
  Taxe = (aX,aY,apX,apY);
  TMesure = (mAngle,mDistance);

  TOptiqueBitmapForm = class(TForm)
    OpenDialog: TOpenDialog;
    TMSFMXToolBar1: TTMSFMXToolBar;
    OpenBtn: TTMSFMXToolBarButton;
    RegressiBtn: TTMSFMXToolBarButton;
    PanelPreview: TPanel;
    Image: TImage;
    EchelleLabel: TLabel;
    CouleurCB: TTMSFMXToolBarColorPicker;
    Panel1: TPanel;
    EchelleGB: TGroupBox;
    MesureRG: TTMSFMXRadioGroup;
    AngleLabel: TText;
    LabelA: TText;
    LabelO: TText;
    EchelleRG: TComboBox;
    Panel2: TPanel;
    ValeurMesureGB: TGroupBox;
    StatusBar1: TStatusBar;
    largeurSE: TSpinBox;
    Label1: TLabel;
    UniteCB: TComboBox;
    EchelleEdit: TEdit;
    OrigineEdit: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    AngleLabelBis: TLabel;
    procedure OpenButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Single);
    procedure RegressiBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CouleurCBColorSelected(Sender: TObject; AColor: TAlphaColor);
    procedure RazBtnClick(Sender: TObject);
    procedure TraceEchCBChange(Sender: TObject);
    procedure TracePointsCBChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure PanelPreviewResize(Sender: TObject);
    procedure EchelleEditKeyDown(Sender: TObject; var Key: Word;
      var KeyChar: Char; Shift: TShiftState);
    procedure ImageClick(Sender: TObject);
    procedure IntensitePBMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
    procedure IntensitePBMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
  private
    longueurRef,origineEchelle,Gamma : double;
    signeY : integer;
    PointRepere : TarrayPoint;
    e_PointRepere : TarrayPoint;
    BorneSelect : TstyleDragAngle;
    largeur, hauteur : integer;  // bitmap
    LargeurAff,HauteurAff : integer; // Affiche
    LargeurI,HauteurI : integer; // sur l'image
    pasZoom : single; // conversion xy(video) vers xy(cran)
    couleurAxe : TAlphaColor;
    nomFichier : string;
    bitmapCourant : tBitmap;
    coeffUnite : double;
    mesure : Tmesure;
    axeI,axeE : Taxe;
    Pente,invCosa : double;
    magnetInterf : integer;
    largeurTrait : single;
    couleurTrait : TAlphaColor;
    xRef,xCourant : single;
    axeDesX : Tunite;
    LigneCourante : integer;
    procedure Dessine;
    procedure EcranVersImage;
    procedure SetCurseur(Acursor : Tcursor);
    function AppartientImage(var X, Y: single) : boolean;
    Function GetBorne(x,y : single) : TstyleDragOptique;
    procedure setBorne(aBorne : TStyleDragOptique);
    procedure setBoutons;
    procedure ImageVersEcran;
    procedure effectuerZoom;
    procedure ConvertitXY(Sender: TObject;var X, Y: single);
    procedure setEchelle;
    procedure traceAngle;
    procedure traceDistance;
    procedure TraceFleche(P1,P2: TpointF);
    procedure cherchePente;
    procedure traceBorne;
    procedure setRGB;
    procedure calculIntensite;
    function RGBToL(Couleur : TAlphaColorRec) : double;
    procedure OnApplicationHint(Sender: TObject);
  public
  end;

var
  OptiqueBitmapForm: TOptiqueBitmapForm;

implementation

{$R *.fmx}

uses math, avcodec, avlib,
     regutil, compile, valeurs, graphker, grapheVariabU, regmain;

const
    Titre = 'Optique';
    FormCaption = 'Lecture d''angle ';
    TickEchelle = 4;
    rayonAngle = 50;
    longFleche = 16;
    hintX : array[boolean] of string =
          ('Cliquer pour mesurer une distance',
           'Cliquer pour indiquer uniquement le point courant');

procedure SetPixel(abitmap : tbitmap;i,j : integer;acolor : TAlphaColor);
 // example of setting a single pixel in a Tbitmap.
var
  vBitMapData  : TBitmapData;
begin
 // lock and set the bitmap pixels
      if aBitmap.Map(TMapAccess.Write, vBitMapData) then begin
          vBitmapData.SetPixel(i,j, aColor); // set the pixel colour at i, j
          aBitmap.Unmap(vBitMapData);         // unlock the bitmap
      end;
end;

function GetPixel(abitmap : tbitmap;i,j : integer) : TAlphaColorRec;
 // example of setting a single pixel in a Tbitmap.
var
  vBitMapData  : TBitmapData;
  LColor : TAlphaColor;
  rec : TAlphaColorRec;
begin
 // lock and get the bitmap pixels
      if aBitmap.Map(TMapAccess.Read, vBitMapData) then begin
          LColor := vBitmapData.GetPixel(i,j); // set the pixel colour at i, j
          result := TAlphaColorRec(LColor);
          aBitmap.Unmap(vBitMapData);         // unlock the bitmap
      end
      else begin
         rec.A := $FF;
         rec.R := $1C;
         rec.G := $90;
         rec.B := $EF;
         result := rec;
      end;
end;


procedure TOptiqueBitmapForm.ConvertitXY(Sender: TObject;var X, Y: single);
begin
    x := x-panelPreview.Position.x;
    y := y-panelPreview.Position.y;
end;

procedure TOptiqueBitmapForm.OpenButtonClick(Sender: TObject);

procedure RecupereData;
begin
       largeur := bitmapCourant.Width;
       hauteur := bitmapCourant.Height;
       PointRepere[bsOrigine] := pointF(Largeur/3,2*Hauteur/3);
       PointRepere[bsEchelle1] := pointF(Largeur/4,3*Hauteur/4);
       PointRepere[bsEchelle2] := pointF(3*Largeur/4,3*Hauteur/4);
       EffectuerZoom;
end;

begin
  if OpenDialog.Execute then begin
    NomFichier := OpenDialog.FileName;
    setCurseur(crDefault);
    pasZoom := 1;
    setBorne(bsNone);
    Caption := 'Lecture de '+extractFileName(NomFichier);
    bitmapCourant.LoadFromFile(NomFichier);
    RecupereData;
    dessine;
    setBoutons;
  end;
end;

procedure TOptiqueBitmapForm.PanelPreviewResize(Sender: TObject);
begin
    effectuerZoom;
end;

procedure TOptiqueBitmapForm.RazBtnClick(Sender: TObject);
begin
      if OKReg('Suppression des donnes',0) then begin
         dessine;
         setBoutons;
     end;
end;

procedure TOptiqueBitmapForm.RegressiBtnClick(Sender: TObject);

function Ajoute(nom,unite,signif : string) : codeGrandeur;
var  v : codeGrandeur;
begin
      v :=indexNom(nom);
      if v=grandeurInconnue then begin
         v := ajouteExperimentale(Nom,variable);
         grandeurs[v].fonct.expression := signif;
         grandeurs[v].NomUnite := unite;
      end;
      result := v;
end;

begin // SauveData
end; // SauveData

procedure TOptiqueBitmapForm.FormActivate(Sender: TObject);
begin
    setBoutons;
    Application.OnHint := OnApplicationHint;
end;

procedure TOptiqueBitmapForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     if pages[pageCourante].nmes=0 then FRegressiMain.FileNewSimulationItemClick(Sender);
     Application.OnHint := OnApplicationHint;
end;

procedure TOptiqueBitmapForm.FormCreate(Sender: TObject);
var i : TStyleDragOptique;
    j : integer;
    FichierIni : TInifile;
begin
  LargeurAff := 640;
  HauteurAff := 480;
  Largeur := 640;
  Hauteur := 480;
  pasZoom := 1;
  signeY := -1;
  bitmapCourant := tbitmap.Create;
  couleurAxe := claBlue;
  couleurCB.SelectedColor := couleurAxe;
  borneSelect := bsNone;
  for i := low(TstyleDragOptique) to High(TstyleDragOptique) do begin
        e_pointRepere[i].x := 0;
        e_pointRepere[i].y := 0;

  LigneCourante := 1;
  NomFichier := '';
  axeDesX := Tunite.Create;
  axeDesX.nom := 'x';
  axeDesX.nomUnite := 'm';
  FichierIni := TIniFile.create(registreReg);
  LargeurSE.value := FichierIni.readInteger(Titre,'Largeur',2);
  largeurTrait := LargeurSE.value;
  EchelleEdit.text := FichierIni.readString(Titre,'Echelle','10');
  OrigineEdit.text := FichierIni.readString(Titre,'Origine','0');
  Gamma := FichierIni.readInteger(Titre,'Gamma',100)/100;
  uniteCB.itemIndex  := FichierIni.readInteger(Titre,'Unite',2);
  coeffUnite := power(10,-3*uniteCB.ItemIndex);
  echelleRG.itemIndex := FichierIni.readInteger(Titre,'TypeEchelle',0);
  setEchelle;
  OpenDialog.filterIndex := FichierIni.readInteger(Titre,'FilterOpen',0);
  couleurCB.selectedColor := FichierIni.readInteger(Titre,'CouleurAngle',0);
  couleurTrait := couleurCB.SelectedColor;
  FichierIni.free;
  setRGB;
  end;
end;

procedure TOptiqueBitmapForm.FormDestroy(Sender: TObject);
begin
    bitmapCourant.free;
end;

procedure TOptiqueBitmapForm.setBorne(aBorne : TStyleDragOptique);
begin
     borneSelect := aBorne;
     if BorneSelect in [asOrigine,asEchelle1,asEchelle2,asAngle1,asAngle2,asAngle3]
         then setCurseur(crNone)
         else begin
            setCurseur(crDefault); // bsNone
         end;
end;

procedure TOptiqueBitmapForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
    convertitXY(sender,x,y);
    if (Button<>TMouseButton.mbLeft) or not appartientImage(x,y) then exit;
    if borneSelect=bsNone then
       setBorne(GetBorne(x,y));
    e_pointRepere[BorneSelect].x := x;
    e_pointRepere[BorneSelect].y := y;
    ecranVersImage;
end;

procedure TOptiqueBitmapForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single);
var BS : TstyleDragOptique;
    redessin : boolean;
begin  // MouseMove
convertitXY(sender,x,y);
if not appartientImage(x,y) then begin
   setCurseur(crDefault);
   exit;
end;
Redessin := borneSelect in [bsOrigine,bsEchelle1,bsEchelle2];
image.OnMouseMove := nil;
e_pointRepere[BorneSelect].x := x;
e_pointRepere[BorneSelect].y := y;
case BorneSelect of
     bsOrigine, bsEchelle1, bsEchelle2 : ;
     asAngle1,asAngle2,asAngle3 : ;
     asNone : begin
           BS := GetBorne(x,y);
           if BS in [bsEchelle1,bsEchelle2,bsOrigine]
              then setCurseur(crSizeAll)
              else setCurseur(crDefault);
     end;
end;
ecranVersImage;
if Redessin then dessine;
image.OnMouseMove := formMouseMove;
end;

procedure TOptiqueBitmapForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  convertitXY(sender,x,y);
  if not appartientImage(x,y) then exit;
  e_pointRepere[BorneSelect].x := x;
  e_pointRepere[BorneSelect].y := y;
  ecranVersImage;
  case BorneSelect of
     bsNone : ;
     bsEchelle1,bsEchelle2,bsOrigine : begin
         setBorne(bsNone);
         dessine;
     end;
     asAngle1,asAngle2,asAngle3 : begin
         setBorne(bsNone);
         dessine;
     end;
  end;
end;

procedure TOptiqueBitmapForm.TraceEchCBChange(Sender: TObject);
begin
    dessine;
end;

procedure TOptiqueBitmapForm.TracePointsCBChange(Sender: TObject);
begin
    dessine;
end;

procedure TOptiqueBitmapForm.ImageVersEcran;
var i : TstyleDragOptique;
begin
    for i := low(TstyleDragOptique) to High(TstyleDragOptique) do
        e_pointRepere[i] := pointRepere[i]*pasZoom;
end;

procedure TOptiqueBitmapForm.ImageClick(Sender: TObject);
begin
  case BorneSelect of
     bsNone : ;
     bsEchelle1,bsEchelle2,bsOrigine : begin
         setBorne(bsNone);
         dessine;
     end;
     asAngle1,asAngle2,asAngle3 : begin
         setBorne(bsNone);
         dessine;
     end;
  end;
end;

procedure TOptiqueBitmapForm.IntensitePBMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Single);
begin
     xRef := x;
end;

procedure TOptiqueBitmapForm.IntensitePBMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Single);
begin // PaintBoxMouseMove
        xCourant := x;
end;

procedure TOptiqueBitmapForm.EchelleEditKeyDown(Sender: TObject; var Key: Word;
  var KeyChar: Char; Shift: TShiftState);
begin
  if not CharInSet(keyChar,['0'..'9','.',',',
                     crSupprArr,crCR,crTab,crGauche,crDroite]) then keyChar := #0
end;

procedure TOptiqueBitmapForm.EcranVersImage;
var i : TstyleDragOptique;
begin
       for i := low(TstyleDragOptique) to High(TstyleDragOptique) do
           pointRepere[i] := e_pointRepere[i]/pasZoom;
end;

procedure TOptiqueBitmapForm.CouleurCBColorSelected(Sender: TObject;
  AColor: TAlphaColor);
begin
  couleurAxe := Acolor;
  dessine;
end;

procedure TOptiqueBitmapForm.SetCurseur(Acursor : Tcursor);
begin
      image.Cursor := aCursor;
end;

function TOptiqueBitmapForm.AppartientImage(var X, Y: single) : boolean;
begin
    result := (x>0) and (y>0) and (y<hauteurAff) and (x<largeurAff)
end;

Function TOptiqueBitmapForm.GetBorne(x,y : single) : TstyleDragOptique;
const
    Magnet = 16;
var i,imin,imax : TstyleDragOptique;
    aPoint : TPointF;
begin
      result := bsNone;
      aPoint := PointF(x,y);
 //     imin := Low(TstyleDragOptique);
      imax := High(TstyleDragOptique);
      case mesure of
           mAngle : begin
              imin := bsEchelle1;
              imax := asAngle3;
           end;
           mDistance : begin
              imin := asEchelle1;
              imax := asAngle2;
           end;
           else begin
              imin := bsOrigine;
              imax := bsEchelle2;
           end;
      end;
      for i := imin to imax do
          if e_PointRepere[i].distance(aPoint) < Magnet
              then begin
                 result := i;
                 exit;
              end;
end;

procedure TOptiqueBitmapForm.EffectuerZoom;
var zoomX,zoomY,zoom : single;
begin
    if (nomFichier='') then exit;
    zoomX := panelPreview.width / largeur;
    zoomY := panelPreview.height / hauteur;
    if zoomX>zoomY then zoom := zoomY else zoom := zoomX;
    if zoom>1 then pasZoom := floor(zoom) else pasZoom := zoom;
    hauteurAff := round(hauteur*pasZoom);
    largeurAff := round(largeur*pasZoom);
    Image.Height := hauteurAff;
    Image.Width := largeurAff;
    imageVersEcran;
end;

procedure TOptiqueBitmapForm.Dessine;
const tailleCroix = 3;
var imageCanvas : TCanvas;

Procedure TraceCroix(Apoint : TpointF);
var point1,point2 : TPointF;
begin
       point1 := Apoint;
       point2 := Apoint;
       point1.Offset(-tailleCroix,0);
       point2.Offset(+tailleCroix,0);
       imageCanvas.drawLine(point1,point2,1);
       point1 := Apoint;
       point2 := Apoint;
       point1.offset(0,-tailleCroix);
       point2.offset(0,+tailleCroix);
       imageCanvas.drawLine(point1,point2,1);
end;

Procedure TraceOrigine;

Procedure TraceEchelle;
begin
      imageCanvas.drawLine(PointRepere[bsEchelle1],PointRepere[bsechelle2],1);
      TraceCroix(PointRepere[bsEchelle1]);
      TraceCroix(PointRepere[bsechelle2]);
end;

procedure TraceFleche(dX,dY : single);
    var
      PC: TPointF; // centre de la base de la flche
      deltaX, deltaY: single;
      Points:  TPolygon; // Flche
      PE : TPointF; // extrmit de l'axe
      rayon : single;
    begin
       rayon := min(largeur,hauteur)/3;
       PE := PointRepere[bsOrigine];
       deltaX := rayon*dX;
       deltaY := rayon*dY;
       PE.offset(deltaX,deltaY);
       imageCanvas.drawLine(PointRepere[bsOrigine],PE,1);
       rayon := rayon/10;
       PC := PE;
       deltaX := rayon*dX;
       deltaY := rayon*dY;
       PC.Offset(-deltaX,-deltaY);
       deltaX := deltaX/3;
       deltaY := deltaY/3;
       setLength(Points,3);
       Points[0] := PE;
       PC.offset(-deltaY,+deltaX);
       Points[1] := PC;
       PC.offset(+2*deltaY,-2*deltaX);
       Points[2] := PC;
       imageCanvas.DrawPolygon(Points,1);
    end;

begin // traceOrigine
         imageCanvas.stroke.color := couleurAxe;
         TraceFleche(0,signeY);
         TraceFleche(1,0);
         TraceEchelle;
end; // traceOrigine

begin // Dessine
    if (nomFichier='') then exit;
    Image.Bitmap.Assign(bitmapCourant);
    imageCanvas := Image.bitmap.Canvas;
    ImageCanvas.BeginScene;
    imageCanvas.stroke.thickness := ceil(1/pasZoom);
    TraceOrigine;
    ImageCanvas.EndScene;
end; // Dessine

procedure TOptiqueBitmapForm.setBoutons;
var OK : boolean;
begin
    OK := (nomFichier<>'');
    RegressiBtn.enabled := OK;
    if (borneSelect=bsNone) then begin
       setCurseur(crDefault);
    end;
end; // setBoutons

function GetScanline(ABitmap: TBitmap; ligne : integer): PAlphaColorArray;
var
  bitdata1: TBitmapData;
begin
  if ABitmap.Map(TMapAccess.Read, bitdata1) then begin
    try
      result := bitdata1.getScanLine(ligne);
    finally
      ABitmap.Unmap(bitdata1);
    end;
  end
  else result := nil
end;


Procedure TOptiqueBitmapForm.SetEchelle;
var EchelleEnPixel : single;
    longueurEchelle : single;
begin
       if PointRepere[bsEchelle2].y<0 then
            PointRepere[bsEchelle2].y := 0;
       if PointRepere[bsEchelle2].y>hauteur then
            PointRepere[bsEchelle2].y := hauteur;
       if PointRepere[bsEchelle1].x>largeur then
            PointRepere[bsEchelle1].x := largeur;
       if PointRepere[bsEchelle1].x<0 then
            PointRepere[bsEchelle1].x := 0;
        echelleEnPixel := PointRepere[bsEchelle1].distance(PointRepere[bsechelle2]);
        try
           LongueurEchelle := StrToFloatWin(EchelleEdit.text);
        except
           showMessage('Erreur lors de la lecture de l''chelle');
           EchelleEdit.text := '1.0';
           EchelleEdit.SetFocus;
           longueurEchelle := 1;
        end;
        try
        except
           showMessage('Erreur lors de la lecture de l''chelle');
        end;
        pente := LongueurEchelle/EchelleEnPixel;
end;

procedure ToptiqueBitmapForm.TraceAngle;
var angleRef,angleFin : TStyleDragOptique;
    imageCanvas : TCanvas;

   function valeurAngle : string;
   var CA,CB : TpointF;
       PS,PV : single;
       valeur : double;
   begin
      CA.x := pointRepere[asAngle1].x-pointRepere[asAngle2].x;
      CA.y := pointRepere[asAngle1].y-pointRepere[asAngle2].y;
      CB.x := pointRepere[asAngle3].x-pointRepere[asAngle2].x;
      CB.y := pointRepere[asAngle3].y-pointRepere[asAngle2].y;
      PS := CA.X*CB.X+CA.Y*CB.Y;
      PV := -CA.X*CB.Y+CA.Y*CB.X;
      valeur := radToDeg(arcTan2(PV,PS));
      if valeur>0 then angleRef := asAngle1 else angleRef := asAngle3;
      if angleRef=asAngle1 then angleFin := asAngle3 else angleFin := asAngle1;
      result := intToStr(round(valeur))+' ';
   end;

   var xt,yt,signe : single;
       lT : double;
       dx,dy : single;
       Points : TPolygon;
       StartAngle,StopAngle : single;
   begin
       angleLabel.text := valeurAngle;
       angleLabelBis.text := valeurAngle;
       imageCanvas := Image.bitmap.Canvas;
       StartAngle := pointRepere[asAngle2].Angle(pointRepere[asAngle1]);
       StopAngle := pointRepere[asAngle2].Angle(pointRepere[asAngle3]);
       imageCanvas.DrawArc(pointRepere[asAngle2],TpointF.Create(rayonAngle, rayonAngle),
                           StartAngle,StopAngle-StartAngle,1);
       imageCanvas.DrawLine(pointRepere[asAngle2],pointRepere[asAngle1],1);
       imageCanvas.DrawLine(pointRepere[asAngle2],pointRepere[asAngle3],1);
       xT := -pointRepere[asAngle2].x+((pointRepere[asAngle3].x+pointRepere[asAngle1].x)/2);
       yT := -pointRepere[asAngle2].y+((pointRepere[asAngle3].y+pointRepere[asAngle1].y)/2);
       lT := rayonAngle/sqrt(sqr(xt)+sqr(yt));
       angleLabel.position.x := round(pointRepere[asAngle2].x+xT*lT/pasZoom);
       if angleRef=asAngle1 then signe := 0 else signe := -1;
       angleLabel.position.y := round(pointRepere[asAngle2].y/pasZoom+yT*lT+signe*angleLabel.Height);
// L'arc traverse le primtre de l'ellipse circonscrite par les points (X1,Y1) et (X2,Y2).
// L'arc est dessin en suivant le primtre de l'ellipse, dans le sens contraire des aiguilles d'une montre
// jusqu'au point d'arrive. Le point de dpart est dfini par l'intersection de l'ellipse et d'une ligne dfinie
// par le centre de l'ellipse et (X3,Y3). Le point d'arrive est dfini par l'intersection de l'ellipse et d'une ligne
// dfinie par le centre de l'ellipse et (X4,Y4).
        dx := pointRepere[asAngle3].X- pointRepere[asAngle2].X;
        dy := pointRepere[asAngle3].y- pointRepere[asAngle2].y;
        lT := rayonAngle/sqrt(sqr(dX) + sqr(dY));
        SetLength(Points, 3);
        Points[0] := PointF(pointRepere[asAngle2].X+lT * dX,
                           pointRepere[asAngle2].y+lT * dY);
        lT := longFleche/sqrt(sqr(dX) + sqr(dY));
        dX := round(lT * dX);
        dY := round(lT * dY);
        if angleRef=asAngle1 then begin dX := -dX; dY := -dY end;
        Points[1] := PointF(Points[0].x + dY - (dX/3), Points[0].Y - dX - (dY/3));
        Points[2] := PointF(Points[0].x + dY + (dX/3), Points[0].y - dX + (dY/3));
        imageCanvas.DrawPolygon(Points,1); // triangle 3 points
   end;

   procedure ToptiqueBitmapForm.TraceDistance;
   var dimPix,Larrondi : double;
       unite : string;
       xt,yt : single;
       valeur : double;
       NbrePixelEchelle : double;
       imageCanvas : TCanvas;
   begin
       unite := ' m';
       imageCanvas := Image.bitmap.Canvas;
       case echelleRG.itemIndex of
          1 : dimPix := longueurRef;
          0 : begin
             NbrePixelEchelle := sqrt(
                sqr(PointRepere[bsEchelle1].x-PointRepere[bsEchelle2].x)+
                sqr(PointRepere[bsEchelle1].y-PointRepere[bsechelle2].y));
             dimPix := LongueurRef/NbrePixelEchelle;
          end;
          else begin
             dimPix := 1;
             unite := '';
          end;
       end;
       LArrondi  := power(10, floor(log10(abs(dimPix))));
       valeur := sqrt(sqr(pointRepere[asAngle2].x-pointRepere[asAngle1].x)+
                      sqr(pointRepere[asAngle2].y-pointRepere[asAngle1].y))
                 *dimPix;
       angleLabel.text := FormatReg(LArrondi * round(valeur / LArrondi))+unite;
       angleLabelBis.text := angleLabel.text;
       imageCanvas.drawLine(pointRepere[asAngle2],pointRepere[asAngle1],1);
       traceFleche(pointRepere[asAngle1],pointRepere[asAngle2]);
       traceFleche(pointRepere[asAngle2],pointRepere[asAngle1]);
       xT := ((pointRepere[asAngle2].x+pointRepere[asAngle1].x)/2)+20;
       yT := ((pointRepere[asAngle2].y+pointRepere[asAngle1].y)/2)+20;
       angleLabel.position.x := round(xt/pasZoom);
       angleLabel.position.y := round(yt/pasZoom);
   end;

    procedure ToptiqueBitmapForm.TraceFleche(P1,P2: TpointF);
    var
      dx,dy: single;
      Points:  TPolygon;
      d : double;
      imageCanvas : TCanvas;
    begin // P2 = extrmit de la flche ; P1-P2 = direction
      imageCanvas := Image.bitmap.Canvas;
      dx := P2.X-P1.X;
      dy := P2.y-P1.y;
      d := longFleche/sqrt(sqr(dX) + sqr(dY));
      dX := round(d * dX);
      dY := round(d * dY);
      SetLength(Points, 3);
      Points[0] := P2;
      Points[1] := TPointF.create(P2.x - dX - (dY/3), P2.Y - dY + (dX/3));
      Points[2] := TPointF.create(P2.x - dX + (dY/3), P2.y - dY - (dX/3));
      imageCanvas.DrawPolygon(Points,1); // triangle 3 points
    end;

    procedure ToptiqueBitmapForm.cherchePente;
var penteE : double;
begin
  if (PointRepere[bsOrigine].x=PointRepere[bsDirection].x)
        then begin
           axeI := aY;
           pente := 0;
           invCosA := 1;
        end
        else if (PointRepere[bsOrigine].y=PointRepere[bsDirection].y)
           then begin
              axeI := aX;
              pente := 0;
              invCosA := 1;
           end
           else begin
              pente := (PointRepere[bsOrigine].y-PointRepere[bsdirection].y)/
                       (PointRepere[bsOrigine].x-PointRepere[bsdirection].x);
              if abs(pente)<=1
                 then begin
                    axeI := apX;
                    invcosa := sqrt(1+sqr(pente));
                 end
                 else begin
                    axeI := apY;
                    invcosa := sqrt(1+sqr(1/pente));
                 end;
           end;
  if (PointRepere[bsEchelle1].x=PointRepere[bsEchelle2].x)
        then axeE := aY
        else if (PointRepere[bsEchelle1].y=PointRepere[bsEchelle2].y)
           then axeE := aX
           else begin
              penteE := (PointRepere[bsOrigine].y-PointRepere[bsdirection].y)/
                       (PointRepere[bsOrigine].x-PointRepere[bsdirection].x);
              if abs(penteE)<=1
                 then axeE := apX
                 else axeE := apY;
           end;
end;

procedure TOptiqueBitmapForm.setRGB;
begin
    CalculIntensite;
end;

Procedure TOptiqueBitmapForm.CalculIntensite;
var dimPix : double;
    origineX,origineY : single;

Procedure chercheEchelle;
var NbrePixelEchelle : double;
begin
   OrigineX := PointRepere[bsOrigine].x;
   OrigineY := PointRepere[bsOrigine].y;
   case echelleRG.itemIndex of
        1 : dimPix := LongueurRef;
        0 : begin
          NbrePixelEchelle := sqrt(
             sqr(PointRepere[bsEchelle1].x-PointRepere[bsEchelle2].x)+
             sqr(PointRepere[bsEchelle1].y-PointRepere[bsechelle2].y));
          dimPix := LongueurRef/NbrePixelEchelle;
          case axeI of
             aX : OrigineX := PointRepere[bsEchelle1].x;
             aY : OrigineY := PointRepere[bsEchelle1].y;
             else begin
                OrigineX := round((pente*(PointRepere[bsEchelle1].y-PointRepere[bsOrigine].y)+
                             sqr(pente)*PointRepere[bsOrigine].x+
                             PointRepere[bsEchelle1].x)/(1+sqr(pente)));
                OrigineY := round((pente*(PointRepere[bsEchelle1].x-PointRepere[bsOrigine].x)+
                             sqr(pente)*PointRepere[bsEchelle1].y+
                             PointRepere[bsOrigine].y)/(1+sqr(pente)));
             end;
          end;
        end;
        else dimPix := 1;
   end;
   case axeI of
      aX,apX : begin
           dimPix := dimPix*invCosA;
      end;
      aY,apY : begin
           dimPix := dimPix*invCosA;
      end;
   end;
end; // chercheEchelle

Procedure cherche(decale : integer); // sur l'image grce au stretch
var i : integer;
    xi,yi : single;
    valeur : TAlphaColorRec;
    rr,gg,bb : integer;
    x0,y0 : single;
begin
     case axeI of
       aX,apX : begin
            x0 := origineX;
            y0 := origineY+decale;
       end;
       aY,apY : begin
            x0 := origineX+decale;
            y0 := origineY;
       end
       else begin // pour le compilateur
            x0 := origineX;
            y0 := origineY;
       end;
     end;
     yi := y0;
     xi := x0;
end; // cherche

procedure getEchelle;
begin
     case echelleRG.itemIndex of
          0 : begin // echelle
     try
    //     OrigineEchelle := StrToFloat(OrigineEdit.text)*coeffUnite;
     except
         showMessage('Erreur chelle');
         origineEchelle := 0;
     //    OrigineEdit.text := '0';
     end;
          end;
       1 : origineEchelle := 0; // pixel
       else begin // index
            origineEchelle := 0;
            longueurRef := 1;
            coeffUnite := 1;
            exit;
       end;
     end;
     try
     longueurRef := StrToFloat(EchelleEdit.text)*coeffUnite;
     if echelleRG.itemIndex=0 then LongueurRef := longueurRef-origineEchelle;
     except
         showMessage('Erreur chelle');
         EchelleEdit.Text := '10';
         longueurRef := coeffUnite;
     end;
     setEchelle;
end; // getEchelle

var i,Nbre : integer;
begin // CalculIntensite
     if nomFichier='' then exit;
     panel2.cursor := crHourGlass;
     traceBorne; // efface
     getEchelle;
     for i := -maxVecteurOptique to maxVecteurOptique do begin
     end;
     cherchePente;
     chercheEchelle;
     cherche(0);
     traceBorne; // retrace les bornes
     panel2.cursor := crDefault;
end; // CalculIntensite

Procedure TOptiqueBitmapForm.TraceBorne;
var imageCanvas : TCanvas;

Procedure TraceCroix(bsi : TstyleDragOptique);
begin with PointRepere[bsi],imageCanvas do begin
       if bsi=bsEchelle1
          then drawLine(TpointF.Create(x-MagnetInterf,y),TpointF.Create(x,y),1)
          else drawLine(TpointF.Create(x-MagnetInterf,y),TpointF.Create(x+MagnetInterf,y),1);
       drawLine(TpointF.Create(x,y-MagnetInterf),TpointF.Create(x,y+MagnetInterf),1);
end end;

Procedure Marque(bsi : TstyleDragOptique);
var dX,dY : single;
begin with PointRepere[bsi],imageCanvas do begin
       case axeE of
            aX,apX: begin
               if y<hauteurI div 2
                  then dY := +magnetInterf
                  else dY := -magnetInterf-labelO.Height;
               dX := -labelO.Width/2;
            end;
            aY,apY: begin
              if x<largeurI div 2
                then dX := magnetInterf
                else dX := -magnetInterf-labelO.Width;
              dY := -labelO.Height/2;
            end
            else begin
                dX := 0;
                dY := 0;
            end
       end;
       case bsi of
            bsNone:;
            bsOrigine:;
            bsDirection:;
            bsEchelle1: begin
               labelO.position.x := x/pasZoom+dx;
               labelO.position.y := y/pasZoom+dy;
            end;
            bsEchelle2: begin
               labelA.position.x := x/pasZoom+dx;
               labelA.position.y := y/pasZoom+dy;
            end;
       end;
end end; // Marque

Procedure TraceEchelle;
var deltaX,deltaY : single;

Procedure TraceTick(i : integer);
var Kp : integer;
    Xa,Ya,Xb,Yb : integer;
    xx,yy : integer;
begin
      xx := round(PointRepere[bsEchelle1].x+deltaX*i);
      yy := round(PointRepere[bsEchelle1].y+deltaY*i);
      if abs(deltaX)>abs(deltaY)
         then begin
              Ya := yy+tickEchelle;
              Yb := yy-tickEchelle;
              Kp := round(-tickEchelle*deltaY/deltaX);
              Xa := xx+Kp;
              Xb := xx-Kp;
         end
         else begin
              Xa := xx+tickEchelle;
              Xb := xx-tickEchelle;
              Kp := round(-tickEchelle*deltaX/deltaY);
              Ya := yy+Kp;
              Yb := yy-Kp;
         end;
    imageCanvas.drawLine(TpointF.Create(Xa,Ya),TpointF.Create(Xb,Yb),1);
end;// traceTick

var i : integer;
begin // traceEchelle
          TraceCroix(bsEchelle1);
          Marque(bsEchelle1);
          Marque(bsEchelle2);
          traceFleche(pointRepere[bsEchelle1],pointRepere[bsEchelle2]);
          deltaX := (PointRepere[bsEchelle2].x-PointRepere[bsEchelle1].x)/10;
          deltaY := (PointRepere[bsEchelle2].y-PointRepere[bsEchelle1].y)/10;
          with imageCanvas do begin
              if pasZoom>1
                 then stroke.thickness := pasZoom
                 else stroke.thickness := 1/pasZoom;
              drawLine(PointRepere[bsEchelle1],PointRepere[bsEchelle2],1);
          end;
          for i := 1 to 9 do TraceTick(i);
end; // traceEchelle

var p : single;
    xdroit,ydroit,xgauche,ygauche : single;
    decaleX,decaleY : integer;
begin // traceBorne
imageCanvas := Image.bitmap.Canvas;
if nomFichier<>'' then with imageCanvas do begin
       stroke.color := couleurTrait;
       stroke.thickness := round(largeurTrait*pasZoom);
       stroke.kind := TbrushKind.none;
       labelA.Visible := echelleRG.itemIndex=0;
       labelO.Visible := echelleRG.itemIndex=0;
       angleLabel.Visible := mesure in [mAngle,mDistance];
       ValeurMesureGB.Visible := mesure in [mAngle,mDistance];
       case axeI of
          aY : begin
               xdroit := PointRepere[bsOrigine].x;
               xgauche := xdroit;
               ydroit := 0;
               ygauche := hauteurI;
          end;
          aX : begin
               ydroit := PointRepere[bsOrigine].y;
               ygauche := ydroit;
               xdroit := 0;
               xgauche := largeurI;
          end;
          apX,apY : begin
               p := (PointRepere[bsOrigine].y-PointRepere[bsDirection].y)/
                    (PointRepere[bsOrigine].x-PointRepere[bsDirection].x);
               // intersection xmax
               xdroit := largeurI;
               ydroit := round(p*(xdroit-PointRepere[bsOrigine].x)+PointRepere[bsOrigine].y);
               if (pente>0) and (ydroit>hauteurI) then  begin
                   ydroit := hauteurI;
                   xdroit := round((ydroit-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
               end;
               if (pente<0) and (ydroit<0) then  begin
                   xdroit := round((0-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
                   ydroit := 0;
               end;
               // intersection xmin
               xgauche := 0;
               ygauche := round(p*(0-PointRepere[bsOrigine].x)+PointRepere[bsOrigine].y);
               if (pente<0) and (ygauche>hauteurI) then  begin
                  ygauche := hauteurI;
                  xgauche := round((ygauche-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
               end;
               if (pente>0) and (ygauche<0) then  begin
                  xgauche := round((0-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
                  ygauche := 0;
               end;
            end
            else begin
               ydroit := hauteurI;
               ygauche := 0;
               xdroit := 0;
               xgauche := largeurI;
            end;
       end;
       stroke.thickness := largeurTrait*pasZoom;
       case mesure of
            mAngle : traceAngle;
            mDistance : traceDistance;
       end;
       if echelleRG.itemIndex=0 then traceEchelle;
end end;  // traceBorne

function TOptiqueBitmapForm.RGBToL(couleur : TAlphaColorRec) : double;
begin
end;

procedure TOptiqueBitmapForm.OnApplicationHint(Sender: TObject);
begin
   label1.Text := Application.Hint;
end;

end.

