unit ArduinoGraphe;

interface

uses
  System.SysUtils,
  System.Types,
  System.UITypes,
  System.Classes,
  System.Rtti,
  System.math.vectors,
  System.UIConsts,

  FMX.Types,
  FMX.Controls,
  FMX.Forms,
  FMX.Dialogs,
  FMX.Objects,
  FMX.Menus,
  FMX.Graphics,
  FMX.ExtCtrls,
  FMX.StdCtrls,
  FMX.ListBox,
  FMX.Memo,
  FMX.TabControl,
  FMX.Layouts,
  FMX.TextLayout,
  FMX.Edit,
  FMX.Platform,
  FMX.Controls.Presentation,

  FMX.TMSBaseControl,
  FMX.TMSGridOptions,
  FMX.TMSGridData,
  FMX.TMSGrid,
  FMX.TMSGridCell,
  FMX.TMSCustomGrid,
  FMX.TMSEdit,

  maths,
  math,
  regutil;

const
        MaxVecteurArduino = 8192;
        ordreMaxSinc = 16;
        IntSansSignif = -1;
        CouleurFond = claWhite;
        mondeX = 0;
        mondeY = 1;
        mondeYmax = 6;

type
        IndiceVecteurArduino = 0..MaxVecteurArduino;
        TvecteurArduino = array[indiceVecteurArduino] of double;
        indiceMondeArduino = mondeX..mondeYMax;

TmondeArduino = class
    FminiStr,FmaxiStr : string;
    public
    mini,maxi : double;
    Valeur,ValeurNew,ValeurSinc : TvecteurArduino;
    a,b : double; // entier = rel*a+b
    nom,unite : string;
    deltaAxe : double;
    NbreChiffres : integer;
    exposant : double;
    PointDebut : double;
    valeurC : double;
    c : coefUnite;
    property miniStr : string read FminiStr write FminiStr;
    property maxiStr : string read FmaxiStr write FMaxiStr;
    function getTitreOscillo : string;
    function getTitre : string;
    constructor create;
    Procedure GetMinMax(Npoints,NpointsNew : integer;avecCourant : boolean);
    function formatMonde(x : double) : string;
    procedure setDelta;
    function RealToWindow(xr : double) : single;
end;

const
    couleurArduino :  array[indiceMondeArduino] of TAlphaColor =
        (claBlack,claBlue,claRed,claPurple,claGreen,claTeal,claFuchsia);
type

   TgrapheArduino = class
      FNbrePoints : integer;
      FNbreSinc : integer;
      FCanvas : Tcanvas;
      procedure setNbrePoints(aNbre : integer);
      public
      monde : array[indiceMondeArduino] of TmondeArduino;
      mondeMax : integer;
      NbrePointsNew : integer;
      limiteCourbe : TrectF;
      indexRoll : integer;
      modePoint : boolean;
      deltatRoll : double;
      layout: TTextLayout;
      ordreSinc : integer;
      avecSinc : boolean;
      isOscillo : boolean;
      property NbrePoints : integer read FNbrePoints write setNbrePoints;
      property NbreSinc : integer read FNbreSinc;
      Constructor Create(aCanvas : TCanvas);
      Procedure DrawG;
      Procedure DrawAxis;
      procedure DrawText(P : tpointF;S : string);
      procedure InterpoleSinc(m : indiceMondeArduino);
      destructor Destroy;override;
   end;

var magnet : integer;

IMPLEMENTATION

(*
procedure Fleche(canvas : Tcanvas;P : TPointF;angle : single);

function PointRel(P : TPointF;angle : single;larg,haut : integer) : TpointF;
var dx,dy : single;
begin
     result := P;
     dx := larg*cos(angle)+haut*sin(angle);
     dy := -larg*sin(angle)+haut*cos(angle);
     result.offset(dx,dy);
end;

var aPoly : TPolygon;
begin
    setLength(aPoly,3);
    aPoly[0] := PointRel(P,angle,2*magnet,magnet);
    aPoly[1] := P;
    aPoly[2] := PointRel(P,angle,2*magnet,-magnet);
    canvas.stroke.kind := TBrushKind.Solid;
    canvas.fill.Color := claBlack;
    canvas.DrawPolygon(aPoly,1);
    canvas.FillPolygon(aPoly,1);
    Finalize(aPoly);
end;
*)

constructor TmondeArduino.Create;
begin
    inherited create;
    A := 1;
    B := 0;
    Maxi := 0;
    Mini := 0;
    FmaxiStr := '';
    FminiStr := '';
end;

function TmondeArduino.getTitreOscillo : string;
var Lunite : string;
    Lc : coefUnite;
begin
    if unite='s'
    then begin
       Lc := c;
       dec(Lc,2);
       if Lc=nulle
          then Lunite := unite
          else Lunite := prefixeUnite[Lc]+unite;
       result := nom+'/'+Lunite
    end
    else result := nom+'/1000' // donnes binaires
end;

function TmondeArduino.getTitre : string;
var Lunite : string;
begin
    if unite<>''
    then begin
       if c=nulle
          then Lunite := unite
          else Lunite := prefixeUnite[c]+unite;
       if pos('/',unite)>0
         then result := nom+' '+Lunite
         else result := nom+'/'+Lunite
    end
    else result := nom;
end;

Procedure TgrapheArduino.setNbrePoints(aNbre : integer);
var m : indiceMondeArduino;
begin
    FNbrePoints := aNbre;
    if aNbre=0 then begin
    for m := mondeX to mondeYmax do
    with monde[m] do begin
       mini := 0;
       maxi := 0;
    end;
    indexRoll := 0;
    end;
end;


Function TmondeArduino.RealToWindow(xr : double) : single;
begin
    result := A*xr+B
end;

destructor TgrapheArduino.destroy;
var m : indiceMondeArduino;
begin
    for m := mondeX to mondeYmax do
        monde[m].free;
    layout.Free;
end;

procedure TgrapheArduino.InterpoleSinc(m : indiceMondeArduino);
Const a = 7; // sept points  droite et  gauche soit 15 points
      NijMax = (ordreMaxSinc+1)*(2*a+1); // soit 255
var
   Nij : array[0..NijMax] of double;
   i,j,k : integer;
   imin,imax : integer;
   yy,deltaX,w : double;
   X,Y : TvecteurArduino;
   Xsinc,Ysinc : TvecteurArduino;
   m1 : indiceMondeArduino;
begin
    FNbreSinc := FNbrePoints*ordreSinc;
    if FNbreSinc>maxVecteurArduino then begin
       ordreSinc := ceil(maxVecteurArduino/FNbrePoints);
       FNbreSinc := FNbrePoints*ordreSinc;
    end;
    if ordreSinc=1 then begin
       avecSinc := false;
       for m1 := mondeX to mondeYmax do
           for i := 0 to maxVecteurArduino do
               monde[m1].valeurSinc[i] := monde[m1].valeurSinc[i];
       exit;
    end;
    Nij[0] := 1; // pour viter div par 0
		for j := 1 to NijMax do begin // calcul du noyau
        yy := pi*j/ordreSinc;
        Nij[j] := sin(yy)*sin(yy/a)*a/sqr(yy);
    end;
    X := monde[mondeX].valeur;
    Y := monde[m].valeur;
    deltaX := X[1]/ordreSinc;
// calcul de Xsinc, Ysinc
    for j := 0 to FNbreSinc-1 do begin
        k := round(j/ordreSinc);
        Xsinc[j] := j*deltaX;
        imin := k-a;
        if imin<0 then imin := 0;
        imax := imin+2*a; // 2*a+1 points
        if imax>=FNbrePoints then begin
           imax := FNbrePoints-1;
           imin := imax-2*a;
           if imin<0 then imin := 0;
        end;
        yy := 0;
        w := 0;
        for i := imin to imax do begin
            yy := yy + Y[i]*Nij[abs(i*ordreSinc-j)];
            w := w + Nij[abs(i*ordreSinc-j)];
        end;
	   	  Ysinc[j] := yy/w;
    end;
    monde[mondeX].valeurSinc  := Xsinc;
    monde[m].valeurSinc := Ysinc;
end; // InterpoleSinc

constructor TgrapheArduino.Create;
var m : indiceMondeArduino;
begin
   inherited create;
   for m := mondeX to mondeYMax do
       monde[m] := TmondeArduino.create;
   mondeMax := mondeY;
   FNbrePoints := 0;
   NbrePointsNew := 0;
   indexRoll := 0;
   layout := TTextLayoutManager.DefaultTextLayout.Create;
   FCanvas := aCanvas;
   layout.layoutCanvas := Fcanvas;
   layout.Color := claBlack;
end;

Procedure TmondeArduino.GetMinMax(Npoints,NpointsNew : integer;avecCourant : boolean);
var i : integer;
    marge,margeM,margeP : double;
    miniImpose : boolean;
begin
     mini := strToFloatWin(FminiStr);
     if isNan(mini) then begin
        if Npoints>0 then Mini := Valeur[0] else mini := 0;
        miniImpose := false;
     end
     else miniImpose := true;
     maxi := strToFloatWin(FmaxiStr);
     if isNan(maxi) then if (Npoints>0) then Maxi := Valeur[0] else maxi := 1;
     verifMinMaxReal(mini,maxi);
     for i := 1 to pred(Npoints) do
         if Valeur[i]>Maxi
            then Maxi := Valeur[i]
            else if Valeur[i]<Mini
               then Mini := Valeur[i];
     if NpointsNew>3 then
     for i := 1 to pred(NpointsNew) do
         if ValeurNew[i]>Maxi
            then Maxi := ValeurNew[i]
            else if ValeurNew[i]<Mini
               then Mini := ValeurNew[i];
     if avecCourant then
        if ValeurC>Maxi
            then Maxi := ValeurC
            else if ValeurC<Mini
               then Mini := ValeurC;
     if not miniImpose then begin
        margeM := (maxi - mini)/16;
        margeP := abs(maxi + mini)/16;
        if margeM>margeP then marge := margeM else marge := margeP;
        maxi := maxi + marge;
        if (mini>=0) and (mini<marge)
           then mini := 0
           else mini := mini - marge
     end;
     if maxi=mini then begin
        mini := 0;
        if maxi=0 then maxi := 1;
     end;
end;

procedure TgrapheArduino.DrawG;

procedure ChercheMinMax;
var m : indiceMondeArduino;
begin
    for m := mondeX to mondeMax do
        monde[m].GetMinMax(FNbrePoints,nbrePointsNew,modePoint);
    for m := mondeY to mondeMax do with monde[m] do begin
        A := (LimiteCourbe.top-LimiteCourbe.bottom)/(Maxi-Mini);
        B := LimiteCourbe.bottom-Mini*A;
    end;
    with monde[mondeX] do begin
        A := (LimiteCourbe.right-LimiteCourbe.left)/(Maxi-Mini);
        B := LimiteCourbe.left-Mini*A;
    end;
    if modePoint and (mondeMax>mondeY) then // mondeY = abscisse
    with monde[mondeY] do begin
        A := (LimiteCourbe.right-LimiteCourbe.left)/(Maxi-Mini);
        B := LimiteCourbe.left-Mini*A;
    end;
    DrawAxis;
end;// ChercheMinMax

procedure TraceCouleur;
var x,y : single;
    i : integer;
    m : indiceMondeArduino;
    Fpath : TPathData;
begin
  for m := mondeY to mondeMax do begin
      FPath := TPathData.Create;
      with monde[mondeX] do x := RealToWindow(valeur[0]);
      with monde[m] do y := RealToWindow(valeur[0]);
      FCanvas.stroke.color := couleurArduino[m];
      FPath.moveTo(TpointF.Create(x,y));
      for i := 1 to pred(FnbrePoints) do begin
           with monde[mondeX] do x := RealToWindow(valeur[i]);
           with monde[m] do y := RealToWindow(valeur[i]);
           FPath.LineTo(TpointF.Create(x,y));
      end;
      FCanvas.DrawPath(FPath, 1);
      FPath.Free;
  end;
end;// TraceCouleur

procedure TraceSinc;
var x,y : single;
    i : integer;
    m : indiceMondeArduino;
    Fpath : TPathData;
begin
  for m := mondeY to mondeMax do begin
      InterpoleSinc(m);
      FPath := TPathData.Create;
      with monde[mondeX] do x := RealToWindow(valeurSinc[0]);
      with monde[m] do y := RealToWindow(valeurSinc[0]);
      FCanvas.stroke.color := couleurArduino[m];
      FPath.moveTo(TpointF.Create(x,y));
      for i := 1 to pred(FnbreSinc) do begin
           with monde[mondeX] do x := RealToWindow(valeurSinc[i]);
           with monde[m] do y := RealToWindow(valeurSinc[i]);
           FPath.LineTo(TpointF.Create(x,y));
      end;
      FCanvas.DrawPath(FPath, 1);
      FPath.Free;
  end;
end;// TraceSinc

procedure TraceCouleurRoll;
var x,y : single;
    i : integer;
    m : indiceMondeArduino;
    t : double;
    Fpath : TPathData;
begin
  for m := mondeY to mondeMax do begin
      t := 0;
      Fpath := TPathData.create;
      with monde[mondeX] do x := RealToWindow(0);
      with monde[m] do y := RealToWindow(valeur[indexRoll]);
      FCanvas.stroke.color := couleurArduino[m];
      FPath.moveTo(TpointF.Create(x,y));
      for i := indexRoll+1 to pred(FnbrePoints) do begin
           t := t+deltatRoll;
           with monde[mondeX] do x := RealToWindow(t);
           with monde[m] do y := RealToWindow(valeur[i]);
           FPath.LineTo(TpointF.Create(x,y) );
      end;
      for i := 0 to indexRoll-1 do begin
           t := t+deltatRoll;
           with monde[mondeX] do x := RealToWindow(t);
           with monde[m] do y := RealToWindow(valeur[i]);
           FPath.LineTo(TpointF.Create(x,y));
      end;
      FCanvas.DrawPath(FPath, 1);
      FPath.Free;
  end;
end;// TraceCouleurRoll

procedure TraceCouleurRollTemps;
var x,y : single;
    i : integer;
    m : indiceMondeArduino;
    tZero : double;
    Fpath : TPathData;
begin
  for m := mondeY to mondeMax do begin
      Fpath := TPathData.create;
      with monde[mondeX] do begin
         tZero := valeur[indexRoll];
         x := RealToWindow(0);
      end;
      with monde[m] do y := RealToWindow(valeur[indexRoll]);
      FCanvas.stroke.color := couleurArduino[m];
      Fpath.moveTo(TpointF.Create(x,y));
      for i := indexRoll+1 to pred(FnbrePoints) do begin
           with monde[mondeX] do x := RealToWindow(valeur[i]-tZero);
           with monde[m] do y := RealToWindow(valeur[i]);
           FPath.LineTo(TpointF.Create(x,y));
      end;
      for i := 0 to indexRoll-1 do begin
           with monde[mondeX] do x := RealToWindow(valeur[i]-tZero);
           with monde[m] do y := RealToWindow(valeur[i]);
           FPath.LineTo(TpointF.Create(x,y));
      end;
      FCanvas.DrawPath(FPath, 1);
      FPath.Free;
  end;
end;// TraceCouleurRollTemps


procedure TraceCouleurPoint;
var x,y : single;
    i : integer;
    m : indiceMondeArduino;
    taille : integer;
begin
  taille := magnet div 3;
  if mondeMax=mondeY then begin // y=f(index)
      FCanvas.stroke.color := couleurArduino[mondeY];
      for i := 0 to pred(FnbrePoints) do begin
           with monde[mondeX] do x := RealToWindow(valeur[i]);
           with monde[mondeY] do y := RealToWindow(valeur[i]);
           FCanvas.DrawEllipse(TrectF.Create(x-taille,y-taille,x+taille,y+taille),1);
      end;
  end
  else begin
  for m := mondeY+1 to mondeMax do begin // Y2=f(Y1)
      FCanvas.stroke.color := couleurArduino[m];
      for i := 0 to pred(FnbrePoints) do begin
          with monde[mondeY] do x := RealToWindow(valeur[i]);
          with monde[m] do y := RealToWindow(valeur[i]);
          FCanvas.DrawEllipse(TrectF.Create(x-taille,y-taille,x+taille,y+taille),1);
      end;
  end;
  end;
end;// TraceCouleurPoint

procedure TraceCourant;
var x,y : single;
    m : indiceMondeArduino;
begin
if mondeMax=mondeY then begin
    with monde[mondeX] do x := RealToWindow(valeurC);
    with monde[mondeY] do y := RealToWindow(valeurC);
    FCanvas.stroke.color := couleurArduino[mondeY];
    FCanvas.fill.color := couleurArduino[mondeY];
    FCanvas.FillEllipse(trectF.create(x-magnet,y-magnet,x+magnet,y+magnet),1);
end
else begin
  for m := mondeY+1 to mondeMax do begin
      with monde[mondeY] do x := RealToWindow(valeurC);
      with monde[m] do y := RealToWindow(valeurC);
      FCanvas.stroke.color := couleurArduino[m];
      FCanvas.fill.color := couleurArduino[m];
      FCanvas.FillEllipse(trectF.create(x-magnet,y-magnet,x+magnet,y+magnet),1);
  end;
end;
end;// TraceCourant

procedure TraceGris;
var x,y : single;
    i : integer;
    m : indiceMondeArduino;
    Fpath : TPathData;
begin
      FCanvas.stroke.color := claLightGray;
      for m := mondeY to mondeMax do begin
         Fpath := TPathData.create;
         with monde[mondeX] do x := RealToWindow(valeurNew[0]);
         with monde[m] do y := RealToWindow(valeurNew[0]);
         FPath.moveTo(TpointF.Create(x,y));
         for i := 0 to pred(nbrePointsNew) do begin
           with monde[mondeX] do x := RealToWindow(valeurNew[i]);
           with monde[m] do y := RealToWindow(valeurNew[i]);
           FPath.LineTo(TpointF.Create(x,y));
         end;
         FCanvas.DrawPath(FPath, 1);
         FPath.Free;
      end;
end;// TraceGris

begin
     if mondeMax>mondeYmax then mondeMax := mondeYmax;
     if mondeMax<=0 then exit;
     with FCanvas do begin
         stroke.kind := TbrushKind.Solid;
         fill.color := couleurFond;
         stroke.thickness := 1;
         beginScene;
         FillRect(LimiteCourbe,0,0,AllCorners,1);
         if isOscillo
            then limiteCourbe.Left := limiteCourbe.Left+4
            else limiteCourbe.Left := limiteCourbe.Left+48;
         limiteCourbe.Bottom := limiteCourbe.Bottom-24;
         limiteCourbe.Top:= limiteCourbe.Top+8;
     end;
     ChercheMinMax;
     if modePoint then begin
         TraceCouleurPoint;
         TraceCourant;
     end
     else if FnbrePoints>8 then begin
        if avecSinc
           then traceSinc
           else if indexRoll=0
              then TraceCouleur
              else if deltatRoll=0
                 then TraceCouleurRollTemps
                 else TraceCouleurRoll;
        if nbrePointsNew>8 then TraceGris;
     end;
     FCanvas.EndScene;
end; // tgrapheArduino.drawG

procedure TGrapheArduino.DrawAxis;
const quadrillage = false;
var yAxeX,xAxeY : single;
    penWidth : single;
    Fpath : TPathData;
    xmax : single;

procedure segment(x1,y1,x2,y2 : single);
begin
    FPath.MoveTo(TpointF.Create(x1,y1));
    FPath.LineTo(TpointF.Create(x2,y2));
end;

Procedure graduationX(x : single);
begin
  if quadrillage
       then segment(x,0,x,yAxeX)
       else segment(x,yAxex,x,yAxeX-magnet)
end;

Procedure graduationY(y : single);
begin
   if quadrillage
       then segment(xAxeY,y,xmax,y)
       else segment(xAxeY,y,xAxeY+magnet,y)
end;

  procedure traceAxeX(m : indicemondeArduino);
  var xs : single;
      PointCourant : double;
      xTitre : single;
      titreX : string;
      demiLargeur : single;
  begin with monde[m] do begin
    setDelta;
    PointCourant := PointDebut;
    if isOscillo then titreX := getTitreOscillo else titreX := getTitre;
    xTitre := xmax-Fcanvas.TextWidth(titreX)-16;
    DrawText(TpointF.Create(xTitre,yaxeX+4),titreX);
    segment(0,yAxeX,xmax,yAxeX);
    segment(xmax-magnet*2,yAxeX+magnet div 2,xmax,yAxeX);
    segment(xmax-magnet*2,yAxeX-magnet div 2,xmax,yAxeX);
    repeat
          xs := realToWindow(PointCourant);
          titreX := formatMonde(pointCourant);
          demiLargeur := Fcanvas.TextWidth(titreX)/2;
          xs := xs-demiLargeur;
          if (xs<(xTitre-demiLargeur)) and (titreX<>'0') then begin
             DrawText(TPointF.Create(xs,yAxeX+4),titreX);
             graduationX(xs);
          end;
          PointCourant := PointCourant+deltaAxe;
    until PointCourant>=maxi;
  end end; // traceAxeX

  procedure traceAxeY(m : indiceMondeArduino);
  var ys,deltay : single;
      PointCourant : double;
      grad : string;
      titreY : string;
  begin with monde[m] do begin
      setDelta;
      deltay := FCanvas.TextHeight('A')/2;
      if isOscillo then titreY := getTitreOscillo else titreY := getTitre;
      DrawText(TpointF.Create(xAxeY+magnet,4),titreY);
      PointCourant := PointDebut;
      repeat
           ys := RealToWindow(PointCourant);
           grad := formatMonde(pointCourant);
           if (ys<yAxeX) and (grad<>'0') then begin
              drawText(TpointF.Create(4,ys-deltaY),grad);
              graduationY(ys);
           end;
           pointCourant := pointCourant+deltaAxe;
      until pointCourant>=maxi;
      segment(xAxeY,yAxeX,xAxeY,0);
      segment(xAxeY-magnet div 2,2*magnet,xAxeY,0);
      segment(xAxeY+magnet div 2,2*magnet,xAxeY,0);
  end end; // traceAxeY

begin // DrawAxis
     Fpath := TPathData.create;
     penWidth := limiteCourbe.width/1024;
     if penWidth<1 then penWidth := 1;
     FCanvas.stroke.Dash := TStrokeDash.Solid;
     FCanvas.stroke.thickness := penWidth;
     FCanvas.Font.size := 10;
     FCanvas.stroke.Color := claBlack;
     xAxeY := limiteCourbe.left;
     yAxeX := limiteCourbe.bottom;
     xmax := limiteCourbe.right;
     if modePoint and (mondeMax>mondeY) then begin
       traceAxeX(mondeY);
       traceAxeY(mondeY+1);
     end
     else begin
       traceAxeX(mondeX);
       if not isOscillo then traceAxeY(mondeY);
     end;
     FCanvas.DrawPath(FPath, 1);
     FPath.Free;
end; // DrawAxis

procedure TmondeArduino.SetDelta;
// Exposant=puissance de 10 multiple de 3 de manire  avoir entre
// NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
// DeltaAxe est l'intervalle entre les graduations qui commencent  PointDebut
const
  indexMax = 5;
  nbreGradMin = 5;
  coeff: array[0..indexMax] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
  AbsMax, AbsMin, largeur : double;
  Exponent, N, index : integer;
begin // setDelta
  Exposant := 1;
  AbsMax := abs(maxi);
  AbsMin := abs(mini);
  if AbsMax < AbsMin then  AbsMax := AbsMin;
  Exponent := floor(Log10(Maxi - Mini));
  largeur  := (maxi - mini) * dix(-exponent);
  index := 0;
  while ((largeur / coeff[index]) >= NbreGradMin) and (index < indexMax) do
        Inc(index);
  DeltaAxe  := coeff[index] * dix(exponent);
  NbreChiffres := ceil(Log10(AbsMax)) - floor(Log10(DeltaAxe));
  if exponent = -1 then exponent := 0;
// nombre <~1 crit sous forme 0.8 plutt que 800.10-3
  Index := Exponent div 3;
  if ((Exponent mod 3) < 0) then Dec(index);
//la division euclidienne des informaticiens <> div des maths !
  Exposant := power(10,3 * Index);
  c := CoefUnite(index+ord(nulle));
  N := ceil(Log10(AbsMax / Exposant));
  if N > NbreChiffres then NbreChiffres := N;
  if NbreChiffres < 3 then NbreChiffres := 3;
  PointDebut := ceil(mini / deltaAxe) * deltaAxe;
end; // setDelta

function TmondeArduino.formatMonde(x : double) : string;
begin
   result := FloatToStrF(x / exposant, ffGeneral, NbreChiffres, 0)
end;

procedure TGrapheArduino.DrawText(P : tpointF;S : string);
var L,H : single;
begin
   L := FCanvas.TextWidth(S)+16;
   H := FCanvas.TextHeight(S)+8;
   layout.BeginUpdate;
   layout.TopLeft := P;
   layout.MaxSize := TPointF.Create(L, H);
   layout.Text := S;
   layout.EndUpdate;
   layout.RenderLayout(FCanvas);
end;

initialization
   magnet := round(screen.Width) div 256;
   if magnet<5 then magnet := 5;
end.

