unit ArduinoUMac;

Interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  system.inifiles, system.dateUtils, system.UIConsts, system.SyncObjs,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Objects, FMX.StdCtrls, FMX.Controls.Presentation, FMX.Edit,
  FMX.TMSBaseControl, FMX.TMSGridCell,
  FMX.TMSGridOptions, FMX.TMSGridData, FMX.TMSCustomGrid, FMX.TMSGrid,
  FMX.TMSToolBar, FMX.TMSBitmapContainer,
  ArduinoGraphe
   {$IFDEF MacOS}
  ,Winsoft.OSX.ComPort
   {$ENDIF}
  ;

const NbreBoutons = 3;
type
  TModeTemporel = (mDeclenche,mRelaxe,mRoll);
  TTrigger = (tMontant,tDescendant,tClavier);
  tTerminateur = (tNone,tCR,tLF,tCRLF);
  TArduinoForm = class(TForm)
    TMSFMXToolBar1: TTMSFMXToolBar;
    ToolsBtn: TTMSFMXToolBarButton;
    StartBtn: TTMSFMXToolBarButton;
    TMSFMXToolBarSeparator1: TTMSFMXToolBarSeparator;
    StopBtn: TTMSFMXToolBarButton;
    RazBtn: TTMSFMXToolBarButton;
    RegressiBtn: TTMSFMXToolBarButton;
    TMSFMXToolBarSeparator2: TTMSFMXToolBarSeparator;
    Button6: TTMSFMXToolBarButton;
    CommandEdit: TEdit;
    Button7: TTMSFMXToolBarButton;
    PanelVoies: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    grid: TTMSFMXGrid;
    PaintBox: TPaintBox;
    TimerGraphe: TTimer;
    Button8: TTMSFMXToolBarButton;
    Button9: TTMSFMXToolBarButton;
    TMSFMXBitmapContainer1: TTMSFMXBitmapContainer;
    TimerAcq: TTimer;
    triggerTB: TTrackBar;
    ErreurCaracLabel: TLabel;
    TimerCarac: TTimer;
    Label6: TLabel;
    Label7: TLabel;
    TMSFMXToolBarSeparator3: TTMSFMXToolBarSeparator;
    procedure PaintBoxPaint(Sender: TObject; Canvas: TCanvas);
    procedure FormCreate(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure TimerGrapheTimer(Sender: TObject);
    procedure StartBtnClick(Sender: TObject);
    procedure RazBtnClick(Sender: TObject);
    procedure RegressiBtnClick(Sender: TObject);
    procedure ToolsBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button6Click(Sender: TObject);
    procedure CommandEditExit(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TimerAcqTimer(Sender: TObject);
    procedure gridCellEditSetData(Sender: TObject; ACol, ARow: Integer;
      CellEditor: TFmxObject; var CellString: string);
    procedure triggerTBChange(Sender: TObject);
    procedure TimerCaracTimer(Sender: TObject);
    procedure CommandEditChange(Sender: TObject);
  private
    VoieSerie: TXComPort;
    indiceCourant : integer;
    monocoup : boolean;
    pointparpoint : boolean;
    attenteTrigger : boolean;
    voieTrigger : indiceMondeArduino;
    seuilTrigger : double;
    precedent : double;
    trigger : TTrigger;
    liste : TstringList;
    tempsActif : boolean;
    grapheA : TgrapheArduino;
    indexFichier : integer;
    premiereLigne : boolean; //  sauter car non complte par dfaut de synchro
    NbreColonnes : integer;
    tZero : TTime;
    tempsZero : double;
    PremierAppel : boolean;
    labelVoie : array[indiceMondeArduino] of Tlabel;
    SerialDevices: TstringList;
    sendCommand  : array[0..NbreBoutons] of string;
    SendBtn : array[0..NbreBoutons] of TTMSFMXToolBarButton;
    modeTemps : TmodeTemporel;
    dureeMax : single;
    startCommand,stopCommand : string;
    chaineLue : AnsiString;
    terminateur : Tterminateur;
    tempsArduino : boolean;
    procedure TriggerData(aChaine : ansiString);
    procedure getSerialDevices;
    procedure EnvoieCommande(aCommand : string);
    procedure activeVoie;
    procedure affiche;
  public
  end;

var
  ArduinoForm: TArduinoForm;

implementation

{$R *.fmx}

uses regutil, compile, regmain, graphker, valeurs, grapheVariabU, arduinoMacCfg;

const
    terminateurStr : array[tTerminateur] of String = ('',crCR,crLF,crCR+crLF);

procedure TArduinoForm.GetSerialDevices;
var SerialDevicesXcom: TArray<string>;
    lName : string;
    posCu : integer;
    i : integer;
begin
  SerialDevicesXcom := VoieSerie.EnumDevices;
  SerialDevices.clear;
  if Length(SerialDevicesXcom) > 0 then begin
     for i := 0 to pred(Length(SerialDevicesXcom)) do begin
        lName := SerialDevicesXcom[i];
        posCu := pos('dev/cu',lName);
        if posCu>0 then SerialDevices.add(lName);
     end;
  end;
end;

procedure TArduinoForm.gridCellEditSetData(Sender: TObject; ACol, ARow: Integer;
  CellEditor: TFmxObject; var CellString: string);
begin
    if Acol>0 then
    case Arow of
         0 : grapheA.monde[Acol-1].nom := cellString;
         1 : grapheA.monde[Acol-1].unite := cellString;
         2 : grapheA.monde[Acol-1].miniStr := cellString;
         3 : grapheA.monde[Acol-1].maxiStr := cellString;
         else exit;
    end;
    paintBox.Repaint;
end;

procedure TArduinoForm.FormClose(Sender: TObject; var Action: TCloseAction);
var Rini : TMemIniFile;
    j : integer;
    indexM : integer;
begin
     tempsActif := false;
     timerGraphe.Enabled := false;
     timerAcq.Enabled := false;
     VoieSerie.active := false;

     Rini := TMemIniFile.create(registreReg);
     Rini.WriteString('Arduino','Com',voieSerie.DeviceName);
     for j := 0 to NbreBoutons do begin
         Rini.WriteString('Arduino','Commande'+intToStr(j),sendCommand[j]);
         Rini.WriteString('Arduino','Caption'+intToStr(j),sendBtn[j].text);
     end;
     Rini.WriteString('Arduino','Start',startCommand);
     Rini.WriteString('Arduino','Stop',stopCommand);
     Rini.WriteInteger('Arduino','DureeMax',round(dureeMax));
     Rini.WriteInteger('Arduino','Terminateur',ord(terminateur));
     Rini.WriteInteger('Arduino','Baud',ord(voieSerie.baudRate));
     Rini.WriteInteger('Arduino','Mode',ord(modeTemps));
     Rini.WriteInteger('Arduino','Seuil',round(triggerTB.Value));
     Rini.WriteInteger('Arduino','Trigger',ord(trigger));
     Rini.WriteBool('Arduino','Monocoup',monocoup);
     Rini.WriteBool('Arduino','PointParPoint',pointParpoint);
     Rini.WriteBool('Arduino','TempsArduino',TempsArduino);
     for j := 1 to pred(grid.columnCount) do begin
     indexM := j-1;
         if grid.cells[j,0]<>'' then
            Rini.writeString('Arduino','Nom'+intToStr(indexM),grid.cells[j,0]);
         if grid.cells[j,1]<>'' then
            Rini.writeString('Arduino','Unite'+intToStr(indexM),grid.cells[j,1]);
         if grid.cells[j,2]<>'' then
            Rini.writeString('Arduino','Mini'+intToStr(indexM),grid.cells[j,2]);
         if grid.cells[j,3]<>'' then
            Rini.writeString('Arduino','Maxi'+intToStr(indexM),grid.cells[j,3]);
     end;
     Rini.UpdateFile;
     Rini.Free;
     if pages[pageCourante].nmes=0 then FRegressiMain.FileNewSimulationItemClick(Sender);
end;

procedure TArduinoForm.FormCreate(Sender: TObject);
const nomDefaut : array[indiceMondeArduino] of string  = ('t','X','Y','Z','aX','aY','aZ');
      uniteDefaut : array[indiceMondeArduino] of string  = ('s','','','','','','');
      minDefaut : array[indiceMondeArduino] of string  = ('0','0','0','0','0','0','0');
      maxDefaut : array[indiceMondeArduino] of string  = ('','1','1','1','1','1','1');
var i,j,col : integer;
    Rini : TMemIniFile;
    lName : string;
    indexM : integer;
begin
     voieSerie := TXComPort.Create(self);
     voieSerie.DataBits := db8;
     voieSerie.Parity := paNone;
     voieSerie.StopBits := sb1;
     grapheA := TgrapheArduino.create(paintBox.canvas);
     sendBtn[0] := button6;
     sendBtn[1] := button7;
     sendBtn[2] := button8;
     sendBtn[3] := button9;
     LabelVoie[0] :=  label1;
     LabelVoie[1] :=  label2;
     LabelVoie[2] :=  label3;
     LabelVoie[3] :=  label4;
     LabelVoie[4] :=  label5;
     LabelVoie[5] :=  label6;
     LabelVoie[6] :=  label7;
     for j := 0 to mondeYmax do begin
         col := j+1;
         labelVoie[j].textSettings.fontColor := couleurArduino[j];
         for i := 0 to 3 do
             grid.FontColors[Col,i] := couleurArduino[j];
     end;
     indexFichier := 0;
     SerialDevices := TStringList.create;
     getSerialdevices;
     Rini := TMemIniFile.create(registreReg);
     lName := Rini.ReadString('Arduino','Com','');
     if SerialDevices.count=0
        then showMessage('Pas de voie srie')
        else if (lName<>'') and (SerialDevices.IndexOf(lName)>=0)
             then VoieSerie.DeviceName := lName;
     VoieSerie.BaudRate := TBaudRate(Rini.ReadInteger('Arduino','Baud',ord(br9600)));
     terminateur := Tterminateur(Rini.readInteger('Arduino','Terminateur',0));
     for j := 0 to NbreBoutons do begin
         sendCommand[j] := Rini.ReadString('Arduino','Commande'+intToStr(j),'');
         sendBtn[j].text := Rini.ReadString('Arduino','Caption'+intToStr(j),'');
     end;
     for j := 1 to NbreBoutons do
         sendBtn[j].visible := sendCommand[j]<>'';
     sendBtn[0].enabled := sendCommand[0]<>'';
     TempsArduino := Rini.ReadBool('Arduino','TempsArduino',false);
     commandEdit.Text := sendCommand[0];
     startCommand := Rini.ReadString('Arduino','Start','');
     stopCommand := Rini.ReadString('Arduino','Stop','');
     dureeMax := Rini.ReadInteger('Arduino','DureeMax',15);
     TriggerTB.value := Rini.ReadInteger('Arduino','Seuil',512);
     modeTemps := TmodeTemporel(Rini.ReadInteger('Arduino','Mode',0));
     monocoup := Rini.ReadBool('Arduino','Monocoup',true);
     pointParPoint := Rini.ReadBool('Arduino','PointParPoint',true);
     trigger := Ttrigger(Rini.ReadInteger('Arduino','Trigger',0));
     voieTrigger := mondeY;
     {$IFDEF Debug}
     voieSerie.logFile := 'Arduino.log';
     {$ELSE}
     voieSerie.logFile := '';
     {$ENDIF}
     liste := TstringList.Create;
     grid.Cells[0,0] := 'Nom';
     grid.Cells[0,1] := 'Unit';
     grid.Cells[0,2] := 'Mini';
     grid.Cells[0,3] := 'Maxi';
     for j := 1 to pred(grid.columnCount) do begin
         indexM := j-1;
         grid.Cells[j,0] := Rini.readString('Arduino','Nom'+intToStr(indexM),nomDefaut[indexM]);
         grapheA.monde[indexM].nom := grid.Cells[j,0];
         grid.Cells[j,1] := Rini.readString('Arduino','Unite'+intToStr(indexM),uniteDefaut[indexM]);
         grapheA.monde[indexM].unite := grid.Cells[j,1];
         grid.Cells[j,2] := Rini.readString('Arduino','Mini'+intToStr(indexM),minDefaut[indexM]);
         grapheA.monde[indexM].miniStr := grid.Cells[j,2];
         grid.Cells[j,3] := Rini.readString('Arduino','Maxi'+intToStr(indexM),maxDefaut[indexM]);
         grapheA.monde[indexM].maxiStr := grid.Cells[j,3];
     end;
     Rini.Free;
     indiceCourant := 0;
     nbreColonnes := -1;
     tempsActif := false;
     affiche;
end;

procedure TArduinoForm.FormDestroy(Sender: TObject);
begin
  grapheA.Free;
  serialDevices.free;
end;

procedure TArduinoForm.FormShow(Sender: TObject);
begin
    activeVoie;
end;

procedure TArduinoForm.activeVoie;
begin
     try
     if not(voieSerie.active) and (VoieSerie.DeviceName<>'') then
        voieSerie.active := True;
     if not voieSerie.active then exit;
     chaineLue := '';
     voieSerie.Clear;
     startBtn.Enabled := true;
     timerAcq.Enabled := pointParpoint;
     timerGraphe.Enabled := pointParPoint;
     except
          ShowMessage('Problme voie srie');
     end;
end;

procedure TArduinoForm.affiche;
begin
     if pointParPoint then begin
        Grid.Cells[1,0] := 'index';
        Grid.Cells[1,1] := '';
     end
     else begin
        Grid.Cells[1,0] := 'temps';
        Grid.Cells[1,1] := 's';
     end;
     triggerTB.Visible := not(pointParPoint) and (modeTemps=mDeclenche) and (trigger<>tClavier);
     startBtn.Enabled := false;
     stopBtn.Enabled := false;
     timerAcq.Enabled := false;
     PanelVoies.Visible := pointParPoint;
     tempsActif := false;
     chaineLue := '';
     timerAcq.Enabled := pointParpoint;
     timerGraphe.Enabled := pointParPoint;
end;


procedure TArduinoForm.PaintBoxPaint(Sender: TObject; Canvas: TCanvas);
begin
      grapheA.mondeMax := nbreColonnes;
      if not pointParPoint then begin
         grapheA.Monde[mondex].miniStr := '0';
         grapheA.Monde[mondex].maxiStr := dureeMax.toString;
      end;
      grapheA.LimiteCourbe := paintBox.ClipRect;
      grapheA.drawG;
end;

procedure TArduinoForm.RazBtnClick(Sender: TObject);
begin
    grapheA.nbrePoints := 0;
    grapheA.nbrePointsNew := 0;
    paintBox.repaint;
end;

procedure TArduinoForm.RegressiBtnClick(Sender: TObject);

function Ajoute(nom,unite : string) : codeGrandeur;
var  v : codeGrandeur;
begin
      v := indexNom(nom);
      if v=grandeurInconnue then begin
         v := ajouteExperimentale(Nom,variable);
         grandeurs[v].NomUnite := unite;
      end;
      result := v;
end;

var i,k : integer;
    t : double;
    indexXY : array[indiceMondeArduino] of codeGrandeur;
begin
     for k := 0 to grapheA.mondeMax do begin  // noms et unites
         if grid.Cells[k+1,0]='' then grid.Cells[k+1,0] := 'x'+intToStr(k);
         indexXY[k] := ajoute(grid.Cells[k+1,0],grid.Cells[k+1,1]);
     end;
     ajoutePage;
     pages[pageCourante].nmes := grapheA.NbrePoints;
     if modeTemps=mRoll
     then if grapheA.deltatRoll>0
        then begin
           t := 0;
           for i := grapheA.indexRoll to pred(grapheA.nbrePoints) do begin // data
             pages[pageCourante].valeurVar[indexXY[0],i] := t;
             t := t + grapheA.deltatRoll;
             for k := 1 to grapheA.mondeMax do
                 pages[pageCourante].valeurVar[indexXY[k],i] := grapheA.monde[k].valeur[i];
           end;
           for i := 0 to grapheA.indexRoll-1  do begin // data
             pages[pageCourante].valeurVar[indexXY[0],i] := t;
             t := t + grapheA.deltatRoll;
             for k := 1 to grapheA.mondeMax do
                 pages[pageCourante].valeurVar[indexXY[k],i] := grapheA.monde[k].valeur[i];
           end;
        end
        else begin
            for i := grapheA.indexRoll to pred(grapheA.nbrePoints) do begin // data
                for k := 0 to grapheA.mondeMax do
                    pages[pageCourante].valeurVar[indexXY[k],i] := grapheA.monde[k].valeur[i];
            end;
            for i := 0 to grapheA.indexRoll-1  do begin // data
                for k := 0 to grapheA.mondeMax do
                   pages[pageCourante].valeurVar[indexXY[k],i] := grapheA.monde[k].valeur[i];
            end;
        end
     else begin
        for i := 0 to pred(indiceCourant) do // data
            for k := 0 to grapheA.mondeMax do
                pages[pageCourante].valeurVar[indexXY[k],i] := grapheA.monde[k].valeur[i];
     end;
     graphe.coordonnee[1].nomX := grapheA.monde[mondeX].nom;
     graphe.coordonnee[1].nomY := grapheA.monde[mondeY].nom;
     if pointParPoint then begin
          if grapheA.mondeMax>mondeY then begin
             graphe.coordonnee[1].nomX := grapheA.monde[mondeY].nom;
             graphe.coordonnee[1].nomY := grapheA.monde[mondeY+1].nom;
          end;
          graphe.coordonnee[1].Trace := [trPoint];
          graphe.coordonnee[1].couleur := couleurArduino[mondeY];
          graphe.coordonnee[1].Motif := mCroix;
     end
     else begin
          graphe.coordonnee[1].Trace := [trLigne];
          graphe.coordonnee[1].Ligne := LiDroite;
     end;
     graphe.configCharge := true;
     graphe.useDefaut := false;
     NomFichierData := '';
     ModifFichier := true;
     modeAcquisition := acqArduino;
     FRegressiMain.WMRegMaj(MajFichier);
     FRegressiMain.windowTab.ActiveTab := FRegressiMain.grapheTab;
     FValeurs.feuillets.ActiveTab := Fvaleurs.VariabSheet;
     Close;
end;

procedure TArduinoForm.StartBtnClick(Sender: TObject);
var i : integer;
begin
    timerGraphe.enabled := true;
    grapheA.indexRoll := 0;
    if pointParPoint
    then begin
       grapheA.monde[mondeX].valeurC := indiceCourant;
       for i := 0 to nbreColonnes do
           grapheA.monde[i].valeur[indiceCourant] := grapheA.monde[i].valeurC;
       inc(indiceCourant);
       startBtn.Enabled := indiceCourant<MaxvecteurArduino;
       grapheA.nbrePoints := indiceCourant;
       regressiBtn.Enabled := indiceCourant>3;
       razBtn.Enabled := indiceCourant>0;
       stopBtn.enabled := false;
       PanelVoies.Visible := true;
    end
    else begin
        indiceCourant := 0;
        grapheA.NbrePoints := 0;
        grapheA.NbrePointsNew := 0;
        premiereLigne := true;
        stopBtn.enabled := true;
        startBtn.Enabled := False;
        premierAppel := true;
        regressiBtn.Enabled := false;
        tZero := getTime;
        if startCommand<>'' then envoieCommande(startCommand);
        tempsActif := true;
        PanelVoies.Visible := false;
        voieSerie.Clear;
        if (modeTemps=mDeclenche) and (trigger<>tClavier) then begin
          attenteTrigger := true;
          PanelVoies.Visible := true;
          if trigger=tMontant
             then precedent := grapheA.monde[voieTrigger].maxi
             else precedent := grapheA.monde[voieTrigger].mini
        end;
    end;
    if voieSerie.baudRate=br9600
       then timerAcq.Interval := 20
       else timerAcq.Interval := 2;
    timerAcq.Enabled := true;
end;

procedure TArduinoForm.StopBtnClick(Sender: TObject);
begin
   if tempsActif and (stopCommand<>'') then envoieCommande(stopCommand);
   tempsActif := false;
   razBtn.Enabled := grapheA.nbrePoints>0;
   regressiBtn.Enabled := grapheA.nbrePoints>3;
   stopBtn.enabled := false;
   startBtn.Enabled := voieserie.active;
   timerGraphe.enabled := pointParPoint and voieserie.active;
   PanelVoies.Visible := timerGraphe.enabled;
   grapheA.nbrePointsNew := 0;
   timerGrapheTimer(nil);
end;

procedure TArduinoForm.TimerAcqTimer(Sender: TObject);
const imax = 32;
var carac : AnsiChar;
    i : integer;
    lectureLigne : boolean;
begin // 6 voies avec 4 carac + virgule + CR + LF = 32
    timerAcq.Enabled := false;
    i := 0;
    lectureLigne := true;
    try
        while lectureLigne and (voieSerie.InputCount>0) do begin
        carac := voieSerie.ReadAnsiChar;
        if carac>=' '
           then begin
              chaineLue := chaineLue + carac;
              inc(i);
              lectureLigne := i<imax;
              if not TimerCarac.enabled  and (ord(carac)>ord(';')) then begin
                 TimerCarac.enabled := true;
                 ErreurCaracLabel.visible := true;
              end;
           end
           else if chaineLue<>'' then begin // premier caractere de controle CR NL ...
              TriggerData(chaineLue);
              chaineLue := '';
              lectureLigne := false;
           end;
        end;
        timerAcq.Enabled := true;
    except
            voieSerie.Active := false;
            StopBtn.enabled := false;
            StartBtn.enabled := false;
            toolsBtn.enabled := true;
            ShowMessage('Vrifier la voie srie');
    end;
end;

procedure TArduinoForm.TimerCaracTimer(Sender: TObject);
begin
    timerCarac.enabled := false;
    erreurCaracLabel.visible := false;
end;

procedure TArduinoForm.TimerGrapheTimer(Sender: TObject);
var i : integer;
begin
      if pointParPoint then begin
         for i := 1 to NbreColonnes do
             labelVoie[i].text := grapheA.monde[i].nom+'='+liste[i-1]+' '+grapheA.monde[i].unite;
         labelVoie[0].text := 'index='+indiceCourant.ToString
      end
      else begin
          if tempsArduino
          then begin
             for i := 1 to NbreColonnes do
                 labelVoie[i].text := grapheA.monde[i].nom+'='+liste[i]+' '+grapheA.monde[i].unite;
          end
          else begin
             for i := 1 to NbreColonnes do
                 labelVoie[i].text := grapheA.monde[i].nom+'='+liste[i-1]+' '+grapheA.monde[i].unite;
          end;
          labelVoie[0].text := 't='+floatToStrF(grapheA.monde[mondeX].valeurC,ffFixed,5,2)+' s';
      end;
      for i := succ(NbreColonnes) to mondeYmax do
          labelVoie[i].text := '';
      grapheA.modePoint := pointParPoint;
      seuilTrigger := grapheA.monde[voieTrigger].mini+
              (triggerTB.Max-triggerTB.value)*(grapheA.monde[voieTrigger].maxi-grapheA.monde[voieTrigger].mini)/triggerTB.Max;
      paintBox.repaint;
end;

procedure TArduinoForm.ToolsBtnClick(Sender: TObject);

procedure AffecteIn;
var i : integer;
begin
   ArduinoDlg.DeviceNameCB.Items.Clear;
   for i := 0 to pred(SerialDevices.count) do
      ArduinoDlg.DeviceNameCB.Items.Add(SerialDevices[i]);
   ArduinoDlg.DeviceNameCB.ItemIndex := SerialDevices.IndexOf(VoieSerie.deviceName);
   if ArduinoDlg.DeviceNameCB.ItemIndex<0
      then ArduinoDlg.DeviceNameCB.ItemIndex := 0;
   for i := 0 to pred(ArduinoDlg.BaudCB.Items.count) do
      if ord(voieSerie.baudRate)=i+ord(br9600) then
            ArduinoDlg.baudCB.ItemIndex := i;
   if ArduinoDlg.baudCB.ItemIndex<0
      then ArduinoDlg.baudCB.ItemIndex := 0;
   if pointparPoint
      then ArduinoDlg.PointRB.isChecked := true
      else ArduinoDlg.tempsRB.IsChecked := true;
   ArduinoDlg.TempsCB.ItemIndex := ord(modeTemps);
   ArduinoDlg.TempsCB.ItemIndex := ord(modeTemps);
   ArduinoDlg.TerminateurCB.ItemIndex := ord(terminateur);
   ArduinoDlg.commande1Edit.text := sendCommand[1];
   ArduinoDlg.commande2Edit.text := sendCommand[2];
   ArduinoDlg.commande3Edit.text := sendCommand[3];
   ArduinoDlg.edit1.text := sendBtn[1].text;
   ArduinoDlg.edit2.text := sendBtn[2].text;
   ArduinoDlg.edit3.text := sendBtn[3].text;
   ArduinoDlg.dureeMaxSE.Value := dureeMax;
   ArduinoDlg.startEdit.Text := startCommand;
   ArduinoDlg.stopEdit.Text := stopCommand;
   ArduinoDlg.monocoupCB.isChecked := monocoup;
   ArduinoDlg.tempsArduinoCB.isChecked := tempsArduino;
   ArduinoDlg.TriggerCB.ItemIndex := ord(trigger);
 end;

procedure AffecteOut;
var i : integer;
begin
     sendCommand[1] := ArduinoDlg.commande1Edit.text;
     sendBtn[1].text := ArduinoDlg.edit1.text;
     sendCommand[2] := ArduinoDlg.commande2Edit.text;
     sendBtn[2].text := ArduinoDlg.edit2.text;
     sendCommand[3] := ArduinoDlg.commande3Edit.text;
     sendBtn[3].text := ArduinoDlg.edit3.text;
     for i := 1 to NbreBoutons do begin
         sendBtn[i].visible := sendCommand[i]<>'';
         if (sendBtn[i].text='')
            then sendBtn[i].text := sendCommand[i];
     end;
     pointParPoint := ArduinoDlg.pointRB.IsChecked;
     modeTemps := TModeTemporel(ArduinoDlg.TempsCB.ItemIndex);
     terminateur := Tterminateur(ArduinoDlg.TerminateurCB.ItemIndex);
     trigger := TTrigger(ArduinoDlg.TriggerCB.ItemIndex);
     voieSerie.baudRate := TbaudRate(ArduinoDlg.BaudCB.ItemIndex+ord(br9600));
     if ArduinoDlg.DeviceNameCB.itemIndex>=0 then
        VoieSerie.DeviceName := ArduinoDlg.DeviceNameCB.Items[ArduinoDlg.DeviceNameCB.itemIndex];
     dureeMax := ArduinoDlg.dureeMaxSE.Value;
     startCommand := ArduinoDlg.startEdit.Text;
     stopCommand := ArduinoDlg.stopEdit.Text;
     monocoup := ArduinoDlg.monocoupCB.isChecked;
     tempsArduino := ArduinoDlg.tempsArduinoCB.isChecked;
     affiche;
end;

begin
     tempsActif := false;
     timerGraphe.Enabled := false;
     timeracq.Enabled := false;
     VoieSerie.active := False;
     getSerialDevices;
     if SerialDevices.count=0 then begin
        showMessage('Pas de voie srie');
        exit;
     end;
     if ArduinoDlg=nil then Application.CreateForm(TArduinoDlg,ArduinoDlg);
     affecteIn;
     if ArduinoDlg.ShowModal=mrOK then affecteOut;
     activeVoie;
end;

procedure TArduinoForm.TriggerData(aChaine : ansiString);

procedure TriggerDataTemps;
var i : integer;
    m : indiceMondeArduino;
    triggerAtteint : boolean;
begin
      if premiereLigne then begin
         premiereLigne := false;
         tZero := getTime;
         exit;
      end;
      try
      if tempsArduino
         then begin
            nbreColonnes := liste.Count-1;
            if nbreColonnes>mondeYMax then nbreColonnes := mondeYMax;
            for i := 0 to nbreColonnes do
                grapheA.monde[i].valeurC := strToFloatWin(liste[i]);
            if (indiceCourant=0) then if modeTemps=mRoll
                then begin
                   if premierAppel then tempsZero := grapheA.monde[mondeX].valeurC;
                end
                else tempsZero := grapheA.monde[mondeX].valeurC;
            grapheA.monde[mondeX].valeurC := grapheA.monde[mondeX].valeurC-tempsZero;
         end
         else begin
            grapheA.monde[mondeX].valeurC := milliSecondsBetween(getTime,tZero)/1000.0;
            for i := 1 to nbreColonnes do
                grapheA.monde[i].valeurC := strToFloatWin(liste[i-1]);
         end;
      for i := 0 to nbreColonnes do begin
          if (modeTemps=mRelaxe) and not(premierAppel)
               then grapheA.monde[i].valeurNew[indiceCourant] := grapheA.monde[i].valeurC
               else grapheA.monde[i].valeur[indiceCourant] := grapheA.monde[i].valeurC;
      end;
      regressiBtn.Enabled := indiceCourant>10;
      case modeTemps of
          mDeclenche : case trigger of
              tClavier : begin
                 inc(indiceCourant);
                 grapheA.nbrePoints := indiceCourant;
              end;
              tMontant,tDescendant : if attenteTrigger
                 then begin
                      tZero := getTime;
                      if trigger=tMontant
                         then TriggerAtteint := (grapheA.monde[voieTrigger].valeurC>=seuilTrigger) and
                                                (precedent<seuilTrigger)
                         else TriggerAtteint := (grapheA.monde[voieTrigger].valeurC<=seuilTrigger) and
                                                (precedent>seuilTrigger);

                      if TriggerAtteint then begin
                         grapheA.nbrePoints := 1;
                         indiceCourant := 1;
                         attenteTrigger := false;
                      end;
                      Precedent := grapheA.monde[voieTrigger].valeurC;
                 end
                 else begin
                    inc(indiceCourant);
                    grapheA.nbrePoints := indiceCourant;
                 end;
          end;
          mRelaxe : begin
             inc(indiceCourant);
             if premierAppel
                then grapheA.nbrePoints := indiceCourant
                else grapheA.NbrePointsNew := indiceCourant;
          end;
          mRoll : begin
             inc(indiceCourant);
             if premierAppel
               then grapheA.nbrePoints := indiceCourant
               else grapheA.indexRoll := indiceCourant;
          end;
      end;
      except
      end;
      if (indiceCourant>=MaxVecteurArduino) or
         (grapheA.monde[mondeX].valeurC>dureeMax) then
      case modeTemps of
         mDeclenche : case trigger of
            tClavier : stopBtnClick(nil);
            tDescendant,tMontant : if monocoup
               then stopBtnClick(nil)
               else begin
                  indiceCourant := 0;
                  premierAppel := false;
                  grapheA.nbrePointsNew := 0;
                  tZero := getTime;
                  attenteTrigger := true;
                  if trigger=tMontant
                      then precedent := grapheA.monde[voieTrigger].maxi
                      else precedent := grapheA.monde[voieTrigger].mini
               end;
         end;
         mRelaxe : begin
             if not premierAppel then begin
                grapheA.nbrePoints := grapheA.NbrePointsNew;
                for m := mondeX to grapheA.mondeMax do
                    for i := 0 to pred(grapheA.nbrePoints) do
                        grapheA.monde[m].valeur[i] := grapheA.monde[m].valeurNew[i];
             end;
             indiceCourant := 0;
             premierAppel := false;
             grapheA.nbrePointsNew := 0;
             tZero := getTime;
             if startCommand<>'' then envoieCommande(startCommand);
         end;
         mRoll : begin
             premierAppel := false;
             grapheA.indexRoll := 0;
             tZero := getTime;
             if tempsArduino
                then grapheA.deltatRoll := 0
                else grapheA.deltatRoll := (grapheA.monde[mondeX].valeur[indiceCourant-1]-
                                            grapheA.monde[mondeX].valeur[0])/(indiceCourant-1);
             indiceCourant := 0;
         end;
      end;
end;

var i : integer;
begin
      liste.commaText := string(aChaine);
      if liste.Count<1 then exit;
      nbreColonnes := liste.Count; // temps ou index en plus
      if nbreColonnes>mondeYMax then nbreColonnes := mondeYMax;
      if pointParPoint
      then begin
         grapheA.monde[0].valeurC := indiceCourant;
         for i := 1 to nbreColonnes do
             grapheA.monde[i].valeurC := strToFloatWin(liste[i-1]);
      end
      else if TempsActif then TriggerDataTemps;
end;

procedure TArduinoForm.triggerTBChange(Sender: TObject);
begin
     seuilTrigger := grapheA.monde[voieTrigger].mini+
              (triggerTB.Max-triggerTB.value)*(grapheA.monde[voieTrigger].maxi-grapheA.monde[voieTrigger].mini)/triggerTB.Max;
                                  // le 0 est en haut !
end;

procedure TArduinoForm.Button6Click(Sender: TObject);
begin
    EnvoieCommande(sendCommand[(sender as TTMSFMXToolBarButton).tag]+terminateurStr[terminateur]);
end;

procedure TArduinoForm.CommandEditChange(Sender: TObject);
begin
    sendCommand[0] := commandEdit.Text;
    sendBtn[0].enabled := sendCommand[0]<>'';
end;

procedure TArduinoForm.CommandEditExit(Sender: TObject);
begin
    sendCommand[0] := commandEdit.Text;
    sendBtn[0].enabled := sendCommand[0]<>'';
end;

procedure TArduinoForm.EnvoieCommande(aCommand : string);
var sauveTimer : boolean;
begin
   if voieSerie.active then begin
       sauveTimer := timerAcq.Enabled;
       timerAcq.Enabled := false;
       VoieSerie.WriteAnsiString(AnsiString(aCommand));
       sleep(50);
       timerAcq.Enabled := sauveTimer;
   end;
end;

end.
