unit CaptureForm;

interface

uses
  System.SysUtils,
  System.Types,
  System.UITypes,
  System.Classes,
  System.UIConsts,
  System.Math,
  System.Math.Vectors,
  System.Generics.Collections,

  vidgrabconst,
  compile,
  regmain,
  regutil,
  images,

  FMX.Types,
  FMX.Controls,
  FMX.Forms,
  FMX.Dialogs,
  FMX.Objects,
  FMX.Media,
  FMX.StdCtrls,
  FMX.Layouts,
  FMX.ListBox,
  FMX.ExtCtrls,
  FMX.Colors,
  FMX.Edit,
  FMX.Graphics,
  FMX.Spinbox,
  FMX.Memo,
  FMX.ScrollBox,
  FMX.EditBox,
  FMX.Controls.Presentation,
  FMX.TMSBaseControl,
  FMX.TMSCustomPicker,
  FMX.TMSColorPicker;

const
     MaxVecteurOptique = 4096;
     CouleurFond = claWhite;
     crTab = #9;

type

IndiceVecteurOptique = -MaxVecteurOptique..MaxVecteurOptique;
Tetat = (eNone,eCapture,eAnalyse);
TvecteurOptique = array[indiceVecteurOPtique] of double;
Techelle = (eEchelle,ePixel,eIndex);
TstyleDrag = (bsNone,bsOrigine,bsDirection,bsEchelle1,bsEchelle2,bsAngle1,bsAngle2,bsAngle3);
TMesure = (mIntensite,mAngle,mDistance);
Taxe = (aX,aY,apX,apY);

TRepereShape = class(TShape)
 private
 protected
    procedure Paint; override;
    procedure TraceFleche(P1,P2: TpointF);
    procedure TraceAngle;
    procedure TraceDistance;
 public
   LabelA: TLabel;
   LabelAngle: TLabel;
   LabelO: TLabel;
   PointRepere : array[TstyleDrag] of TpointF; // sur l'cran
   pente : single;
   axeEchelle,axeI : Taxe;
   invCosA : single;
   BorneSelect : TstyleDrag;
   mesure : Tmesure;
   echelle : Techelle;
   longueurRef : single;
   zoomAffichage : single;
   moyenneI : integer;
   constructor Create(AOwner: TComponent); override;
   Procedure AffecteBorne(x,y : single;horizVert : boolean);
   Function GetBorne(x,y : single) : TstyleDrag;
   procedure CherchePente;
   procedure SetMoveDown(move : TMouseMoveEvent; down,up : TMouseEvent);
 end;

 Tgraphe = class
      Canvas : Tcanvas;
      miniY,maxiY : double;
      aX,bX,aY,bY : double; // entier = rel*a+b
      debutG,finG : integer;
      LimiteCourbe : TrectF;
      intensiteY,vertY,rougeY,bleuY : TvecteurOptique;
      withG,withB,withR,withL : boolean;
      Constructor Create;
      Procedure DrawG;
      Procedure WindowXY(xr,yr : double;var xi,yi : single);
   end;

  TFCapture = class(TForm)
    Layout1: TLayout;
    SaveDialog: TSaveDialog;
    PanelBoutons: TPanel;
    LissageGB: TGroupBox;
    Label1: TLabel;
    MoyenneSE: TSpinBox;
    Memo1: TMemo;
    EchelleGB: TGroupBox;
    Label2: TLabel;
    EchelleEdit: TEdit;
    EchelleEditLabel: TLabel;
    UniteCB: TComboBox;
    OrigineEdit: TEdit;
    OrigineEditLabel: TLabel;
    OrigineZeroBtn: TButton;
    GroupBox3: TGroupBox;
    EchelleRG: TComboBox;
    CouleursGB: TGroupBox;
    BlueBtn: TSpeedButton;
    RedBtn: TSpeedButton;
    GreenBtn: TSpeedButton;
    gammaGB: TGroupBox;
    GammaEdit: TEdit;
    MesureRG: TGroupBox;
    couleurGB: TGroupBox;
    IntensiteRB: TRadioButton;
    DistanceRB: TRadioButton;
    AngleRB: TRadioButton;
    IntensitePB: TPaintBox;
    Panel1: TPanel;
    PaintBox1: TPaintBox;
    RegressiBtn: TSpeedButton;
    Memo2: TMemo;
    SpeedButton2: TSpeedButton;
    OpenDialog: TOpenDialog;
    LabelA: TLabel;
    LabelAngle: TLabel;
    LabelO: TLabel;
    StatusBar1: TStatusBar;
    Aide: TLabel;
    LumBtn: TSpeedButton;
    ColorComboBox: TTMSFMXColorPicker;
    procedure FormCreate(Sender: TObject);
    procedure CaptureBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure IntensiteRBChange(Sender: TObject);
    procedure IntensitePBPaint(Sender: TObject; Canvas: TCanvas);
    procedure EchelleRGChange(Sender: TObject);
    procedure MoyenneSEChange(Sender: TObject);
    procedure ColorComboBox1Change(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject; Canvas: TCanvas);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure FormResize(Sender: TObject);
    procedure RegressiBtnClick(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure GreenBtnClick(Sender: TObject);
  private
    Debut,Fin : integer;
    Intensite,PosPoint,rouge,vert,bleu : TvecteurOptique;
    origineEchelle,Gamma : double;
    graphe : Tgraphe;
    aBitmap : TBitmap;
    etat : Tetat;
    procedure setEchelle;
    Procedure CalculIntensite;
    procedure RegleZoom(initRepere : boolean);
  public
    Trace: TrepereShape;
  end;

var
  FCapture: TFCapture;

implementation

{$R *.fmx}

const
    Magnet = 8;
    TickEchelle = 4;
    rayonArc = 50;
    longFleche = 16;
    hintDrag : array[TstyleDrag] of string =
      ('',
       'Dplacer la position de mesure',
       'Modifier la direction de mesure, Maj pour verticale ou horizontale',
       'Dplacer l''origine de l''chelle',
       'Dplacer l''extrmit de l''chelle',
       'Dplacer la direction de rfrence de l''angle',
       'Dplacer la base de l''angle',
       'Dplacer la direction  reprer');

function RGBToL(red,green,blue : byte) : double;
var maxRGB : Double;
begin
    maxRGB := Max(Max(Red, Green), Blue) ;
    result := maxRGB / 255;
end;

procedure TFCapture.FormCreate(Sender: TObject);
begin
  Trace := TrepereShape.Create(paintBox1);
  trace.visible := false;
  trace.SetMoveDown(paintBox1MouseMove,paintBox1MouseDown,paintBox1MouseUp);
  trace.LabelO := labelO;
  trace.LabelA := labelA;
  trace.LabelAngle := labelAngle;
  graphe := Tgraphe.Create;
  graphe.canvas := intensitePB.canvas;
  aBitmap := Tbitmap.Create(640,480);
  colorComboBox.SelectedColor := claWhite;
  BlueBtn.FontColor := claBlue;
  RedBtn.FontColor := claRed;
  GreenBtn.FontColor := claGreen;
  LumBtn.FontColor := claBlack;
  etat := eNone;
end;

procedure TFCapture.FormDestroy(Sender: TObject);
begin
  graphe.Free;
end;

procedure TFCapture.FormResize(Sender: TObject);
begin
    regleZoom(false);
end;

procedure TFCapture.GreenBtnClick(Sender: TObject);
begin
    calculIntensite;
end;

procedure TFCapture.RegressiBtnClick(Sender: TObject);
var
    i : integer;
    Ligne : String;
begin
     calculIntensite;
     Ligne := 'x';
     if blueBtn.isPressed then ligne := ligne+crTab+'B';
     if redBtn.isPressed then ligne := ligne+crTab+'R';
     if greenBtn.isPressed then ligne := ligne+crTab+'G';
     if lumBtn.isPressed then ligne := ligne+crTab+'L';
     memo2.lines.clear;
     memo2.lines.add(Ligne);
     if echelleRG.itemIndex<2
        then Ligne := 'm'
        else Ligne := '';
     if blueBtn.isPressed then ligne := ligne+crTab+'';
     if redBtn.isPressed then ligne := ligne+crTab+'';
     if greenBtn.isPressed then ligne := ligne+crTab+'';
     if lumBtn.isPressed then ligne := ligne+crTab+'';
     memo2.lines.add(Ligne);
     if echelleRG.itemIndex<2
        then Ligne := 'Distance'
        else Ligne := 'Numro pixel';
     if blueBtn.isPressed then ligne := ligne+crTab+'Bleu';
     if redBtn.isPressed then ligne := ligne+crTab+'Rouge';
     if greenBtn.isPressed then ligne := ligne+crTab+'Vert';
     if lumBtn.isPressed then ligne := ligne+crTab+'Intensit relative';
     memo2.lines.add(ligne);
     for i := Debut to Fin do begin
         Ligne := FloatToStrPoint(posPoint[i]);
         if blueBtn.isPressed then ligne := ligne+crTab+FloatToStrPoint(bleu[i]);
         if redBtn.isPressed then ligne := ligne+crTab+FloatToStrPoint(rouge[i]);
         if greenBtn.isPressed then ligne := ligne+crTab+FloatToStrPoint(vert[i]);
         if lumBtn.isPressed then ligne := ligne+crTab+FloatToStrPoint(intensite[i]);
         memo2.lines.add(Ligne);
     end;
     memo2.SelStart := 0;
     memo2.SelLength := length(memo2.Text);
     memo2.CopyToClipboard;
     if pageCourante=0
         then FRegressiMain.ImportePressePapier
         else FRegressiMain.AjoutePressePapier;
     modeAcquisition := acqBitmap;
     FRegressiMain.FinOuvertureFichier(true);
     close;
end;

procedure TFCapture.SpeedButton2Click(Sender: TObject);
begin
    if OpenDialog.execute then begin
        trace.axeI := aX;
        trace.axeEchelle := aX;
        aBitmap.LoadFromFile(openDialog.FileName);
        etat := eAnalyse;
        regleZoom(true);
        regressiBtn.Enabled := true;
        trace.Visible := true;
        paintBox1.repaint;
    end;
end;

procedure TFCapture.CaptureBtnClick(Sender: TObject);
begin
    if SaveDialog.Execute then aBitmap.SaveToFile(SaveDialog.FileName)
end;

procedure TFCapture.ColorComboBox1Change(Sender: TObject);
begin
    trace.stroke.color := colorComboBox.selectedColor;
    labelO.FontColor := colorComboBox.selectedColor;
    labelA.FontColor := colorComboBox.selectedColor;
    labelAngle.FontColor := colorComboBox.selectedColor;
    trace.Repaint;
end;

procedure TFCapture.EchelleRGChange(Sender: TObject);
begin
     trace.echelle := Techelle(echelleRG.ItemIndex);
     echelleGB.Visible := echelleRG.ItemIndex<2;
     // rafraichissement ?
end;

constructor TRepereShape.Create(AOwner: TComponent);
begin
  inherited;
  Parent := TpaintBox(aOwner);
  Stroke.Dash := TStrokeDash.DashDot;
  Stroke.Thickness := 1;
  stroke.color := claWhite;
  zoomAffichage := 1;
  moyenneI := 0;
  align := TAlignLayout.None;
end;

Procedure TRepereShape.Paint;
var xdroit,ydroit,xgauche,ygauche : single;

Procedure TraceCroix(bsi : TstyleDrag);
begin with PointRepere[bsi],canvas do begin
       if bsi=bsEchelle1
          then drawLine(TpointF.create(x-Magnet,y),TpointF.create(x,y),1)
          else drawLine(TpointF.create(x-Magnet,y),TpointF.create(x+Magnet,y),1);
       drawLine(TpointF.create(x,y-Magnet),TpointF.create(x,y+Magnet),1);
end end;

Procedure MarqueEchelle(bsi : TstyleDrag);
var dX,dY : single;
begin with PointRepere[bsi] do begin
       case axeEchelle of
            aX,apX: begin
               if y<height/2
                  then dY := +magnet
                  else dY := -magnet-labelO.Height;
               dX := -labelO.Width/2;
            end;
            aY,apY: begin
              if x<width/2
                then dX := magnet
                else dX := -magnet-labelO.Width;
              dY := -labelO.Height/2;
            end
            else begin
                dX := 0;
                dY := 0;
            end
       end;
       case bsi of
            bsEchelle1: begin
               labelO.position.x := x+dx;
               labelO.position.y := y+dy;
            end;
            bsEchelle2: begin
               labelA.position.x := x+dx;
               labelA.position.y := y+dy;
            end;
       end;
end end;

Procedure TraceEchelle;
var deltaX,deltaY : single;

Procedure TraceTick(i : integer);
var Kp : integer;
    Xa,Ya,Xb,Yb : integer;
    xx,yy : integer;
begin
      xx := round(PointRepere[bsEchelle1].x+deltaX*i);
      yy := round(PointRepere[bsEchelle1].y+deltaY*i);
      if abs(deltaX)>abs(deltaY)
         then begin
              Ya := yy+tickEchelle;
              Yb := yy-tickEchelle;
              Kp := round(-tickEchelle*deltaY/deltaX);
              Xa := xx+Kp;
              Xb := xx-Kp;
         end
         else begin
              Xa := xx+tickEchelle;
              Xb := xx-tickEchelle;
              Kp := round(-tickEchelle*deltaX/deltaY);
              Ya := yy+Kp;
              Yb := yy-Kp;
         end;
      canvas.drawLine(TpointF.Create(Xa,Ya),TpointF.Create(Xb,Yb),1);
end;// traceTick

var i : integer;
begin // traceEchelle
          TraceCroix(bsEchelle1);
          MarqueEchelle(bsEchelle1);
          MarqueEchelle(bsEchelle2);
          traceFleche(pointRepere[bsEchelle1],pointRepere[bsEchelle2]);
          deltaX := (PointRepere[bsEchelle2].x-PointRepere[bsEchelle1].x)/10;
          deltaY := (PointRepere[bsEchelle2].y-PointRepere[bsEchelle1].y)/10;
          canvas.stroke.Thickness := 1;
          canvas.drawLine(PointRepere[bsEchelle1],PointRepere[bsEchelle2],1);
          for i := 1 to 9 do TraceTick(i);
end; // traceEchelle

procedure chercheAxe;
var p : single;
begin
             case axeI of
          aY : begin
               xdroit := PointRepere[bsOrigine].x;
               xgauche := xdroit;
               ydroit := 0;
               ygauche := height;
          end;
          aX : begin
               ydroit := PointRepere[bsOrigine].y;
               ygauche := ydroit;
               xdroit := 0;
               xgauche := width;
          end;
          apX,apY : begin
               p := (PointRepere[bsOrigine].y-PointRepere[bsDirection].y)/
                    (PointRepere[bsOrigine].x-PointRepere[bsDirection].x);
               // intersection xmax
               xdroit := width;
               ydroit := round(p*(xdroit-PointRepere[bsOrigine].x)+PointRepere[bsOrigine].y);
               if (pente>0) and (ydroit>height) then  begin
                   ydroit := height;
                   xdroit := round((ydroit-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
               end;
               if (pente<0) and (ydroit<0) then  begin
                   xdroit := round((0-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
                   ydroit := 0;
               end;
               // intersection xmin
               xgauche := 0;
               ygauche := round(p*(0-PointRepere[bsOrigine].x)+PointRepere[bsOrigine].y);
               if (pente<0) and (ygauche>height) then  begin
                  ygauche := height;
                  xgauche := round((ygauche-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
               end;
               if (pente>0) and (ygauche<0) then  begin
                  xgauche := round((0-PointRepere[bsOrigine].y)/p+PointRepere[bsOrigine].x);
                  ygauche := 0;
               end;
            end
            else begin
               ydroit := height;
               ygauche := 0;
               xdroit := 0;
               xgauche := width;
            end;
       end;
end;

var decaleX,decaleY : integer;
begin // repereShape.paint
       labelA.Visible := echelle=eEchelle;
       labelO.Visible := echelle=eEchelle;
       Stroke.Thickness := 1;
       Fill.Kind := TBrushKind.None;
       Stroke.Dash := TStrokeDash.Solid;
       LabelAngle.Visible := mesure in [mAngle,mDistance];
       if mesure=mIntensite then begin
          with PointRepere[bsOrigine] do begin
               canvas.drawEllipse(TrectF.create(x-10,y-10,x+10,y+10),1);
               canvas.drawEllipse(TrectF.create(x-2,y-2,x+2,y+2),1);
          end;
          traceCroix(bsDirection);
       end;
       chercheAxe;
       stroke.Thickness := moyenneI+1;
       Stroke.Dash := TStrokeDash.DashDot;
       if mesure=mIntensite then begin
          canvas.drawLine(TpointF.Create(xdroit,ydroit),TpointF.Create(xgauche,ygauche),1);
       if moyenneI>2 then begin
          case axeI of
            aX,apX: begin
               decaleY := round(moyenneI*invCosA);
               decaleX := 0;
            end;
            aY,apY: begin
               decaleX := round(moyenneI*invCosa);
               decaleY := 0;
            end;
            else begin
               decaleX := 0;
               decaleY := 0;
            end;
          end;
          stroke.Thickness := 1;
          canvas.drawLine(TpointF.Create(xdroit+decaleX,ydroit+decaleY),TpointF.Create(xgauche+decaleX,ygauche+decaleY),1);
          canvas.drawLine(TpointF.Create(xdroit-decaleX,ydroit-decaleY),TpointF.Create(xgauche-decaleX,ygauche-decaleY),1);
       end;
       end;
       if echelle=eEchelle then traceEchelle;
       case mesure of
            mIntensite : ;
            mAngle : traceAngle;
            mDistance : traceDistance;
       end;
end;  // repereShape.paint

Procedure TrepereShape.AffecteBorne(x,y : single;horizVert : boolean);
var deltaX,deltaY : single;
    p : single;
begin
    case BorneSelect of
        bsOrigine,bsDirection : if
           (x<magnet) or
           (y<magnet) or
           (x>(width-magnet)) or
           (y>(height-magnet)) then exit;
        bsEchelle1,bsEchelle2 : if
          (x<0) or
          (y<0) or
          (x>width-1) or
          (y>height-1) then exit;
       bsNone : exit;
    end;
    cherchePente;
    case BorneSelect of
        bsOrigine : begin
             deltaX := x-PointRepere[bsOrigine].x;
             PointRepere[bsOrigine].x := x;
             deltaY := y-PointRepere[bsOrigine].y;
             PointRepere[bsOrigine].y := y;
             PointRepere[bsDirection].x := PointRepere[bsDirection].x+deltaX;
             if PointRepere[bsDirection].x>width then
                  PointRepere[bsDirection].x := (width-magnet);
             if PointRepere[bsDirection].x<magnet then
                  PointRepere[bsDirection].x := magnet;
             if PointRepere[bsDirection].x<magnet then
                PointRepere[bsDirection].x := magnet;
             PointRepere[bsDirection].y := PointRepere[bsDirection].y+deltaY;
             if PointRepere[bsDirection].y>(height-magnet) then
                  PointRepere[bsDirection].y := (height-magnet);
             if PointRepere[bsDirection].y<magnet then
                  PointRepere[bsDirection].y := magnet;
             if PointRepere[bsDirection].y<magnet then
                  PointRepere[bsDirection].y := magnet;
             if horizVert and
               (PointRepere[bsOrigine].x<>PointRepere[bsDirection].x) then begin
                p := abs((PointRepere[bsOrigine].y-PointRepere[bsDirection].y)/
                     (PointRepere[bsOrigine].x-PointRepere[bsDirection].x));
                if p>1
                   then PointRepere[bsDirection].x := PointRepere[bsOrigine].x
                   else PointRepere[bsDirection].y := PointRepere[bsOrigine].y;
             end;
        end;
        bsDirection : begin
           if (abs(x-PointRepere[bsOrigine].x)+
               abs(y-PointRepere[bsOrigine].y))>magnet
              then begin
                 PointRepere[bsDirection].x := x;
                 PointRepere[bsDirection].y := y;
              end;
           if horizVert and
              (PointRepere[bsOrigine].x<>PointRepere[bsDirection].x) then begin
                p := abs((PointRepere[bsOrigine].y-PointRepere[bsDirection].y)/
                     (PointRepere[bsOrigine].x-PointRepere[bsDirection].x));
                if p>1
                   then PointRepere[bsOrigine].x := PointRepere[bsDirection].x
                   else PointRepere[bsOrigine].y := PointRepere[bsDirection].y;
           end;
        end;
        bsEchelle1 : begin
           if (abs(x-PointRepere[bsEchelle2].x)+
               abs(y-PointRepere[bsEchelle2].y))>magnet
              then begin
                   PointRepere[bsEchelle1].x := X;
                   PointRepere[bsEchelle1].y := Y;
              end;
           if horizVert and
              (PointRepere[bsEchelle1].x<>PointRepere[bsEchelle2].x) then begin
                p := abs((PointRepere[bsEchelle1].y-PointRepere[bsEchelle2].y)/
                     (PointRepere[bsEchelle1].x-PointRepere[bsEchelle2].x));
                if p>1
                  then PointRepere[bsEchelle2].x := PointRepere[bsEchelle1].x
                  else PointRepere[bsEchelle2].y := PointRepere[bsEchelle1].y;
           end;
        end;
        bsEchelle2 : begin
            if (abs(x-PointRepere[bsEchelle1].x)+
                abs(y-PointRepere[bsEchelle1].y))>magnet
              then begin
                 PointRepere[bsEchelle2].x := X;
                 PointRepere[bsEchelle2].y := Y;
              end;
           if horizVert and
              (PointRepere[bsEchelle1].x<>PointRepere[bsEchelle2].x) then begin
                p := abs((PointRepere[bsEchelle1].y-PointRepere[bsEchelle2].y)/
                     (PointRepere[bsEchelle1].x-PointRepere[bsEchelle2].x));
                if p>1
                  then PointRepere[bsEchelle1].x := PointRepere[bsEchelle2].x
                  else PointRepere[bsEchelle1].y := PointRepere[bsEchelle2].y;
           end;
        end;
        bsAngle1 : if (abs(x-PointRepere[bsAngle2].x)+
                abs(y-PointRepere[bsAngle2].y))>magnet
              then begin
                 PointRepere[bsAngle1].x := X;
                 PointRepere[bsAngle1].y := Y;
        end;
        bsAngle3 : if (abs(x-PointRepere[bsAngle2].x)+
                abs(y-PointRepere[bsAngle2].y))>magnet
              then begin
                 PointRepere[bsAngle3].x := X;
                 PointRepere[bsAngle3].y := Y;
        end;
        bsAngle2 : if ((abs(x-PointRepere[bsAngle1].x)+
                        abs(y-PointRepere[bsAngle1].y))>magnet) and
                      ((abs(x-PointRepere[bsAngle3].x)+
                        abs(y-PointRepere[bsAngle3].y))>magnet)
              then begin
                 PointRepere[bsAngle2].x := X;
                 PointRepere[bsAngle2].y := Y;
        end;
    end;
    repaint; // retrace
    if BorneSelect=bsNone
       then cursor := crDefault
       else cursor := crHandPoint;
end; // AffecteBorne

procedure TFCapture.IntensitePBPaint(Sender: TObject; Canvas: TCanvas);
begin with graphe do begin
        if (debut=fin) then exit;
        LimiteCourbe := TrectF.create(0,0,
           abitmap.width*trace.zoomAffichage,
           IntensitePB.height);
        debutG := debut;
        finG := fin;
        intensiteY := intensite;
        rougeY := rouge;
        vertY := vert;
        bleuY := bleu;
        finG := fin;
        debutG := debut;
        withR := redbtn.isPressed;
        withB := blueBtn.isPressed;
        withG := greenbtn.isPressed;
        withL := LumBtn.isPressed;
        drawG;
end end;

procedure TFCapture.IntensiteRBChange(Sender: TObject);
begin
     if angleRB.isChecked then trace.mesure := mAngle;
     if intensiteRB.isChecked then trace.mesure := mIntensite;
     if distanceRB.isChecked then trace.mesure := mDistance;
     CouleursGB.Visible := trace.mesure = mIntensite;
end;

procedure TFCapture.MoyenneSEChange(Sender: TObject);
begin
    trace.moyenneI := round(moyenneSE.Value);
    calculIntensite;
end;

procedure TFCapture.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
   if (etat<>eAnalyse) then exit;
   trace.BorneSelect := trace.GetBorne(x,y);
end;

procedure TFCapture.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single);
var BS : tstyleDrag;
begin
   if (etat<>eAnalyse) then exit;
   if (ssLeft in Shift) or (ssRight in Shift)
      then trace.AffecteBorne(x,y,ssShift in shift)
      // ssShift touche majuscule donc vert ou horiz
      else begin
          BS := trace.GetBorne(x,y);
          if BS=bsNone
             then paintBox1.cursor := crDefault
             else paintBox1.cursor := crHandPoint;
          trace.Cursor := paintBox1.cursor;
          aide.text := hintDrag[BS];
      end;
end;

procedure TFCapture.PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
     if (etat<>eAnalyse) then exit;
     trace.AffecteBorne(x,y,ssShift in shift);
     CalculIntensite;
     aide.text := '';
     trace.BorneSelect := bsNone;
end;

procedure TFCapture.PaintBox1Paint(Sender: TObject; Canvas: TCanvas);
var MyRect,MyRectDest : TRectF;
begin
   if etat=eNone then exit;
   if trace.zoomAffichage=0 then regleZoom(false);
   MyRect.Create(0, 0, aBitmap.width, aBitmap.height);
   MyRectDest.Create(0, 0, aBitmap.width*trace.zoomAffichage, aBitmap.height*trace.zoomAffichage);
   paintBox1.Canvas.BeginScene;
   paintBox1.canvas.DrawBitmap(abitmap, myRect,MyRectDest,1);
   paintBox1.Canvas.EndScene;
end;

Function TRepereShape.GetBorne(x,y : single) : TstyleDrag;
var i,iMax,iMin : TstyleDrag;
begin
      result := bsNone;
      case mesure of
           mAngle : begin
              imin := bsEchelle1;
              imax := bsAngle3;
           end;
           mDistance : begin
              imin := bsEchelle1;
              imax := bsAngle2;
           end;
           mIntensite : begin
              imin := bsOrigine;
              if echelle=eEchelle
                 then imax := bsEchelle2
                 else imax := bsDirection;
           end
           else begin
              imin := bsOrigine;
              imax := bsEchelle2;
           end;
      end;
      for i := iMin to imax do
          if abs(x-PointRepere[i].x)+
             abs(y-PointRepere[i].y) < Magnet
               then begin
                  result := i;
                  break;
               end;
end;

 procedure TRepereShape.TraceFleche(P1,P2: TpointF);
    var
      dx,dy: single;
      Points:  TPolygon;
      d : double;
    begin // P2 = extrmit de la flche ; P1-P2 = direction
      dx := P2.X-P1.X;
      dy := P2.y-P1.y;
      d := longFleche/sqrt(sqr(dX) + sqr(dY));
      dX := round(d * dX);
      dY := round(d * dY);
      SetLength(Points, 4);
      Points[0] := TPointF.create(P2.x, P2.y);
      Points[1] := TPointF.create(P2.x - dX - (dY/3), P2.Y - dY + (dX/3));
      Points[2] := TPointF.create(P2.x - dX + (dY/3), P2.y - dY - (dX/3));
      Points[3] := Points[0];
      canvas.drawPolygon(Points,1); // triangle 3 points
    end;

  procedure TrepereShape.TraceAngle;
  var angleRef,angleFin : TStyleDrag;

   function valeurAngle : string;
   var CA,CB : TpointF;
       PS,PV : single;
       valeur : double;
   begin
      CA.x := pointRepere[bsAngle1].x-pointRepere[bsAngle2].x;
      CA.y := pointRepere[bsAngle1].y-pointRepere[bsAngle2].y;
      CB.x := pointRepere[bsAngle3].x-pointRepere[bsAngle2].x;
      CB.y := pointRepere[bsAngle3].y-pointRepere[bsAngle2].y;
      PS := CA.X*CB.X+CA.Y*CB.Y;
      PV := -CA.X*CB.Y+CA.Y*CB.X;
      valeur := radToDeg(arcTan2(PV,PS));
      if valeur>0 then angleRef := bsAngle1 else angleRef := bsAngle3;
      if angleRef=bsAngle1 then angleFin := bsAngle3 else angleFin := bsAngle1;
      result := intToStr(round(valeur))+' ';
   end;

   var xt,yt,signe : single;
       lT : single;
       dx,dy : single;
       StartAngle, SweepAngle: Single;
       Points:  TPolygon;
   begin
       LabelAngle.text := valeurAngle;
       StartAngle := 0;
       SweepAngle := 10;
       Canvas.drawArc(pointRepere[bsAngle2],
                      TPointF.Create(rayonArc,rayonArc),
                      StartAngle,SweepAngle,1);
       canvas.drawLine(pointRepere[bsAngle2],pointRepere[bsAngle1],1);
       canvas.drawLine(pointRepere[bsAngle2],pointRepere[bsAngle3],1);
       xT := -pointRepere[bsAngle2].x+((pointRepere[bsAngle3].x+pointRepere[bsAngle1].x)/2);
       yT := -pointRepere[bsAngle2].y+((pointRepere[bsAngle3].y+pointRepere[bsAngle1].y)/2);
       lT := rayonArc/sqrt(sqr(xt)+sqr(yt));
       LabelAngle.position.x := (pointRepere[bsAngle2].x+round(xT*lT));
       if angleRef=bsAngle1 then signe := 0 else signe := -1;
       LabelAngle.position.y := (pointRepere[bsAngle2].y+round(yT*lT)+signe*LabelAngle.Height);
// L'arc traverse le primtre de l'ellipse de centre (X1,Y1) et de rayon (X2,Y2).
// L'arc est dessin en suivant le primtre de l'ellipse, dans le sens contraire des aiguilles d'une montre
// jusqu'au point d'arrive.
        dx := pointRepere[bsAngle3].X- pointRepere[bsAngle2].X;
        dy := pointRepere[bsAngle3].y- pointRepere[bsAngle2].y;
        lT := rayonArc/sqrt(sqr(dX) + sqr(dY));
        Points[0] := TPointF.create(pointRepere[bsAngle2].X+round(lT * dX),
                           pointRepere[bsAngle2].y+round(lT * dY));
        lT := longFleche/sqrt(sqr(dX) + sqr(dY));
        dX := round(lT * dX);
        dY := round(lT * dY);
        if angleRef=bsAngle1 then begin dX := -dX; dY := -dY end;
        SetLength(Points, 4);
        Points[1] := TPointF.create(Points[0].x + dY - (dX/3), Points[0].Y - dX - (dY/3));
        Points[2] := TPointF.create(Points[0].x + dY + (dX/3), Points[0].y - dX + (dY/3));
        Points[3] := Points[0];
        canvas.drawPolygon(Points,1); // triangle 3 points
   end;

   procedure TrepereShape.TraceDistance;
   var xt,yt : single;
       arrondi,valeur : double;
       unite : string;
       NbrePixelEchelle : double;
       dimPix : double;
   begin
       unite := ' m';
       case echelle of
          ePixel : dimPix := longueurRef;
          eEchelle : begin
             NbrePixelEchelle := PointRepere[bsEchelle1].distance(PointRepere[bsechelle2]);
             dimPix := LongueurRef/NbrePixelEchelle;
          end;
          eIndex : begin
             dimPix := 1;
             unite := '';
          end
          else dimPix := 1;
       end;
       canvas.DrawLine(pointRepere[bsAngle2],pointRepere[bsAngle1],1);
       traceFleche(pointRepere[bsAngle1],pointRepere[bsAngle2]);
       traceFleche(pointRepere[bsAngle2],pointRepere[bsAngle1]);
       xT := (pointRepere[bsAngle2].x+pointRepere[bsAngle1].x)/2;
       yT := (pointRepere[bsAngle2].y+pointRepere[bsAngle1].y)/2;
       LabelAngle.position.x := xt;
       LabelAngle.position.y := yt;
       Arrondi  := power(10, floor(log10(abs(dimPix))));
       valeur := pointRepere[bsAngle2].distance(pointRepere[bsAngle2]) * dimPix;
       LabelAngle.text := FloatToStrF(Arrondi * round(valeur / Arrondi),ffFixed,5,3)+unite;
   end;

procedure TRepereShape.cherchePente;
var penteE : single;
begin
 if (PointRepere[bsOrigine].x=PointRepere[bsDirection].x)
        then begin
           axeI := aY;
           pente := 0;
           invCosA := 1;
        end
        else if PointRepere[bsOrigine].y=PointRepere[bsDirection].y
           then begin
              axeI := aX;
              pente := 0;
              invCosA := 1;
           end
           else begin
              pente := (PointRepere[bsOrigine].y-PointRepere[bsdirection].y)/
                       (PointRepere[bsOrigine].x-PointRepere[bsdirection].x);
              if abs(pente)<=1
                 then begin
                    axeI := apX;
                    invcosa := sqrt(1+sqr(pente));
                 end
                 else begin
                    axeI := apY;
                    invcosa := sqrt(1+sqr(1/pente));
                 end;
          end;
  if (PointRepere[bsEchelle1].x=PointRepere[bsEchelle2].x)
        then axeEchelle := aY
        else if (PointRepere[bsEchelle1].y=PointRepere[bsEchelle2].y)
           then axeEchelle := aX
           else begin
              penteE := (PointRepere[bsOrigine].y-PointRepere[bsdirection].y)/
                       (PointRepere[bsOrigine].x-PointRepere[bsdirection].x);
              if abs(penteE)<=1
                 then axeEchelle := apX
                 else axeEchelle := apY;
          end;
end;

Procedure TFCapture.CalculIntensite;
var origineX,origineY : single;
    dimPix : double;

Procedure chercheEchelle;
var NbrePixelEchelle : double;
begin with trace do begin
   OrigineX := PointRepere[bsOrigine].x;
   OrigineY := PointRepere[bsOrigine].y;
   case echelleRG.itemIndex of
        1 : dimPix := LongueurRef;
        0 : begin
          NbrePixelEchelle := round(PointRepere[bsEchelle1].distance(PointRepere[bsEchelle2])/zoomAffichage);
          dimPix := LongueurRef/NbrePixelEchelle;
          case axeI of
             aX : OrigineX := PointRepere[bsEchelle1].x;
             aY : OrigineY := PointRepere[bsEchelle1].y;
             else begin
                OrigineX := round((pente*(PointRepere[bsEchelle1].y-PointRepere[bsOrigine].y)+
                             sqr(pente)*PointRepere[bsOrigine].x+
                             PointRepere[bsEchelle1].x)/(1+sqr(pente)));
                OrigineY := round((pente*(PointRepere[bsEchelle1].x-PointRepere[bsOrigine].x)+
                             sqr(pente)*PointRepere[bsEchelle1].y+
                             PointRepere[bsOrigine].y)/(1+sqr(pente)));
             end;
          end;
        end;
        else dimPix := 1;
   end;
   case axeI of
      aX,apX : begin
           debut := round(-OrigineX);
           fin := round(width-OrigineX);
      end;
      aY,apY : begin
           debut := round(-OrigineY);
           fin := round(height-OrigineY);
      end;
   end;
   dimPix := dimPix*invCosA;
   debut := ceil(debut/zoomAffichage);
   fin := trunc(fin/zoomAffichage);
   OrigineX := OrigineX/zoomAffichage;
   OrigineY := OrigineY/zoomAffichage;
   if debut<-maxVecteurOptique then debut := -maxVecteurOptique;
   if fin>maxVecteurOptique then fin := maxVecteurOptique;
end end; // chercheEchelle

Procedure cherche(decale : integer); // sur le bitmap
var xi,yi : integer;
    x0,y0 : single;

procedure getxiyi(i : integer);
begin
      case trace.axeI of
            aX,apX : begin
                 xi := round(x0+i);
                 yi := round(y0+i*trace.pente);
            end;
            else begin
                 yi := round(y0+i);
                 xi := round(x0+i/trace.pente);
            end;
         end;
end;

var
    valeur : TAlphaColor;
    rr,gg,bb : integer;
    vBitMapData : TBitmapData;
    i : integer;
begin with trace do begin
     case axeI of
       aX,apX : begin
            x0 := origineX;
            y0 := origineY+decale;
       end;
       else begin
            x0 := origineX+decale;
            y0 := origineY;
       end;
     end;
     if aBitmap.Map(TMapAccess.Read, vBitMapData) then begin // lock bitmap and get pixels
     getxiyi(debut);
     while (xi<0) or (xi>=abitmap.width) or
           (yi<0) or (yi>=abitmap.height) do begin
              inc(debut);
              getxiyi(debut);
     end;
     getxiyi(fin);
     while (xi<0) or (xi>=abitmap.width) or
           (yi<0) or (yi>=abitmap.height) do begin
             dec(fin);
             getxiyi(fin);
     end;
     for i := debut to fin do begin
         getxiyi(i);
         try
         valeur := vBitmapData.getPixel(xi,yi);
//         vBitmapData.SetPixel(xi,yi, aColor);
//              aa := (valeur shr 24) and $FF; transparence
         rr := TAlphaColorRec(valeur).R;
         rouge[i] := rouge[i]+rr;
         bb := TAlphaColorRec(valeur).B;
         bleu[i] := bleu[i]+bb;
         gg := TAlphaColorRec(valeur).G;
         vert[i] := vert[i]+gg;
         intensite[i] := intensite[i]+RGBtoL(rr,gg,bb);
         except
               intensite[i] := 0;
               vert[i] := 0;
               rouge[i] := 0;
               bleu[i] := 0;
          end;
     end; // for i
     aBitmap.Unmap(vBitMapData);      // unlock the bitmap
     end;
end end; // cherche

procedure getEchelle;
begin
     try
       gamma := StrToFloat(GammaEdit.text);
     except
       gamma := 1;
     end;
     case echelleRG.itemIndex of
       0 : begin // echelle
          try
              OrigineEchelle := StrToFloat(OrigineEdit.text)*power(10,-3*uniteCB.ItemIndex);
          except
          showMessage(erEchelle);
          origineEchelle := 0;
          OrigineEdit.text := '0';
          end;
       end;
       1 : origineEchelle := 0; // pixel
       2 : begin // index
            origineEchelle := 0;
            trace.longueurRef := 1;
            exit;
       end;
     end;
     try
     trace.longueurRef := StrToFloat(EchelleEdit.text)*power(10,-3*uniteCB.ItemIndex);
     if echelleRG.itemIndex=0 then trace.LongueurRef := trace.longueurRef-origineEchelle;
     except
         showMessage(erEchelle);
         EchelleEdit.Text := '1';
         trace.longueurRef := power(10,-3*uniteCB.ItemIndex);
     end;
end; // getEchelle

var i,Nbre : integer;
begin // CalculIntensite
     trace.moyenneI := round(moyenneSE.value);
     getEchelle;
     setEchelle;
     for i := -maxVecteurOptique to maxVecteurOptique do begin
         Intensite[i] := 0;
         vert[i] := 0;
         bleu[i] := 0;
         rouge[i] := 0;
     end;
     trace.cherchePente;
     chercheEchelle;
     cherche(0);
     for i := 1 to trace.moyenneI do begin
        cherche(i);
        cherche(-i);
     end;
     Nbre := 1+2*trace.moyenneI;
     for i := Debut to Fin do begin
         PosPoint[i] := dimPix*i+origineEchelle;
         Intensite[i] := power(Intensite[i]/Nbre,gamma);
         rouge[i] := rouge[i]/255/Nbre;
         vert[i] := vert[i]/255/Nbre;
         bleu[i] := bleu[i]/255/Nbre;
     end;
     intensitePB.repaint;
end; // CalculIntensite

procedure TFCapture.setEchelle;
begin
    case echelleRG.itemIndex of
       0 : if origineEchelle=0
           then begin
              echelleEditLabel.text := trLongEchelle;
              origineEditLabel.text := trOrigineEchelle;
           end
           else begin
              echelleEditLabel.text := 'Coordonne de A'; // curviligne !
              origineEditLabel.text := 'Coordonne de O';
           end;
       1 : EchelleEditLabel.text := trDimPixel;
       2 : ;
    end;
    EchelleEdit.visible := echelleRG.itemIndex<2;
    OrigineEdit.Visible := echelleRG.itemIndex=0;
    OrigineZeroBtn.Visible := echelleRG.itemIndex=0;
    UniteCB.visible := echelleRG.itemIndex<2;
    EchelleEdit.hint := EchelleEditLabel.text+' '+hAideValide;
    trace.Repaint;
end;

Procedure Tgraphe.WindowXY(xr,yr : double;var xi,yi : single);
begin
        if isNan(xr) or isNan(yr)
           then begin
              xi := -1;
              yi := -1;
           end
           else begin
              xi := round(Ax*xr+Bx);
              yi := round(Ay*yr+By);
           end;
end;

procedure Tgraphe.DrawG;

procedure ChercheMonde;
begin
    with LimiteCourbe do begin
         Ax := (right-left)/(finG-debutG);
         Bx := left-debutG*Ax;
         Ay := (top-bottom)/(MaxiY-MiniY);
         By := bottom-MiniY*Ay;
     end;
end;// ChercheMonde

procedure TraceCouleur(vect : TvecteurOptique;couleur : TAlphaColor);
var x,y : single;
    i : integer;
    aPath : TPathData;
begin with LimiteCourbe do begin
      if finG<=debutG then exit;
      windowXY(debutG,vect[debutG],x,y);
      Canvas.stroke.color := couleur;
      aPath := TPathData.Create;
      aPath.moveTo(TPointF.Create(x,y));
      for i := debutG to finG do begin
           windowXY(i,vect[i],x,y);
           if (x>0) and (y>0) then aPath.LineTo(TPointF.Create(x,y))
      end;
      canvas.drawPath(aPath,1);
      aPath.Free;
end end;// TraceCouleur

begin // tgraphe.drawG
     with canvas do begin
         stroke.kind := TBrushKind.Solid;
         fill.color := couleurFond;
         stroke.Thickness := 1;
         FillRect(LimiteCourbe,0,0,AllCorners,1);
     end;
     if finG>maxVecteurOptique then finG := maxVecteurOptique;
     if debutG<-maxVecteurOptique then debutG := -maxVecteurOptique;
     ChercheMonde;
     if withR then TraceCouleur(rougeY,claRed);
     if withB then TraceCouleur(bleuY,claBlue);
     if withG then TraceCouleur(vertY,claGreen);
     if withL then TraceCouleur(intensiteY,claBlack);
end; // tgraphe.drawG

constructor Tgraphe.Create;
begin
   inherited;
   finG := 0;
   debutG := 0;
   miniY := -0.05;
   maxiY := 1.05;
end;

procedure TFCapture.RegleZoom(initRepere : boolean);
var zoomX,zoomY : single;
    largeurI,hauteurI : single;
begin
     ZoomY := panel1.Height/abitmap.height*3/4;
     ZoomX := panel1.Width/aBitmap.width;
     if zoomY>zoomX
        then trace.zoomAffichage := zoomX
        else trace.zoomAffichage := zoomY;
     hauteurI := abitmap.height*trace.zoomAffichage;
     paintBox1.Height := hauteurI;
     largeurI := abitmap.width*trace.zoomAffichage;
     if initRepere then with trace do begin
        PointRepere[bsOrigine] := TpointF.create(largeurI/2,hauteurI/2);
        PointRepere[bsDirection] := TpointF.create(4*largeurI/5,hauteurI/2);
        axeI := aX;
        PointRepere[bsEchelle1] := TpointF.create(largeurI/5,3*hauteurI/4);
        PointRepere[bsEchelle2] := TpointF.create(4*largeurI/5,3*hauteurI/4);
        axeEchelle := aX;
        PointRepere[bsAngle1] := TpointF.create(largeurI/4,2*hauteurI/3);
        PointRepere[bsAngle2] := TpointF.create(largeurI/8,hauteurI/3);
        PointRepere[bsAngle3] := TpointF.create(largeurI/4,hauteurI/3);
     end;
     trace.BorneSelect := bsNone;
     trace.Width := largeurI;
     trace.Height := hauteurI;
end;

procedure TRepereShape.SetMoveDown(move : TMouseMoveEvent; down,up : TMouseEvent);
begin
     OnMouseMove := move;
     OnMouseDown := down;
     OnMouseUp := up;
end;

end.

