unit MediaPlayerForm;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  System.Variants, System.Actions, System.Math.vectors, System.UIConsts,
  System.IOUtils, system.inifiles,

  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Objects, FMX.Layouts,
  FMX.Platform, FMX.clipBoard, FMX.StdCtrls, FMX.ActnList,
  FMX.Memo, FMX.ListBox, FMX.Graphics, FMX.Utils,
  FMX.Controls.Presentation, FMX.Edit, FMX.EditBox, FMX.SpinBox, FMX.Colors,
  FMX.TMSToolBar, FMX.TMSBitmapContainer,

  images, maths, rollingShutterCalc,
  avProbe;

type
  TstyleDrag = (bsNone,bsMesure,bsOrigine,bsEchelle1,bsEchelle2,bsChrono,bsChronoInit,bsAxeX,bsAxeY);
  TarrayPointF = array[TstyleDrag] of TpointF;

  TVideoForm = class(TForm)
    OpenDialog: TOpenDialog;
    TrackBar: TTrackBar;
    TracePointsCB: TCheckBox;
    TraceEchCB: TCheckBox;
    EchelleEdit: TEdit;
    TimerPlay: TTimer;
    TMSFMXToolBar1: TTMSFMXToolBar;
    OpenBtn: TTMSFMXToolBarButton;
    MesureBtn: TTMSFMXToolBarButton;
    CouleurPointCB: TTMSFMXToolBarColorPicker;
    CouleurAxeCB: TTMSFMXToolBarColorPicker;
    RegressiBtn: TTMSFMXToolBarButton;
    NextBtn: TTMSFMXToolBarButton;
    PreviousBtn: TTMSFMXToolBarButton;
    RazBtn: TTMSFMXToolBarButton;
    PlayBtn: TTMSFMXToolBarButton;
    EchelleLabel: TLabel;
    Separator2: TTMSFMXToolBarSeparator;
    Separator3: TTMSFMXToolBarSeparator;
    SaveDialog: TSaveDialog;
    ChronoBtn: TTMSFMXToolBarButton;
    PanelPreview: TPanel;
    ImgPreview: TImage;
    TMSFMXBitmapContainer1: TTMSFMXBitmapContainer;
    RotationLabel: TLabel;
    GaucheBtn: TTMSFMXToolBarButton;
    DroiteBtn: TTMSFMXToolBarButton;
    RetardLabel: TLabel;
    RollingShutterSB: TSpinBox;
    NpointsLabel: TLabel;
    NbrePointsSB: TSpinBox;
    trigoCB: TCheckBox;
    LoupeLabel: TLabel;
    LoupeSB: TSpinBox;
    IncertLabel: TLabel;
    incertSB: TSpinBox;
    TrackbarPanel: TPanel;
    DebutBtn: TButton;
    OptionsPanel: TPanel;
    EditBidon: TEdit;
    TimerLoupe: TTimer;
    procedure OpenButtonClick(Sender: TObject);
    procedure TrackBarChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure acPreviousExecute(Sender: TObject);
    procedure acNextExecute(Sender: TObject);
    procedure VideoMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure VideoMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure VideoMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Single);
    procedure MesureBtnClick(Sender: TObject);
    procedure RegressiBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerPlayTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CouleurPointCBColorSelected(Sender: TObject;
      AColor: TAlphaColor);
    procedure CouleurAxeCBColorSelected(Sender: TObject; AColor: TAlphaColor);
    procedure RazBtnClick(Sender: TObject);
    procedure ChronoBtnClick(Sender: TObject);
    procedure TraceEchCBChange(Sender: TObject);
    procedure TracePointsCBChange(Sender: TObject);
    procedure PlayBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure PanelPreviewResize(Sender: TObject);
    procedure EchelleEditKeyDown(Sender: TObject; var Key: Word;
      var KeyChar: Char; Shift: TShiftState);
    procedure LoupeSBChange(Sender: TObject);
    procedure PanelPreviewMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
//    procedure TimerMoveTimer(Sender: TObject);
    procedure DroiteBtnClick(Sender: TObject);
    procedure trigoCBChange(Sender: TObject);
    procedure NbrePointsSBChange(Sender: TObject);
    procedure VideoClick(Sender: TObject);
    procedure DebutBtnClick(Sender: TObject);
    procedure PanelPreviewKeyDown(Sender: TObject; var Key: Word;
      var KeyChar: Char; Shift: TShiftState);
    procedure VideoTap(Sender: TObject; const Point: TPointF);
    procedure CouleurAxeCBEChange(Sender: TObject);
    procedure TimerLoupeTimer(Sender: TObject);
  private
    iDebug : integer;
    PointRepere : TarrayPointF;
    LongueurEchelle,echelle : double;
    e_PointRepere : TarrayPointF;
    BorneSelect : TstyleDrag;
    NmesVideo : integer;
    mesureEnCours : boolean;
    largeur, hauteur : integer;  // video
    LargeurAff,HauteurAff : integer; // Affiche
    DureeImage : double;
    pasZoom : single; // conversion xy(video) vers xy(cran)
    FProbe:TAVProbe;
    couleurPoint : array[indicePoint] of TAlphaColor;
    NbrePoints,pointCourant : indicePoint;
    bitmapCourant,bitmapFin,bitmapInit : Tbitmap;
    BlueCibleChrono, redCibleChrono,GreenCibleChrono : TColor;
    nomFichierChrono : string;
    ChronoPhotoEnCours : boolean;
    gainLoupe : integer;
    rotation : integer;
    hauteurLoupe,demiHauteurLoupe : integer;
    signeY : integer;
    nouveauPoint : boolean;
    procedure Dessine;
    procedure EcranVersImage;
    procedure calcEchelle;
    procedure SetCurseur(Acursor : Tcursor);
    procedure EnregistrePoint(Sender: TObject;Shift: TShiftState;x,y : single);
    function AppartientImage(var X, Y: single) : boolean;
    Function GetBorne(x,y : single;deplace : boolean) : TstyleDrag;
    procedure setBorne(aBorne : TStyleDrag);
    procedure setBoutons;
    procedure ImageVersEcran;
    procedure sauveImage(index : single);
    procedure effectuerZoom;
    procedure AffecteChrono;
    procedure InitChronoPhoto;
    procedure SaveChronoPhoto;
    procedure videRepertoire;
    function getNomImage(index : single) : string;
    procedure afficheImage(index : single);
    procedure upDateImage(NomImage : string);
    procedure CacheLoupe;
    procedure ConvertitXY(Sender: TObject;var X, Y: single);
    procedure ConvertitCibleXY(Sender: TObject;var X, Y: single);
    Function OrigineImageToScreen : TpointF;
  public
  end;

var
  VideoForm: TVideoForm;

implementation

{$R *.fmx}

uses math, avcodec, avlib, loupeU,
  {$IFDEF MacOS}
     Posix.Unistd,   // pour deleteFile
  {$ENDIF}
     regutil, compile, valeurs, graphker, grapheVariabU, regmain, chronoViewU;

const  // Pour chronoPhoto
    pasCouleur = 16;
    resolution = 192;
    resolutionCibleMin = 32;
    resolutionCibleMax = 6*pasCouleur;
    Magnet = 8;

procedure SetPixel(abitmap : tbitmap;i,j : integer;acolor : TAlphaColor);
 // example of setting a single pixel in a Tbitmap.
var
  vBitMapData  : TBitmapData;
begin
 // lock and get the bitmap pixels
   if  aBitmap.Map(TMapAccess.Write, vBitMapData) then begin
      vBitmapData.SetPixel(i,j, aColor); // set the pixel colour at i, j
      aBitmap.Unmap(vBitMapData);         // unlock the bitmap
   end
end;

function GetPixel(abitmap : tbitmap;i,j : integer) : TAlphaColor;
 // example of setting a single pixel in a Tbitmap.
var
  vBitMapData  : TBitmapData;
begin
 // lock and get the bitmap pixels
   if  aBitmap.Map(TMapAccess.Read, vBitMapData) then begin
      result := vBitmapData.GetPixel(i,j); // set the pixel colour at i, j
      aBitmap.Unmap(vBitMapData);         // unlock the bitmap
   end
   else result := claWhite;
end;

Function TVideoForm.OrigineImageToScreen : TpointF;
var aPoint : TPointF;
begin
    aPoint := PointF(0,0); // Origine imgPreview
    aPoint.Offset(panelPreview.position.x,panelPreview.position.y); // Origine / fentre client
   (*
    {$IFDEF MACOS}
    aPoint.Offset(panelPreview.position.x,panelPreview.position.y); // Origine / fentre client
    {$ELSE}
    aPoint.Offset(panelPreview.position.x,panelPreview.position.y); // Origine / fentre client
    {$ENDIF}
    *)
    aPoint := clientToScreen(aPoint);  // Origine / cran
    result := aPoint;
end;

procedure TVideoForm.ConvertitXY(Sender: TObject;var X, Y: single);
var LOrigine : TPointF;
begin
    if sender=LoupeForm.image then begin
       LOrigine := OrigineImageToScreen;
       x := x+LoupeForm.left-LOrigine.X;
       y := y+LoupeForm.top-LOrigine.Y;
    end;
    if sender=VideoForm then begin
       x := x-panelPreview.Position.x;
       y := y-panelPreview.Position.y;
    end;
end;

procedure TVideoForm.ConvertitCibleXY(Sender: TObject;var X, Y: single);
var LOrigine : TPointF;
begin
    if sender=LoupeForm.image then begin
       LOrigine := OrigineImageToScreen;
       x := demiHauteurLoupe+LoupeForm.left-LOrigine.X;
       y := demiHauteurLoupe+LoupeForm.top-LOrigine.Y;
    end;
    if sender=VideoForm then begin
       x := x-panelPreview.Position.x;
       y := y-panelPreview.Position.y;
    end;
end;

procedure TVideoForm.CacheLoupe;
begin
     if loupeForm.visible then loupeForm.hide;
     case BorneSelect of
          bsEchelle1,bsEchelle2,bsOrigine : setCurseur(crSizeAll);
          bsMesure,bsChronoInit : setCurseur(crCross);
     end;
     FRegressiMain.timerLoupe.enabled := false;
end;

procedure TVideoForm.acNextExecute(Sender: TObject);
var nomImage : string;
    sauveTimer : boolean;
begin
   if (FProbe<>nil) and FProbe.Active and (TrackBar.Value<trackBar.Max) then begin
       sauveTimer := timerPlay.Enabled;
       TimerPlay.Enabled := false;
       NextBtn.Enabled := false;
       PreviousBtn.Enabled := false;
       nomImage := getNomImage(trackBar.Value+1);
       if (nomImage='') or not FileExists(nomImage)
          then begin
               FProbe.NextFrame; // marche pour toutes les videos sauf .mov
               sauveImage(trackBar.Value+1);
          end
          else TrackBar.Value := trackBar.Value+1; // trackBarChange fait le travail
       TimerPlay.Enabled := sauveTimer and (trackBar.Value<trackBar.Max);
       if sauveTimer and not TimerPlay.Enabled then trackBar.value := 0;
   end;
end;

procedure TVideoForm.acPreviousExecute(Sender: TObject);
begin
   if (TrackBar.Value>0) and (FProbe<>nil) and FProbe.Active then begin
       NextBtn.Enabled := false;
       PreviousBtn.Enabled := false;
       TrackBar.Value := round(trackBar.Value)-1;
   end;
end;

procedure TVideoForm.OpenButtonClick(Sender: TObject);

procedure RecupereDataProbe;
begin
     try
          trackBar.tag := 1;
          TrackBar.Max := FProbe.Duration*FProbe.videoframeRate-1;
          trackbar.Min := 0;
          trackBar.Value := 0;
          trackBar.tag := 0;
          largeur := FProbe.VideoWidth;
          hauteur := FProbe.VideoHeight;
          PointRepere[bsOrigine] := pointF(Largeur/2,Hauteur/2);
          PointRepere[bsEchelle1] := pointF(Largeur/4,3*Hauteur/4);
          PointRepere[bsEchelle2] := pointF(3*Largeur/4,3*Hauteur/4);
          EffectuerZoom;
          DureeImage := 1/FProbe.videoframeRate;
          // FProbe.StartTime;
     except
        cursor := crDefault;
     end;
end;


begin
  if OpenDialog.Execute then begin
    iDebug := 0;
    cacheLoupe;
    setCurseur(crDefault);
    NmesVideo := 0;
    pasZoom := 1;
    setBorne(bsNone);
    Caption := 'Acquisition video ['+extractFileName(OpenDialog.FileName)+']';
    FProbe.Active := false;
    videRepertoire;
    FProbe.FileName := OpenDialog.FileName;
    FProbe.Active := true;
    RecupereDataProbe;
    setBoutons;
    FProbe.NextFrame;
    sauveImage(0);
    PlayBtnClick(sender);
  end;
end;

procedure TVideoForm.PanelPreviewKeyDown(Sender: TObject; var Key: Word;
  var KeyChar: Char; Shift: TShiftState);
var pointCourant : TPointF;

procedure decaleSouris(x,y : single);
begin
    PointCourant := Screen.MousePos;
    PointCourant.offset(x,y);
//    Screen.MousePos := PointCourant;
end;

begin
    if (keyChar=' ') and (borneSelect=bsMesure) then begin
        pointCourant := Screen.MousePos;
        pointCourant := ScreenToClient(pointCourant);
        pointCourant.Offset(-PanelPreview.position.X,-PanelPreview.position.y);
        EnregistrePoint(PanelPreview,Shift,pointCourant.x,pointCourant.y);
    end;
    (*
    case keyChar of
         crDroite : decaleSouris(1,0);
         crGauche : decaleSouris(-1,0);
         crBas : decaleSouris(0,1);
         crHaut : decaleSouris(0,-1);
    end;
    *)
end;

procedure TVideoForm.PanelPreviewMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Single);
begin
    cacheLoupe;
end;

procedure TVideoForm.PanelPreviewResize(Sender: TObject);
begin
      effectuerZoom;
end;

procedure TVideoForm.PlayBtnClick(Sender: TObject);
begin
  if timerPlay.enabled then begin
     timerPlay.Enabled := false;
     PlayBtn.Text:='Play';
  end
  else begin
     timerPlay.Enabled := true;
     PlayBtn.Text:='Pause';
  end;
end;

procedure TVideoForm.VideRepertoire;
var info : TSearchRec;
    recherche,nomFichier : string;
begin
  recherche := System.IOUtils.Tpath.combine(supportPath,'*.bmp');
  if findFirst(recherche,faAnyFile,Info)=0 then begin
      repeat
         if (Info.attr and faDirectory)<>faDirectory then begin
            nomFichier := System.IOUtils.Tpath.combine(supportPath,Info.Name);
            // si temp  test dbut du fichier = temp
            deleteFile(nomFichier);
         end;
      until FindNext(Info)<>0;
      FindClose(info);
  end;
end;

procedure TVideoForm.RazBtnClick(Sender: TObject);
begin
      if OKReg('Suppression des donnes',0) then begin
         NmesVideo := 0;
         afficheImage(trackBar.value);
         setBoutons;
     end;
end;

procedure TVideoForm.RegressiBtnClick(Sender: TObject);
begin
       calcEchelle;
       MesureEnCours := false;
       correctionRS.x0 := pointRepere[bsOrigine].X;
       correctionRS.y0 := pointRepere[bsOrigine].Y;
       correctionRS.RSvalue := RollingShutterSB.value;
       correctionRS.nmes := nmesVideo;
       correctionRS.NbrePoints := NbrePoints;
       correctionRS.hauteur := hauteur;
       correctionRS.echelle := echelle;
       correctionRS.angleX := 0;
       correctionRS.signeY := signeY;
       correctionRS.incertXY := round(incertSB.value);
       correctionRS.sauveData;
       NomFichierData := FProbe.FileName;
       ModifFichier := true;
       include(graphe.Modif,gmXY); // le + gnral
{$IFDEF Debug}
       EcritDebug('Sortie video dbut activeTab');
       FRegressiMain.AppelVideo := true;
{$ENDIF}
       FRegressiMain.windowTab.ActiveTab := FRegressiMain.grapheTab;
       FValeurs.feuillets.ActiveTab := Fvaleurs.VariabSheet;
{$IFDEF Debug}
       EcritDebug('Sortie video fin ActiveTab');
{$ENDIF}
       Close;
{$IFDEF Debug}
       EcritDebug('Sortie video Fin');
{$ENDIF}
end; // SauveData

procedure TVideoForm.FormActivate(Sender: TObject);
begin
    setBoutons;
end;

procedure TVideoForm.FormClose(Sender: TObject; var Action: TCloseAction);
var Rini : TMemIniFile;
begin
     Rini := TMemIniFile.create(registreReg);
     Rini.WriteString('Video','Echelle',EchelleEdit.text);
     Rini.WriteBool('Video','TracePoints',tracePointsCB.isChecked);
     Rini.WriteBool('Video','TraceEch',traceEchCB.isChecked);
     Rini.WriteInteger('Video','IncertXY',round(incertSB.Value));
     Rini.WriteInteger('Video','Loupe',round(loupeSB.value));
     Rini.WriteBool('Video','Trigo',trigoCB.isChecked);
     Rini.writeInteger('Video','NbrePoints',round(NbrePointsSB.value));
     Rini.writeInteger('Video','RollingShutter',round(RollingShutterSB.value));
     Rini.UpdateFile;
     Rini.Free;
     if pages[pageCourante].nmes=0 then FRegressiMain.FileNewSimulationItemClick(Sender);
     if FProbe<>nil then FProbe.Active := false;
     videRepertoire;
     if ChronoForm<>nil then ChronoForm.Free;
     cacheLoupe;
     setBorne(bsNone);
     timerPlay.Enabled := false;
     PlayBtn.Text := 'Play';
end;

procedure TVideoForm.FormCreate(Sender: TObject);

procedure CreerLoupe;
begin
{$IFDEF Debug}
      ecritDebug('CreerLoupe-entre');
{$ENDIF}
    if LoupeForm=nil then
       Application.CreateForm(TLoupeForm, LoupeForm);
    LoupeForm.aForm := videoForm;
    hauteurLoupe := LoupeForm.Width;
    demiHauteurLoupe := hauteurLoupe div 2;
    gainLoupe := 1;
    LoupeSB.Value := 1;
    LoupeForm.image.onClick := VideoClick;
    LoupeForm.image.onTap := VideoTap;
    LoupeForm.image.onMouseUp := VideoMouseUp;
    LoupeForm.image.onMouseDown := VideoMouseDown;
    LoupeForm.image.onMouseMove := VideoMouseMove;
    LoupeForm.onMouseUp := VideoMouseUp;
    LoupeForm.onMouseDown := VideoMouseDown;
    LoupeForm.onTap := VideoTap;
    LoupeForm.onMouseMove := VideoMouseMove;
    LoupeForm.Hide;
{$IFDEF Debug}
      ecritDebug('CreerLoupe-sortie');
{$ENDIF}
end;

procedure litIni;
var Rini : TmemInifile;
begin
{$IFDEF Debug}
      ecritDebug('LitIni-entre');
{$ENDIF}
     Rini := TMemIniFile.create(registreReg);
     EchelleEdit.text := Rini.ReadString('Video','Echelle','1');
     longueurEchelle := strToFloatWin(EchelleEdit.text);
     tracePointsCB.isChecked := Rini.ReadBool('Video','TracePoints',true);
     traceEchCB.isChecked := Rini.ReadBool('Video','TraceEch',true);
     incertSB.Value := Rini.ReadInteger('Video','IncertXY',1);
     loupeSB.value := Rini.ReadInteger('Video','Loupe',1);
     trigoCB.isChecked := Rini.ReadBool('Video','Trigo',true);
     if trigoCB.isChecked
          then signeY := -1
          else signeY := 1;
     NbrePointsSB.value := Rini.ReadInteger('Video','NbrePoints',1);
     NbrePoints := round(NbrePointsSB.Value);
     RollingShutterSB.value := Rini.ReadInteger('Video','RollingShutter',0);
     Rini.Free;
{$IFDEF Debug}
      ecritDebug('LitIni-sortie');
{$ENDIF}
end;

var i : TStyleDrag;
    j : indicePoint;
begin
  litIni;
  DureeImage := 1/30;
  LargeurAff := 640;
  HauteurAff := 480;
  Largeur := 640;
  Hauteur := 480;
  pasZoom := 1;
{$IFDEF Debug}
  //    ecritDebug('TAVProbe-Create');
{$ENDIF}
  FProbe := TAVProbe.Create;
  for j := 1 to NPointsMax do
      couleurPoint[j] := claRed;
  couleurPointCB.SelectedColor := claRed;
  couleurAxeCB.SelectedColor := claBlue;
  PointCourant := 1;
  borneSelect := bsNone;
{$IFDEF Debug}
   //   ecritDebug('Tbitmap-Create');
{$ENDIF}
  bitmapCourant := Tbitmap.create;
  for i := low(TstyleDrag) to High(TstyleDrag) do begin
           e_pointRepere[i].x := 0;
           e_pointRepere[i].y := 0;
  end;
  creerLoupe;
end;

procedure TVideoForm.FormDestroy(Sender: TObject);
begin
    FProbe.Free;
    bitmapCourant.Free;
    if LoupeForm<>nil then begin
        LoupeForm.close;
        LoupeForm.free;
        LoupeForm := nil;
    end;
end;

procedure TVideoForm.setBorne(aBorne : TStyleDrag);
begin
     borneSelect := aBorne;
     case BorneSelect of
         bsOrigine,bsEchelle1,bsEchelle2 : setCurseur(crSizeAll);
         bsMesure,bsChronoInit : setCurseur(crNone);
         else setCurseur(crDefault); // bsChrono, bsNone
     end;
     FRegressiMain.timerLoupe.enabled := BorneSelect in [bsMesure,bsEchelle1,bsEchelle2,bsOrigine,bsChronoInit];
end;

procedure TVideoForm.VideoMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
    convertitXY(sender,x,y);
    if (Button<>TMouseButton.mbLeft) or not appartientImage(x,y) then exit;
    if borneSelect=bsNone then
       setBorne(GetBorne(x,y,true));
    e_pointRepere[BorneSelect].x := x;
    e_pointRepere[BorneSelect].y := y;
    ecranVersImage;
    nouveauPoint := true;
end;

procedure TVideoForm.VideoMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single);

procedure Loupe;
var demiHauteur : integer;

procedure reperage;
var ecart : integer;
    Fpath : TPathData;
    largeurTrait : single;
begin
    ecart := hauteurLoupe div 3;
    FPath := TPathData.Create;
    largeurTrait := screen.width/1024;
    if largeurTrait>5 then largeurTrait := 5;
    LoupeForm.bitmapLoupe.canvas.stroke.thickness := largeurTrait;
    LoupeForm.bitmapLoupe.canvas.stroke.color := couleurAxeCB.SelectedColor;
    LoupeForm.bitmapLoupe.canvas.stroke.dash := TStrokeDash.Solid;

    FPath.moveto(TPointF.Create(0,demiHauteurLoupe));
    FPath.lineto(TPointF.Create(ecart,demiHauteurLoupe));
    FPath.moveto(TPointF.Create(2*ecart,demiHauteurLoupe));
    FPath.lineto(TPointF.Create(hauteurLoupe,demiHauteurLoupe));

    FPath.moveto(TPointF.Create(demiHauteurLoupe,0));
    FPath.lineto(TPointF.Create(demiHauteurLoupe,ecart));
    FPath.moveto(TPointF.Create(demiHauteurLoupe,2*ecart));
    FPath.lineto(TPointF.Create(demiHauteurLoupe,hauteurLoupe));

    FPath.moveto(TPointF.Create(demiHauteurLoupe,demiHauteurLoupe-2));
    FPath.lineto(TPointF.Create(demiHauteurLoupe,demiHauteurLoupe+2));
    FPath.moveto(TPointF.Create(demiHauteurLoupe-2,demiHauteurLoupe));
    FPath.lineto(TPointF.Create(demiHauteurLoupe+2,demiHauteurLoupe));
    LoupeForm.bitmapLoupe.Canvas.DrawPath(FPath, 1);
    FPath.Free;
end;

var
   X1,Y1,X2,Y2 : single;
   decaleX,decaleY,margeX,margeY : integer;
   aPoint : TPointF;
   SourceRect,DestRect : TrectF;
begin  // loupe
   demiHauteur := demiHauteurLoupe div gainLoupe;
   X1 := X-demiHauteur;
   Y1 := Y-demiHauteur;
   X2 := X+demiHauteur;
   Y2 := Y+demiHauteur;
   if Y1<0 then begin
      decaleY := -round(Y1*gainLoupe);
      Y1 := 0;
      margeY := 0;
   end
   else begin
      decaleY := 0;
      margeY := hauteurLoupe - demiHauteur*2*gainLoupe;
   end;
   if X1<0 then begin
      decaleX := -round(X1*gainLoupe);
      X1 := 0;
      margeX := 0;
   end
   else begin
      decaleX := 0;
      margeX := hauteurLoupe - demiHauteur*2*gainLoupe;
   end;
   if Y2>hauteurAff then Y2 := hauteurAff;
   if X2>largeurAff then X2 := largeurAff;
   aPoint := OrigineImageToScreen; // position de imgPreview
   aPoint.offset(x,y); // position / imgPreview
   LoupeForm.left := round(aPoint.X-demiHauteurLoupe);
   LoupeForm.top := round(aPoint.Y-demiHauteurLoupe);
   sourceRect := TrectF.create(X1/pasZoom,Y1/pasZoom,X2/pasZoom,Y2/pasZoom);
   destRect := TrectF.create(decaleX,decaleY,
                  decaleX+(X2-X1)*gainLoupe+margeX,
                  decaleY+(Y2-Y1)*gainLoupe+margeY);
   LoupeForm.bitmapLoupe.canvas.beginScene;
   LoupeForm.bitmapLoupe.Canvas.Clear(claNull);
   LoupeForm.bitmapLoupe.canvas.DrawBitmap(bitmapCourant,
             sourceRect,destRect,1);
   Reperage;
   LoupeForm.bitmapLoupe.canvas.endScene;
   loupeForm.avecLoupe := true;
   if not loupeForm.visible then loupeForm.show;
   LoupeForm.Active := true;
   FRegressiMain.timerLoupe.enabled := true;
   setCurseur(crNone);
   loupeForm.Image.invalidateRect(destRect);
   loupeForm.Image.Repaint;
end; // loupe

var BS : TstyleDrag;
begin  // MouseMove
convertitXY(sender,x,y);
if not appartientImage(x,y) then begin
   cacheLoupe;
   setCurseur(crDefault);
   exit;
end;
imgPreview.OnMouseMove := nil;
loupeForm.OnMouseMove := nil;
loupeForm.image.OnMouseMove := nil; // mise  jour longue ...
e_pointRepere[BorneSelect].x := x;
e_pointRepere[BorneSelect].y := y;
ecranVersImage;
case BorneSelect of
     bsEchelle1,bsEchelle2,bsOrigine : afficheImage(trackBar.value); // retrace
     bsMesure,bsChrono,bsChronoInit : ;
     bsNone : begin
         BS := GetBorne(x,y,false);
         if BS in [bsEchelle1,bsEchelle2,bsOrigine]
             then setCurseur(crSizeAll)
             else setCurseur(crDefault);
     end;
end;
if (borneSelect in [bsMesure,bsChronoInit,bsOrigine,bsEchelle1,bsEchelle2])
      then Loupe
      else cacheLoupe;
imgPreview.OnMouseMove := VideoMouseMove;
loupeForm.image.OnMouseMove := VideoMouseMove;
loupeForm.OnMouseMove := VideoMouseMove;
end;

procedure TVideoForm.VideoMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
var xinit,yinit : single;
begin
  xinit := x;
  yinit := y;
  convertitCibleXY(sender,x,y);
  if (Button<>TMouseButton.mbLeft) or not appartientImage(x,y) then exit;
  e_pointRepere[BorneSelect].x := x;
  e_pointRepere[BorneSelect].y := y;
  ecranVersImage;
  case BorneSelect of
     bsNone : ;
     bsEchelle1,bsEchelle2,bsOrigine : begin
         setBorne(bsNone);
         afficheImage(trackBar.value);
     end;
     bsChrono : ;
     bsMesure : begin
        EnregistrePoint(sender,Shift,xinit,yinit);
        nouveauPoint := false;
     end;
     bsChronoInit : InitChronoPhoto;
  end;
end;

(*
procedure TVideoForm.TimerMoveTimer(Sender: TObject);
begin
   imgPreview.OnMouseMove := VideoMouseMove;
   loupeForm.image.OnMouseMove := VideoMouseMove;
   loupeForm.Active := true;
   timerMove.enabled := false;
end;
*)

procedure TVideoForm.TimerLoupeTimer(Sender: TObject);
begin
          if (loupeForm<>nil) and
          loupeForm.visible and
          not loupeForm.avecSouris then
            loupeForm.aForm.Active := true
end;

procedure TVideoForm.TimerPlayTimer(Sender: TObject);
begin
    if trackBar.value>=trackBar.Max
       then begin
          timerPlay.Enabled := false;
          trackBar.Value := 0;
       end
       else acNextExecute(sender);
end;

procedure TVideoForm.TraceEchCBChange(Sender: TObject);
begin
    afficheImage(trackBar.value)
end;

procedure TVideoForm.TracePointsCBChange(Sender: TObject);
begin
    afficheImage(trackBar.value)
end;

procedure TVideoForm.TrackBarChange(Sender: TObject);
begin
     if (TrackBar.Tag = 0) and (FProbe<>nil) and FProbe.Active
       then afficheImage(trackBar.Value);
end;

procedure TVideoForm.AfficheImage(index : single);
var nomImage : string;
begin
    nomImage := getNomImage(index);
    if (nomImage='') or not FileExists(nomImage)
       then begin
           FProbe.TimeStamp := Round(index*dureeImage*AV_TIME_BASE);
           sauveImage(index);
       end
       else updateImage(nomImage);
end;

procedure TVideoForm.ImageVersEcran;
var i : TstyleDrag;
begin
       for i := low(TstyleDrag) to High(TstyleDrag) do
           e_pointRepere[i] := pointRepere[i]*pasZoom;
end;

procedure TVideoForm.VideoTap(Sender: TObject; const Point: TPointF);
begin
    VideoClick(sender);
end;

procedure TVideoForm.VideoClick(Sender: TObject);
begin
 case BorneSelect of
     bsNone : ;
     bsEchelle1,bsEchelle2,bsOrigine : begin
         setBorne(bsNone);
         dessine;
     end;
     bsMesure : ;
  end;
end;

procedure TVideoForm.EchelleEditKeyDown(Sender: TObject; var Key: Word;
  var KeyChar: Char; Shift: TShiftState);
begin
  if not CharInSet(keyChar,['0'..'9','.',',','m',
                     crSupprArr,crCR,crTab,crGauche,crDroite]) then keyChar := #0
end;

procedure TVideoForm.EcranVersImage;
var i : TstyleDrag;
begin
       for i := low(TstyleDrag) to High(TstyleDrag) do
           pointRepere[i] := e_pointRepere[i]/pasZoom;
end;

procedure TVideoForm.calcEchelle;
var EchelleEnPixel : double;
begin
     if (FProbe=nil) or not FProbe.Active then exit;
     echelleEnPixel := PointRepere[bsEchelle1].distance(PointRepere[bsechelle2]);
     try
        LongueurEchelle := StrToFloatWin(EchelleEdit.text);
     except
         showMessage('Erreur lors de la lecture de l''chelle');
         EchelleEdit.text := FormatReg(LongueurEchelle);
         EchelleEdit.SetFocus;
     end;
     Echelle := LongueurEchelle/EchelleEnPixel;
end;

procedure TVideoForm.CouleurAxeCBColorSelected(Sender: TObject;
  AColor: TAlphaColor);
begin
    dessine; // mise  jour
end;

procedure TVideoForm.CouleurAxeCBEChange(Sender: TObject);
begin
     dessine
end;

procedure TVideoForm.CouleurPointCBColorSelected(Sender: TObject;
  AColor: TAlphaColor);
begin
   couleurPoint[PointCourant] := Acolor; // or System.UITypes.TAlphaColorRec.Alpha
   dessine;
end;

procedure TVideoForm.SetCurseur(Acursor : Tcursor);
begin
   imgPreview.Cursor := aCursor;
end;

procedure TVideoForm.EnregistrePoint(Sender: TObject;Shift: TShiftState;x,y : single);
begin
         correctionRS.XY[PointCourant,NmesVideo] := pointRepere[bsMesure];
         inc(PointCourant);
         correctionRS.Temps[NmesVideo] := nmesVideo*dureeImage;
         if PointCourant>NbrePoints then begin
            inc(NmesVideo);
            PointCourant := 1;
         end;
         RegressiBtn.enabled := nmesVideo>2;
         if PointCourant=1 then if trackBar.value<trackBar.Max
                then begin
                   acNextExecute(nil);
                   VideoMouseMove(sender,shift,x,y);
                   editBidon.setFocus;
                end
                else begin
                   setBorne(bsNone);
                   MesureEnCours := false;
                   setBoutons;
                   dessine;
                end;
end;

function TVideoForm.AppartientImage(var X, Y: single) : boolean;
begin
    result := (x>0) and (y>0) and (y<hauteurAff) and (x<largeurAff)
end;

procedure TVideoForm.DroiteBtnClick(Sender: TObject);
var nomImage : string;
    signe : integer;
begin
    if (FProbe=nil) or not FProbe.Active then exit;
    if sender=droiteBtn then signe := 1 else signe := -1;
    rotation := rotation+signe;
    swap(largeur,hauteur);
    droiteBtn.Enabled := rotation<1;
    gaucheBtn.Enabled := rotation>-1;
    effectuerZoom;
    if PointRepere[bsOrigine].X>Largeur/2 then PointRepere[bsOrigine].X := Largeur/2;
    if PointRepere[bsOrigine].Y>Hauteur/2 then PointRepere[bsOrigine].Y := Hauteur/2;
    if PointRepere[bsEchelle1].X>Largeur then PointRepere[bsEchelle1].X := Largeur/4;
    if PointRepere[bsEchelle1].Y>Hauteur then PointRepere[bsEchelle1].Y := 3*Hauteur/4;
    if PointRepere[bsEchelle2].X>Largeur then PointRepere[bsEchelle2].X := 3*Largeur/4;
    if PointRepere[bsEchelle2].Y>Hauteur then PointRepere[bsEchelle2].Y := 3*hauteur/4;
    nomImage := getNomImage(trackBar.Value);
    if (nomImage='') or not FileExists(nomImage)
        then begin
           FProbe.TimeStamp := Round(trackBar.Value*dureeImage*AV_TIME_BASE);
           sauveImage(trackBar.Value);
        end
        else updateImage(nomImage);
end;

Function TVideoForm.GetBorne(x,y : single;deplace : boolean) : TstyleDrag;
var i : TstyleDrag;
    aPoint : TPointF;
begin
      result := bsNone;
      aPoint := PointF(x,y);
      for i in [bsOrigine,bsEchelle1,bsEchelle2] do
          if e_PointRepere[i].distance(aPoint) < Magnet
              then begin
                 result := i;
                 exit;
              end;
end;

procedure TVideoForm.EffectuerZoom;
var zoomX,zoomY,zoom : single;
begin
    if (FProbe=nil) or not FProbe.Active then exit;
    zoomX := panelPreview.width / largeur;
    zoomY := panelPreview.height / hauteur;
    if zoomX>zoomY then zoom := zoomY else zoom := zoomX;
    if zoom>1 then pasZoom := floor(zoom) else pasZoom := zoom;
    hauteurAff := round(hauteur*pasZoom);
    largeurAff := round(largeur*pasZoom);
    ImgPreview.Height := hauteurAff;
    ImgPreview.Width := largeurAff;
    imageVersEcran;
end;

procedure TVideoForm.MesureBtnClick(Sender: TObject);
begin
  mesureEnCours := not mesureEnCours;
  if MesureEnCours
  then begin
     setBorne(bsMesure);
     NmesVideo := 0;
     PointCourant := 1;
     MesureBtn.text := 'STOP';
     editBidon.SetFocus;
  end
  else begin
     setBorne(bsNone);
     MesureBtn.text := 'Mesurer';
  end;
  setBoutons;
end;

procedure TVideoForm.NbrePointsSBChange(Sender: TObject);
begin
    NbrePoints := round(NbrePointsSB.value);
end;

procedure TVideoForm.DebutBtnClick(Sender: TObject);
begin
    trackBar.value := 0;
end;

procedure TVideoForm.Dessine;
const tailleCroix = 3;
var imageCanvas : TCanvas;

Procedure TraceCroix(Apoint : TpointF);
var point1,point2 : TPointF;
begin
       point1 := Apoint;
       point2 := Apoint;
       point1.Offset(-tailleCroix,0);
       point2.Offset(+tailleCroix,0);
       imageCanvas.drawLine(point1,point2,1);
       point1 := Apoint;
       point2 := Apoint;
       point1.offset(0,-tailleCroix);
       point2.offset(0,+tailleCroix);
       imageCanvas.drawLine(point1,point2,1);
end;

Procedure TraceEchelle;
begin
      imageCanvas.drawLine(PointRepere[bsEchelle1],PointRepere[bsechelle2],1);
      TraceCroix(PointRepere[bsEchelle1]);
      TraceCroix(PointRepere[bsechelle2]);
end;

Procedure TraceOrigine;

procedure TraceFleche(dX,dY : single);
    var
      PC: TPointF; // centre de la base de la flche
      deltaX, deltaY: single;
      Points:  TPolygon; // Flche
      PE : TPointF; // extrmit de l'axe
      rayon : single;
    begin
       rayon := min(largeur,hauteur)/3;
       PE := PointRepere[bsOrigine];
       deltaX := rayon*dX;
       deltaY := rayon*dY;
       PE.offset(deltaX,deltaY);
       imageCanvas.drawLine(PointRepere[bsOrigine],PE,1);
       rayon := rayon/10;
       PC := PE;
       deltaX := rayon*dX;
       deltaY := rayon*dY;
       PC.Offset(-deltaX,-deltaY);
       deltaX := deltaX/3;
       deltaY := deltaY/3;
       setLength(Points,3);
       Points[0] := PE;
       PC.offset(-deltaY,+deltaX);
       Points[1] := PC;
       PC.offset(+2*deltaY,-2*deltaX);
       Points[2] := PC;
       imageCanvas.DrawPolygon(Points,1);
    end;

begin // traceOrigine
         imageCanvas.stroke.color :=  couleurAxeCB.SelectedColor;
         imageCanvas.stroke.color :=  claBlue;
         TraceFleche(0,signeY);
         TraceFleche(1,0);
end;

var i : integer;
    j : indicePoint;
begin // Dessine
    if (FProbe=nil) or not FProbe.Active then exit;
    if (borneSelect in [bsChrono,bsChronoInit]) then exit;
    imageCanvas := ImgPreview.bitmap.Canvas;
    ImageCanvas.BeginScene;
    imageCanvas.stroke.thickness := 1/pasZoom;
    imageCanvas.stroke.kind := TBrushKind.solid;
    couleurAxeCB.Visible := traceEchCB.isChecked;
    if traceEchCB.isChecked then begin
       TraceOrigine;
       TraceEchelle;
    end;
    couleurPointCB.Visible := tracePointsCB.isChecked;
    if tracePointsCB.isChecked then begin
       for i := 0 to nmesVideo-1 do begin
           for j := 1 to NbrePoints do begin
              imageCanvas.stroke.color := CouleurPoint[j]; // or System.UITypes.TAlphaColorRec.Alpha;
              TraceCroix(correctionRS.XY[j,i]);
           end;
       end;
       for j := 1 to PointCourant do begin
           imageCanvas.stroke.color := CouleurPoint[j]; // or System.UITypes.TAlphaColorRec.Alpha;
           TraceCroix(correctionRS.XY[j,pred(nmesVideo)]);
       end;
    end; // tracePoints
    ImageCanvas.EndScene;
end; // Dessine


procedure TVideoForm.setBoutons;
var OK : boolean;
begin
{$IFDEF Debug}
      ecritDebug('setBoutons-entre');
{$ENDIF}
    OK := (FProbe<>nil) and FProbe.Active;
    EchelleEdit.enabled := OK;
    PlayBtn.Enabled := OK;
    trackBar.Enabled := OK;
    RegressiBtn.enabled := nmesVideo>2;
    NextBtn.Enabled := OK and (trackBar.value<trackBar.Max);
    PreviousBtn.Enabled := OK and (trackBar.value>trackBar.Min);
    MesureBtn.enabled := OK;
    RazBtn.enabled := OK and not MesureEnCours and (nmesVideo>2);
    if MesureEnCours
       then mesureBtn.text := 'Stop'
       else mesureBtn.text := 'Mesurer';
    if (borneSelect in [bsNone,bsChrono]) then begin
       cacheLoupe;
       setCurseur(crDefault);
    end;
    if ChronoPhotoEnCours
       then ChronoBtn.Text := 'Stop chrono'
       else ChronoBtn.Text := 'ChronoPhoto';
{$IFDEF Debug}
      ecritDebug('setBoutons-sortie');
{$ENDIF}
end; // setBoutons

procedure TVideoForm.SauveImage(index : single);
var nomImage : string;
begin
   bitmapCourant.Assign(FProbe.Bitmap);
   if rotation<>0 then bitmapCourant.Rotate(rotation*90);
   imgPreview.Bitmap.Assign(bitmapCourant);
   trackbar.Tag := 1;
   trackBar.Value := index;
   trackbar.Tag := 0;
  {$IFDEF DEBUG}   {$ENDIF}
   nomImage := getNomImage(index);
   if (nomImage<>'') and not FileExists(nomImage)
        then bitmapCourant.SaveToFile(nomImage);
   // PixelFormat=BGRA
end;

function GetScanline(ABitmap: TBitmap; ligne : integer): PAlphaColorArray;
var
  bitdata1: TBitmapData;
begin
    if ABitmap.Map(TMapAccess.Read, bitdata1) then begin
    try
      result := bitdata1.getScanLine(ligne);
    finally
      ABitmap.Unmap(bitdata1);
    end;
    end
    else result := nil;
end;

procedure TVideoForm.AffecteChrono;
var couleurC,couleurInit : TAlphaColorRec;
    scanLineC,scanLineInit : pAlphaColorArray;
    xPrec,yPrec : integer;
    xAttendu,yAttendu : integer;
    DeltaX,DeltaY : integer;
    x0,y0 : integer;
    ecartMin : integer;

procedure EnregistrePointChrono;
// on affine autour de x0, y0
var xMin,xMax : integer;
    yMin,yMax : integer;
    xc,yc,x1,y1,x2,y2 : integer;
    ecartCible,ecart : integer;
    largeurCible,hauteurCible : integer;
begin
    largeurCible := largeur div 24; // zone de recherche
    hauteurCible := hauteur div 24;
    correctionRS.XY[1,NmesVideo].x := x0;  // par dfaut
    correctionRS.XY[1,NmesVideo].y := y0;
    xMin := x0+hauteurCible;
    xMax := x0-hauteurCible;
    yMin := y0+largeurCible;
    yMax := y0-largeurCible;
    y1 := yMax;y2 := yMin;
    if y1<0 then y1 := 0;
    if y2>=hauteur then y2 := pred(hauteur);
    x1 := xMax;x2 := xMin;
    if x1<0 then x1 := 0;
    if x2>=largeur then x2 := pred(largeur);
    for yc := y1 to y2 do begin
        scanLineC := getScanLine(bitmapCourant,yc);
        scanLineInit := getScanLine(bitmapInit,yc);
        for xc := x1 to x2 do begin
            couleurC := TAlphaColorRec(scanLineC[xc]);
            couleurInit := TAlphaColorRec(scanLineInit[xc]);
            ecart := abs(integer(couleurInit.B)-
                         integer(couleurC.B))+
                     abs(integer(couleurInit.G)-
                     integer(couleurC.G))+
                     abs(integer(couleurInit.R)-
                     integer(couleurC.R));
            ecartCible := abs(BlueCibleChrono-integer(couleurC.B))+
                          abs(GreenCibleChrono-integer(couleurC.G))+
                          abs(RedCibleChrono-integer(couleurC.R));
            if ecartCible<resolution then setPixel(bitmapFin,xc,yc,TAlphaColor(couleurC));
            if (ecart>resolution) and (ecartCible<ecartMin) then begin
            // diffrent de l'tat initial et proche couleur cible
               if xc<xMin then xMin := xc;
               if yc<yMin then yMin := yc;
               if xc>xMax then xMax := xc;
               if yc>yMax then yMax := yc;
            end;
        end;// for xc
    end;// for yc
    correctionRS.Temps[NmesVideo] := trackBar.value*dureeImage;
    correctionRS.XY[1,NmesVideo].x := (xMin+xMax) div 2;
    correctionRS.XY[1,NmesVideo].y := (yMin+yMax) div 2;
    correctionRS.Temps[NmesVideo] := trackbar.value*dureeImage;
    inc(NmesVideo);
end; // EnregistrePointChrono

function chercheMobile : boolean;
var resolutionCible : integer;
    ecart,ecartCible : integer;
    xc,yc : integer;
    procheAttendu,prochePrec : boolean;
    ligneProcheAttendu,ligneProchePrec : boolean;
begin
resolutionCible := resolutionCibleMin;
repeat
    EcartMin := resolutionCible;
    for yc := 0 to pred(hauteur) do begin
        ligneProcheAttendu := abs(yc-yAttendu)<deltaY;
        ligneProchePrec := abs(yc-yPrec)<deltaY;
        if ligneProcheAttendu or ligneProchePrec then begin
           scanLineC := getScanLine(bitmapCourant,yc);
           scanLineInit := getScanLine(bitmapInit,yc);
           for xc := 0 to pred(largeur) do begin
              procheAttendu := (abs(xc-xAttendu)<deltaX) and ligneProcheAttendu;
              prochePrec := (abs(xc-xPrec)<deltaX) and ligneProchePrec;
              if (procheAttendu or prochePrec) then begin
                 couleurC := TAlphaColorRec(scanLineC[xc]);
                 couleurInit := TAlphaColorRec(scanLineInit[xc]);
                 ecart := abs(integer(couleurInit.B)-integer(couleurC.B))+
                          abs(integer(couleurInit.G)-integer(couleurC.G))+
                          abs(integer(couleurInit.R)-integer(couleurC.R));
                 if (ecart>resolution)  then begin
                 // On cherche l o il y eu changement
                    ecartCible := abs(BlueCibleChrono-integer(couleurC.B))+
                                  abs(GreenCibleChrono-integer(couleurC.G))+
                                  abs(RedCibleChrono-integer(couleurC.R));
                    if ecartCible<ecartMin then begin // couleur proche couleurCible
                       ecartMin := ecartCible;
                       x0 := xc;y0 := yc;
                    end;
                 end;  // changement
              end; // point proche
           end;// i
        end;// ligne proche
    end;// j
    result := (x0>0) and (y0>0);
    inc(resolutionCible,pasCouleur);
until result or (resolutionCible>resolutionCibleMax);
end; // chercheMobile

function chercheFixe : boolean;
var ecartCible : integer;
    xc,yc : integer;
    resolutionCible : integer;
    xMin,xMax : integer;
    yMin,yMax : integer;
begin
    resolutionCible := resolutionCibleMin;
    xMin := xPrec-deltaX;
    xMax := xPrec+deltaX;
    yMin := yPrec-deltaY;
    yMax := yPrec+deltaY;
    if yMin<0 then yMin := 0;
    if yMax>=hauteur then yMax := pred(hauteur);
    if xMin<0 then xMin := 0;
    if xMax>=largeur then xMax := pred(largeur);
    repeat
    EcartMin := resolutionCible;
    for yc := ymin to ymax do begin
        scanLineC := getScanLine(bitmapCourant,yc);
        for xc := xmin to xmax do begin
            couleurC := TAlphaColorRec(scanLineC[xc]);
            ecartCible := abs(BlueCibleChrono-integer(TAlphaColorRec(couleurC).B))+
                          abs(GreenCibleChrono-integer(TAlphaColorRec(couleurC).G))+
                          abs(RedCibleChrono-integer(TAlphaColorRec(couleurC).R));
            if ecartCible<ecartMin then begin // couleur proche couleurCible
               ecartMin := ecartCible;
               x0 := xc;y0 := yc;
            end;
        end;// for xc
    end;// for yc
    result := (x0>0) and (y0>0);
    inc(resolutionCible,pasCouleur);
until result or (resolutionCible>resolutionCibleMax);
end; // chercheFixe

var
    trouve : boolean;
begin // affecteChrono
    timerPlay.Enabled := false;
    DeltaX := largeur div 16; // zone de recherche
    DeltaY := hauteur div 16;
    x0 := 0;
    y0 := 0;
    xPrec := round(correctionRS.XY[1,pred(nmesVideo)].x);
    yPrec := round(correctionRS.XY[1,pred(nmesVideo)].y);
    if nmesVideo>1 then begin // Nouvelle position en supposant la vitesse constante
       xAttendu := 2*xPrec-round(correctionRS.XY[1,nmesVideo-2].x);
       yAttendu := 2*yPrec-round(correctionRS.XY[1,nmesVideo-2].y);
    end
    else begin
       xAttendu := xPrec;
       yAttendu := yPrec;
    end;
    trouve := chercheMobile; // on cherche l o l'image a chang
    if not trouve then trouve := chercheFixe; // on cherche la couleur de la cible
    if trouve then begin
       inc(ecartMin,pasCouleur);
       enregistrePointChrono;
       if trackBar.value<trackBar.Max
          then timerPlay.Enabled := true
          else saveChronoPhoto;
    end
    else saveChronoPhoto;
end; // AffecteChrono

procedure TVideoForm.trigoCBChange(Sender: TObject);
begin
       if trigoCB.isChecked
          then signeY := -1
          else signeY := 1;
       e_pointRepere[bsAxeY].x := 2*e_pointRepere[bsOrigine].x-e_pointRepere[bsAxeY].x;
       e_pointRepere[bsAxeY].y := 2*e_pointRepere[bsOrigine].y-e_pointRepere[bsAxeY].y;
       dessine;
end;


procedure TVideoForm.SaveChronoPhoto;
begin
       setBorne(bsNone);
       bitmapFin.saveToFile(nomFichierChrono);
       bitmapFin.free;
       bitmapInit.free;
       if ChronoForm=nil then
          ChronoForm := TChronoForm.Create(self);
       ChronoForm.image.bitmap.loadFromFile(NomFichierChrono);
       if not chronoForm.Visible then ChronoForm.show;
       TimerPlay.Enabled := false;
       ChronoPhotoEnCours := false;
       setBoutons;
end;

procedure TVideoForm.ChronoBtnClick(Sender: TObject);
begin
    if ChronoPhotoEnCours
    then begin // fin chrono
         if (borneSelect=bsChrono) and (trackBar.value>10)
            then saveChronoPhoto;
         setBorne(bsNone);
    end
    else begin // dbut chrono
        if saveDialog.Execute
           then begin
              nomFichierChrono := saveDialog.FileName;
              if (NbrePoints>1) then begin
                  showMessage('Un seul mobile');
                  NbrePointsSB.value := 1;
                  NbrePoints := 1;
              end;
              showMessage('Cliquer sur la cible  suivre');
              setBorne(bsChronoInit);
           end
           else setBorne(bsNone);
    end;
    TimerPlay.enabled := false;
    ChronoPhotoEnCours :=  BorneSelect = bsChronoInit;
    if ChronoPhotoEnCours
       then ChronoBtn.Text := 'Stop chrono'
       else ChronoBtn.Text := 'ChronoPhoto';
end;

procedure TVideoForm.InitChronoPhoto;

procedure InitMesure;
var
    i,j : integer;
    couleur : TAlphaColorRec;
    scanLine : pAlphaColorArray;
    xCibleChrono,yCibleChrono : integer;
begin
     PointCourant := 1;
     correctionRS.Temps[0] := trackBar.value*dureeImage;
     correctionRS.XY[1,0] := pointRepere[bsChronoInit];
     NmesVideo := 1;
     xCibleChrono := round(pointRepere[bsChronoInit].x);
     yCibleChrono := round(pointRepere[bsChronoInit].y);
     RedCibleChrono := 0;
     BlueCibleChrono := 0;
     GreenCibleChrono := 0;
     for j := yCibleChrono-1 to yCibleChrono+1 do begin
          scanLine := getScanLine(bitmapInit,j);
          for i := xCibleChrono-1 to xCibleChrono+1 do begin
            couleur := TAlphaColorRec(scanLine[i]);
            RedCibleChrono := RedCibleChrono+integer(couleur.R);
            BlueCibleChrono := BlueCibleChrono+integer(couleur.B);
            GreenCibleChrono := GreenCibleChrono+integer(couleur.G);
          end;
     end;
     RedCibleChrono := RedCibleChrono div 9;
     BlueCibleChrono := BlueCibleChrono div 9;
     GreenCibleChrono := GreenCibleChrono div 9;
     setBoutons;
end;

begin
     bitmapInit := Tbitmap.create;
     bitmapFin := Tbitmap.create;
     bitmapInit.assign(bitmapCourant);
     bitmapFin.assign(bitmapCourant);
     setBorne(bsChrono);
     initMesure;
     timerPlay.enabled := true;
end; // InitChronoPhoto

procedure TVideoForm.LoupeSBChange(Sender: TObject);
begin
    gainLoupe := round(loupeSB.value)
end;

 function TVideoForm.getNomImage(index : single) : string;
  var nomL : string;
      indexInt : integer;
  begin
     result := '';
     if (index<TrackBar.Min) or (index>TrackBar.max) then exit;
     indexInt := round(index);
     nomL := system.SysUtils.format('test%3d.bmp',[indexInt]);
     result := System.IOUtils.Tpath.combine(supportPath,nomL);
  end;

  procedure TVideoForm.updateImage(nomImage : string);
  begin
    if (FProbe=nil) or not FProbe.Active then exit;
    bitmapCourant.loadFromFile(nomImage);
    if rotation<>0 then bitmapCourant.Rotate(rotation*90);
    imgPreview.Bitmap.Assign(bitmapCourant);
 //   SetImageOpaque(imgPreview,largeur,hauteur);
    case borneSelect of
       bsChrono : AffecteChrono;
       bsChronoInit : ;
       else dessine;
    end;
    NextBtn.Enabled := true;
    PreviousBtn.Enabled := true;
  end; // updateImage

(*
initialization

{$IFDEF MSWINDOWS}
   FFMPEG_DLL_PATH:='\\Mac\Home\Documents\Embarcadero\Studio\Projets\ffmpegfmx\Bin\Win64\';
{$ELSE}
   FFMPEG_DLL_PATH:='/Users/jean-michelmillet/Documents/Embarcadero/Studio/Projets/ffmpegfmx/Bin/OSX64/';
   FFMPEG_DLL_PATH:='./';
{$ENDIF}
*)

end.



