unit NSApplicationOpenFileDelgateUnit.mac;

interface

type

  TOpenFileEvent = reference to procedure(const AFileName: string);

procedure InstallApplicationDelegate2(const AOnOpenFile: TOpenFileEvent);

implementation

uses
  System.SysUtils, System.RTLConsts, System.Messaging, System.Classes,
  Macapi.ObjectiveC, Macapi.CoreFoundation, Macapi.CocoaTypes, Macapi.AppKit,
  Macapi.Foundation, FMX.Forms,
  Macapi.ObjCRuntime,
  FMX.Platform, FMX.Platform.Mac, FMX.Helpers.Mac;

type
  NSApplicationDelegate2 = interface(NSApplicationDelegate)
    ['{BE9AEDB7-80AC-49B1-8921-F226CC9310F4}']
    function application(theApplication: Pointer; openFile: CFStringRef)
      : Boolean; cdecl;
  end;

  TNSApplicationDelegate2 = class(TOCLocal, NSApplicationDelegate2)
  private
    FOnOpenFile: TOpenFileEvent;
  public
    constructor Create(const AOnOpenFile: TOpenFileEvent);
    procedure applicationDidFinishLaunching(Notification
      : NSNotification); cdecl;
    procedure applicationDidHide(Notification: NSNotification); cdecl;
    procedure applicationDidUnhide(Notification: NSNotification); cdecl;
    function applicationShouldTerminate(Notification: NSNotification)
      : NSInteger; cdecl;
    function applicationDockMenu(sender: NSApplication): NSMenu; cdecl;
    procedure applicationWillTerminate(Notification: NSNotification); cdecl;
    function application(theApplication: Pointer; openFile: CFStringRef)
      : Boolean; cdecl;
  end;

constructor TNSApplicationDelegate2.Create(const AOnOpenFile: TOpenFileEvent);
begin
  inherited Create;
  FOnOpenFile := AOnOpenFile;
end;

procedure TNSApplicationDelegate2.applicationDidFinishLaunching
  (Notification: NSNotification); cdecl;
begin
  // Seems we have to have this method even though it is empty.
end;

procedure TNSApplicationDelegate2.applicationDidHide(Notification
  : NSNotification); cdecl;
begin
  // Seems we have to have this method even though it is empty.
end;

procedure TNSApplicationDelegate2.applicationDidUnhide
  (Notification: NSNotification); cdecl;
begin
  // Seems we have to have this method even though it is empty.
end;

function TNSApplicationDelegate2.applicationShouldTerminate
  (Notification: NSNotification): NSInteger; cdecl;
begin
  Result := NSTerminateNow;
end;

function TNSApplicationDelegate2.applicationDockMenu(sender: NSApplication)
  : NSMenu; cdecl;
begin
  Result := nil;
end;

procedure TNSApplicationDelegate2.applicationWillTerminate
  (Notification: NSNotification); cdecl;
begin
  FreeAndNil(FMX.Forms.application);
end;

function TNSApplicationDelegate2.application(theApplication: Pointer;
  openFile: CFStringRef): Boolean; cdecl;
var
  Range: CFRange;
  S: String;
begin
  Result := Assigned(FOnOpenFile);
  if not Result then
    Exit;
  Range.location := 0;
  Range.length := CFStringGetLength(openFile);
  SetLength(S, Range.length);
  CFStringGetCharacters(openFile, Range, PChar(S));
  try
    FOnOpenFile(S);
  except
    on E: Exception do
    begin
      FMX.Forms.application.HandleException(E);
      Result := False;
    end;
  end;
end;

var
  Delegate: NSApplicationDelegate2;

procedure InstallApplicationDelegate2(const AOnOpenFile: TOpenFileEvent);
var
  NSApp: NSApplication;
begin
  NSApp := TNSApplication.Wrap(TNSApplication.OCClass.sharedApplication);
  Delegate := TNSApplicationDelegate2.Create(AOnOpenFile);
  NSApp.setDelegate(Delegate);
end;

end.

// inclure in fromCreate
// InstallApplicationDelegate2(OpenFile);

(*)
procedure TMyMainForm.OpenFile(const AFileName: String);
begin
// Application-specific code to open the file
end;
*)