unit WaveUnit;

interface

uses
  System.SysUtils,
  System.Types,
  System.UITypes,
  System.Classes,

  FMX.Types,
  FMX.Graphics,
  FMX.Controls,
  FMX.Forms,
  FMX.Dialogs,
  FMX.StdCtrls,
  FMX.Media,
  FMX.Layouts,
  FMX.Memo,
  FMX.Objects,
  FMX.Controls.Presentation,

  fft,
  images;

type
  TWaveForm = class(TForm)
    MediaPlayer1: TMediaPlayer;
    Timer1: TTimer;
    SaveDialog1: TSaveDialog;
    ToolBar1: TToolBar;
    recordBtn: TButton;
    StopPlayBtn: TButton;
    PlayBtn: TButton;
    StopRecordBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure recordBtnClick(Sender: TObject);
    procedure StopRecordBtnClick(Sender: TObject);
    procedure PlayBtnClick(Sender: TObject);
    procedure StopPlayBtnClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
 //   FractionLue : double;
    vecteurSon : TvecteurSon;
    pasReg : word;
    Procedure Envoie;
  public
    Mic: TAudioCaptureDevice;
    function OpenFile(Nom : String) : boolean;
  end;

var
  WaveForm: TWaveForm;

implementation

{$R *.fmx}

uses regmain, regutil, compile, maths;

function TWaveForm.OpenFile(Nom : String) : boolean;
begin
     try
     result := true;
     vecteurSon.lit(Nom);
     if vecteurSon.nbrePoints>0 then Envoie;
     except
     On E:exception do begin
       result := false;
       showMessage(E.message)
     end;
     end;
end; // openFile

procedure TWaveForm.FormCreate(Sender: TObject);
begin
    vecteurSon := TvecteurSon.create;
end;

procedure TWaveForm.FormDestroy(Sender: TObject);
begin
     vecteurSon.destroy;
end;

procedure TWaveForm.Envoie;
var dt : double;
    j,nbre : integer;
begin
     pasReg := succ(vecteurSon.NbrePoints div MaxMaxVecteur);
     if pageCourante=0 then begin
        ajouteExperimentale('t',variable);
        grandeurs[0].nomUnite := 's';
        if vecteurSon.stereo then begin
           ajouteExperimentale('sD',variable);
           ajouteExperimentale('sG',variable);
        end
        else  ajouteExperimentale('s',variable);
     end;
     ajoutePage;
     dt := pasReg/vecteurSon.freqAudio;
     pages[pageCourante].commentaireP := 'Acquisition  fech='+IntToStr(round(1/pasReg/dt))+' Hz';
     nbre := vecteurSon.NbrePoints div pasReg;
     pages[pageCourante].nmes := nbre;
     for j := 0 to nbre-1 do begin
         pages[pageCourante].valeurVar[0,j] := j*dt;
         pages[pageCourante].valeurVar[1,j] := vecteurSon.valeur[j*pasReg];
     end;
     if vecteurSon.stereo then  for j := 0 to nbre-1 do
         pages[pageCourante].valeurVar[2,j] := vecteurSon.valeurBis[j*pasReg];
     modeAcquisition := acqSon;
     FRegressiMain.FinOuvertureFichier(true);
end; // Envoie

procedure TWaveForm.recordBtnClick(Sender: TObject);
begin
  ///Get the default microphone
  Mic := TCaptureDeviceManager.Current.DefaultAudioCaptureDevice;
  if Mic <> nil then begin
    //Set the SaveDialog filter to choose only the supported extension
    SaveDialog1.Filter :=  Mic.FilterString;
    if SaveDialog1.Execute then begin
      RecordBtn.Enabled := false;
      StopRecordBtn.Enabled := true;
      StopPlayBtn.Enabled := false;
      PlayBtn.Enabled := false;
      //Gets the name of the file where to save the recorded data
      Mic.FileName := SaveDialog1.FileName;
      Mic.StartCapture;
    end;
  end
  else ShowMessage('Audio capturing device not available');
end;

procedure TWaveForm.StopRecordBtnClick(Sender: TObject);
begin
  if (Mic <> nil) and
     (Mic.State = TCaptureDeviceState.Capturing) then begin
    Mic.StopCapture;
    StopRecordBtn.Enabled := false;
    RecordBtn.Enabled := true;
    PlayBtn.Enabled := true;
  end;
end;

procedure TWaveForm.PlayBtnClick(Sender: TObject);
begin
// This application plays only recorded data, so if there is no data recorded there is nothing to play.
  if (Mic <> nil) and
     (Mic.State = TCaptureDeviceState.Stopped) then begin
    //Specifies the file to be played.
    MediaPlayer1.FileName := Mic.FileName;
    //Checks if the data can be decoded.
    if MediaPlayer1.Media <> nil then begin
      if (MediaPlayer1.State = TMediaState.Stopped) then begin
        Timer1.Enabled := true;
        StopPlayBtn.Enabled := true;
        StopRecordBtn.Enabled := false;
        RecordBtn.Enabled := false;
        MediaPlayer1.Play;
      end;
    end;
  end
  else ShowMessage('The audio device is still capturing');
end;

procedure TWaveForm.StopPlayBtnClick(Sender: TObject);
begin
  if MediaPlayer1.Media <> nil then begin
    if MediaPlayer1.State = TMediaState.Playing then begin
      MediaPlayer1.Stop;
      Timer1.Enabled:=false;
    end
    else begin
      MediaPlayer1.Play;
      Timer1.Enabled:=true;
    end;
  end;
end;

procedure TWaveForm.Timer1Timer(Sender: TObject);
begin
  if (MediaPlayer1.Media <> nil) and
    (MediaPlayer1.CurrentTime = MediaPlayer1.Duration) then begin
    StopPlayBtn.Enabled:=false;
    PlayBtn.Enabled := true;
    RecordBtn.Enabled := true;
    MediaPlayer1.Stop;
    MediaPlayer1.CurrentTime:=0;
  end;
end;

end.