unit arduinoOscMac;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  system.inifiles, system.SyncObjs, system.IOUtils, system.dateUtils,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.ListBox,
  FMX.StdCtrls, FMX.Edit, FMX.EditBox, FMX.SpinBox, FMX.Objects,
  FMX.Controls.Presentation,
  FMX.TMSBaseControl, FMX.TMSGridCell,
  FMX.TMSGridOptions, FMX.TMSGridData, FMX.TMSCustomGrid, FMX.TMSGrid,
  FMX.TMSToolBar, FMX.TMSBitmapContainer,
  Winsoft.OSX.ComPort,
//  Posix.unistd, // pour deleteFile
  arduinoGraphe, parser10, uniteker, math;

type
  tResolution = (r8bits,r10bits,r12bits,r14bits); // Due Zero 12 bits  ; 8 bits pour acclrer
  TArduinoOscilloForm = class(TForm)
    ToolBar1: TTMSFMXToolBar;
    toolsBtn: TTMSFMXToolBarButton;
    startBtn: TTMSFMXToolBarButton;
    regressiBtn: TTMSFMXToolBarButton;
    grid: TTMSFMXGrid;
    triggerTB: TTrackBar;
    PaintBox: TPaintBox;
    ToolBar2: TTMSFMXToolBar;
    Label1: TLabel;
    NbreVoiesSE: TSpinBox;
    singleCB: TCheckBox;
    Label2: TLabel;
    FechList: TComboBox;
    synchroCB: TComboBox;
    DureeLabel: TLabel;
    TMSFMXBitmapContainer1: TTMSFMXBitmapContainer;
    TMSFMXToolBarSeparator1: TTMSFMXToolBarSeparator;
    SynchroLabel: TLabel;
    TMSFMXToolBarSeparator2: TTMSFMXToolBarSeparator;
    TimerAcq: TTimer;
    TMSFMXToolBarSeparator3: TTMSFMXToolBarSeparator;
    StopBtn: TTMSFMXToolBarButton;
    ProgressBar: TProgressBar;
    TimerStop: TTimer;
    procedure toolsBtnClick(Sender: TObject);
    procedure startBtnClick(Sender: TObject);
    procedure regressiBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure NbreVoiesSEChange(Sender: TObject);
    procedure FechListChange(Sender: TObject);
    procedure synchroCBChange(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject; Canvas: TCanvas);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TimerAcqTimer(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure TimerStopTimer(Sender: TObject);
    procedure triggerTBChange(Sender: TObject);
  private
    VoieSerie: TXComPort;
    chaineLue : ansiString;
    resolution : Tresolution;
    liste : TstringList;
    grapheA : TgrapheArduino;
    Parser : Tparser;
    NbrePointsCourant : integer;
    NbreVoies : integer;
    SerialDevices: TstringList;
    NbrePointsArduino : integer;
    ValeurReal : array[0..2] of TvecteurArduino; // sortie CAN
    FechOK,NbreVoiesOK,synchroOK,seuilOK : boolean;
    uniteTemps : Tunite;
    procedure TriggerData(aChaine : ansiString);
    procedure getSerialDevices;
    function EnvoieCommande(aCommand : string) : boolean;
    procedure activeVoie;
    function periodeEch : integer; // en micro seconde
  public
  end;

var
  ArduinoOscilloForm: TArduinoOscilloForm;

implementation

{$R *.fmx}

uses arduinoOscCfg, compile, grapheVariabU, regutil, regmain;

const
  MaxIndexFrequence = 12;
  pauseRs232 = 25;
  MaxResol : array[Tresolution] of integer = (255,1023,4095,16383);

procedure TArduinoOscilloForm.GetSerialDevices;
var SerialDevicesXcom: TArray<string>;
    lName : string;
    posCu : integer;
    i : integer;
begin
  SerialDevicesXcom := VoieSerie.EnumDevices;
  SerialDevices.clear;
  if Length(SerialDevicesXcom) > 0 then begin
     for i := 0 to pred(Length(SerialDevicesXcom)) do begin
        lName := SerialDevicesXcom[i];
        posCu := pos('dev/cu',lName);
        if posCu>0 then SerialDevices.add(lName);
     end;
  end;
end;

procedure TArduinoOscilloForm.FechListChange(Sender: TObject);
var duree : double;
begin
    FechOK := false;
    duree := PeriodeEch*NbrePointsArduino;
    grapheA.monde[mondeX].maxiStr := FloatToStrF(duree,ffFixed,6,1); // en microseconde
    dureeLabel.text := '  Dure = '+uniteTemps.formatValeurEtUnite(duree/1e6);
    if voieSerie.Active then paintBox.Repaint;
    if stopBtn.enabled then begin
       StopBtnClick(sender);
       StartBtnClick(sender);
    end;
end;

procedure TArduinoOscilloForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
var Rini : TMemInifile;
    j,col : integer;
begin
     Rini := TMemIniFile.create(registreReg);
     Rini.WriteString('Arduino','Com',voieSerie.deviceName);
     Rini.WriteInteger('Arduino','Baud',ord(voieSerie.baudRate));
     Rini.WriteInteger('Arduino','Resolution',ord(resolution));
     Rini.WriteInteger('Arduino','NbreOscillo',NbrePointsArduino);
     Rini.WriteInteger('Arduino','OrdreSinc',grapheA.ordreSinc);
     Rini.WriteBool('Arduino','Sinc',grapheA.avecSinc);
     for j := 1 to 2 do begin
         col := j+1;
         if grid.cells[col,0]<>'' then
            Rini.writeString('Arduino','Nom'+intToStr(j),grid.cells[col,0]);
         if grid.cells[col,1]<>'' then
            Rini.writeString('Arduino','Unite'+intToStr(j),grid.cells[col,1]);
         if grid.cells[col,2]<>'' then
            Rini.writeString('Arduino','Convert'+intToStr(j),grid.cells[col,2]);
     end;
     Rini.writeInteger('Arduino','Fech',FechList.ItemIndex);
     Rini.writeInteger('Arduino','NVoies',round(NbreVoiesSE.value));
     Rini.writeInteger('Arduino','Synchro',SynchroCB.itemIndex);
     Rini.writeBool('Arduino','Monocoup',singleCB.isChecked);
     Rini.UpdateFile;
     Rini.Free;
     timerAcq.enabled := false;
     VoieSerie.active := false;
     if pages[pageCourante].nmes=0 then FRegressiMain.FileNewSimulationItemClick(Sender);
end;

procedure TArduinoOscilloForm.FormCreate(Sender: TObject);
const nomDefaut : array[1..2] of string  = ('X','Y');
      uniteDefaut : array[1..2] of string  = ('V','V');
      convertDefaut : array[1..2] of string  = ('V*5.0/1024','V*5.0/1024');
var i,j,col : integer;
    Rini : TMemIniFile;
    m : indiceMondeArduino;
    duree : double;
    indexCom : integer;
begin
     voieSerie := TXComPort.Create(self);
     voieSerie.DataBits := db8;
     voieSerie.Parity := paNone;
     voieSerie.StopBits := sb1;
     grapheA := TgrapheArduino.create(paintBox.canvas);
     grapheA.isOscillo := true;
     uniteTemps := Tunite.Create;
     uniteTemps.nomUnite := 's';
     SerialDevices := TStringList.create;
     getSerialdevices;
     Rini := TMemIniFile.create(registreReg);
     VoieSerie.deviceName := Rini.ReadString('Arduino','Com','');
     indexCom := SerialDevices.IndexOf(VoieSerie.deviceName);
     if indexCom<0 then if serialDevices.Count>0
        then VoieSerie.deviceName := serialDevices[0]
        else VoieSerie.deviceName := '';
     VoieSerie.BaudRate := TbaudRate(Rini.ReadInteger('Arduino','Baud',ord(br57600)));
     NbrePointsArduino := Rini.ReadInteger('Arduino','NbreOscillo',256);
     FechList.ItemIndex := Rini.readInteger('Arduino','Fech',5);
     resolution := Tresolution(Rini.readInteger('Arduino','Resolution',1));
     SynchroCB.itemIndex := Rini.readInteger('Arduino','Synchro',0);
     singleCB.isChecked := Rini.readBool('Arduino','Monocoup',true);
     grapheA.ordreSinc := Rini.readInteger('Arduino','OrdreSinc',2);
     grapheA.avecSinc := Rini.readBool('Arduino','Sinc',false);
     for j := 1 to 2 do begin
         col := j+1;
         grid.columnWidths[col] := 200;
         grid.Cells[col,0] := Rini.readString('Arduino','Nom'+intToStr(j),nomDefaut[j]);
         grid.Cells[col,1] := Rini.readString('Arduino','Unite'+intToStr(j),uniteDefaut[j]);
         grid.Cells[col,2] := Rini.readString('Arduino','Convert'+intToStr(j),convertDefaut[j]);
         for i := 0 to 2 do
             grid.FontColors[Col,i] := couleurArduino[j];
     end;
     NbreVoiesSE.value := Rini.readInteger('Arduino','NVoies',2);
     {$IFDEF Debug}
      voieSerie.logFile := System.IOUtils.TPath.combine(DocumentsPath,'Arduino.log');
     {$ELSE}
      voieSerie.logFile := '';
     {$ENDIF}
     liste := TstringList.Create;
     grid.Cells[0,0] := 'nom';
     grid.Cells[0,1] := 'unit';
     grid.Cells[0,2] := 'conversion';
     grid.Cells[1,0] := 't';
     grid.Cells[1,1] := 's';
     grid.Cells[1,2] := '';
     grapheA.monde[mondeX].mini := 0;
     grapheA.mondeMax := round(NbreVoiesSE.value);
     for j := 1 to 2 do begin
         grapheA.monde[j].maxiStr := '1023';
         grapheA.monde[j].nom := grid.Cells[j+1,0];
         grapheA.monde[j].unite := grid.Cells[j+1,1];
     end;
     for m := mondeX to mondeYmax do
         grapheA.monde[m].miniStr := '0';
     grapheA.monde[0].nom := 't';
     grapheA.monde[0].unite := 's';
     Parser := Tparser.create(nil);
     duree := PeriodeEch*NbrePointsArduino;
     dureeLabel.text := '  Dure = '+uniteTemps.formatValeurEtUnite(duree/1e6);
     grapheA.monde[mondeX].maxiStr := FloatToStrF(duree,ffFixed,6,1); // en microseconde
     ProgressBar.value := 0;
     FechOK := false;
     NbreVoiesOK := false;
     synchroOK := false;
     seuilOK := false;
end;// formCreate

procedure TArduinoOscilloForm.activeVoie;
var m : indiceMondeArduino;
begin
     if voieSerie.DeviceName<>'' then begin
         try
         voieSerie.active := True;
         voieSerie.clear;
         startBtn.Enabled := true;
         stopBtn.Enabled := false;
         except
            ShowMessage('Problme voie srie');
         end;
         for m := mondeY to mondeYmax do
             grapheA.monde[m].maxiStr := MaxResol[resolution].toString;
         TriggerTB.Max := MaxResol[resolution];
         FechOK := false;
         NbreVoiesOK := false;
         synchroOK := false;
         seuilOK := false;
     end;
end;

procedure TArduinoOscilloForm.FormDestroy(Sender: TObject);
begin
    parser.Free;
    uniteTemps.free;
end;

procedure TArduinoOscilloForm.FormShow(Sender: TObject);
begin
     activeVoie;
end;

procedure TArduinoOscilloForm.NbreVoiesSEChange(Sender: TObject);
begin
     NbreVoiesOK := false;
     if stopBtn.enabled then begin
        StopBtnClick(sender);
        StartBtnClick(sender);
     end;
end;

procedure TArduinoOscilloForm.PaintBoxPaint(Sender: TObject; Canvas: TCanvas);
begin
       grapheA.LimiteCourbe := paintBox.ClipRect;
       grapheA.drawG;
end;

procedure TArduinoOscilloForm.regressiBtnClick(Sender: TObject);
var imax : integer;

function Ajoute(nom,unite : string) : codeGrandeur;
var  v : codeGrandeur;
begin
      v :=indexNom(nom);
      if v=grandeurInconnue then begin
         v := ajouteExperimentale(Nom,variable);
         grandeurs[v].NomUnite := unite;
      end;
      result := v;
end;

procedure AffecteValeur;
var m : indiceMondeArduino;
    i : integer;
begin
   imax := pred(grapheA.nbrePoints);
   for m := mondeY to grapheA.mondeMax do
         if grid.cells[m+1,2]<>'' then begin
            Parser.expression := grid.cells[m+1,2];
            for i := 0 to imax do begin // conversion
                try
                   Parser.V := grapheA.monde[m].valeur[i];
                   valeurReal[m,i] :=  Parser.Value;
                except
                   valeurReal[m,i] := grapheA.monde[m].valeur[i];
                end;
            end;
         end
         else begin
              for i := 0 to imax do
                  valeurReal[m,i] := grapheA.monde[m].valeur[i];
         end;
   for i := 0 to imax do // data
         valeurReal[0,i] := grapheA.monde[0].valeur[i]/1e6;
end;

procedure AffecteSinc;
var m : indiceMondeArduino;
    i : integer;
begin
   imax := pred(grapheA.nbreSinc);
   for m := mondeY to grapheA.mondeMax do
         if grid.cells[m+1,2]<>'' then begin
            Parser.expression := grid.cells[m+1,2];
            for i := 0 to imax do begin // conversion
                try
                   Parser.V := grapheA.monde[m].valeurSinc[i];
                   valeurReal[m,i] :=  Parser.Value;
                except
                   valeurReal[m,i] := grapheA.monde[m].valeurSinc[i];
                end;
            end;
         end
         else begin
              for i := 0 to imax do
                  valeurReal[m,i] := grapheA.monde[m].valeurSinc[i];
         end;
   for i := 0 to imax do // data
       valeurReal[0,i] := grapheA.monde[0].valeurSinc[i]/1e6;
end;

var i : integer;
    m : indiceMondeArduino;
    indexXY : array[indiceMondeArduino] of codeGrandeur;
begin
     stopBtnClick(sender);
     indexXY[0] := ajoute('t','s');
     for m := mondeY to grapheA.mondeMax do begin
         if grid.Cells[m+1,0]='' then grid.Cells[m+1,0] := 'x'+intToStr(m);
         indexXY[m] := ajoute(grid.Cells[m+1,0],grid.Cells[m+1,1]);
     end;
     ajoutePage;
     pages[pageCourante].nmes := grapheA.nbrePoints;
     if grapheA.avecSinc then affecteSinc else affecteValeur;
     for i := 0 to imax do begin // data
         for m := mondeX to grapheA.mondeMax do
             pages[pageCourante].valeurVar[indexXY[m],i] := valeurReal[m,i];
     end;
     modeAcquisition := acqArduino;
     Close;
end;

procedure TArduinoOscilloForm.startBtnClick(Sender: TObject);
begin
    nbrePointsCourant := 0;
    startBtn.Enabled := False;
    stopBtn.Enabled := false;
    if not FechOK then
       FechOK := envoieCommande('T'+intToStr(PeriodeEch));
    if not NbrevoiesOK then begin
       NbreVoies := round(NbreVoiesSE.value);
       NbreVoiesOK := envoieCommande('C'+intTosTr(NbreVoies));
       grapheA.mondeMax := NbreVoies;
    end;
    if not synchroOK then
       synchroOK := envoieCommande('H'+intToStr(SynchroCB.itemIndex));
    if not seuilOK and (synchroCB.ItemIndex<>0) then
       seuilOK := envoieCommande('G'+intToStr(round(TriggerTB.max)-round(TriggerTB.value)));
    voieSerie.ClearInput;
    chaineLue := '';
    timerAcq.Enabled := true;
    ProgressBar.value := 0;
    ProgressBar.max := NbrePointsArduino;
    stopBtn.Enabled := True;
    envoieCommande('Start');
end;

procedure TArduinoOscilloForm.StopBtnClick(Sender: TObject);
begin
    envoieCommande('Stop');
    stopBtn.Enabled := false;
    startBtn.Enabled := true;
    timerAcq.Enabled := false;
    paintBox.Repaint;
    regressiBtn.Enabled := grapheA.nbrePoints>100;
end;


procedure TArduinoOscilloForm.synchroCBChange(Sender: TObject);
begin
    synchroOK := false;
    triggerTB.Visible := SynchroCB.itemIndex>0;
    if stopBtn.enabled then begin
       StopBtnClick(sender);
       StartBtnClick(sender);
    end;
end;

procedure TArduinoOscilloForm.TimerAcqTimer(Sender: TObject);
var carac : AnsiChar;
    Ncarac : integer;
    buffer : array of AnsiChar;
    i : integer;
begin
// au plus rapide : 115200 bauds 1 voie sur 3 caractres + "," + temps sur 3 carac + CR + LF
// => 9 caractres => en gros 1 ligne par milliseconde
        timerAcq.Enabled := false;
        try
        Ncarac := voieSerie.InputCount;
        if Ncarac>0 then begin
           setLength(buffer,Ncarac);
           voieSerie.Read(Buffer,Ncarac);
           for i := 0 to Ncarac-1 do begin
              carac := buffer[i];
              if carac>=' '
                 then chaineLue := chaineLue + carac
                 else if chaineLue<>'' then begin // premier caractere de controle CR NL ...
                    TriggerData(chaineLue); // on envoie ligne par ligne
                    chaineLue := '';
                 end;
           end;
        end;
        timerAcq.Enabled := true;
        except
            voieSerie.Active := false;
            StopBtn.enabled := false;
            StartBtn.enabled := false;
            toolsBtn.enabled := true;
            ShowMessage('Vrifier la voie srie');
        end;
end;

procedure TArduinoOscilloForm.TimerStopTimer(Sender: TObject);
begin
    timerStop.Enabled := false;
end;

procedure TArduinoOscilloForm.toolsBtnClick(Sender: TObject);

procedure AffecteIn;
var i : integer;
begin
   ArduinoOscilloDlg.DeviceNameCB.Items.Clear;
   ArduinoOscilloDlg.ResolutionCB.itemIndex := ord(resolution);
   for i := 0 to pred(SerialDevices.count) do
       ArduinoOscilloDlg.DeviceNameCB.Items.Add(SerialDevices[i]);
   ArduinoOscilloDlg.DeviceNameCB.ItemIndex := SerialDevices.IndexOf(VoieSerie.deviceName);
   if ArduinoOscilloDlg.DeviceNameCB.ItemIndex<0
      then ArduinoOscilloDlg.DeviceNameCB.ItemIndex := 0;
   for i := 0 to pred(ArduinoOscilloDlg.BaudCB.Items.count) do
      if ord(voieSerie.baudRate)=i+ord(br9600) then
            ArduinoOscilloDlg.baudCB.ItemIndex := i;
   if ArduinoOscilloDlg.baudCB.ItemIndex<0
      then ArduinoOscilloDlg.baudCB.ItemIndex := 0;
   for i := 0 to pred(ArduinoOscilloDlg.nbreCB.items.count) do
       if nbrePointsArduino.toString=ArduinoOscilloDlg.nbreCB.items[i] then
           ArduinoOscilloDlg.nbreCB.ItemIndex := i;
   ArduinoOscilloDlg.sincCB.IsChecked := grapheA.avecSinc;
   ArduinoOscilloDlg.sincSB.value := grapheA.ordreSinc;
 end;

procedure AffecteOut;
var duree : double;
begin
     voieSerie.baudRate := TbaudRate(ArduinoOscilloDlg.BaudCB.ItemIndex+ord(br9600));
     if ArduinoOscilloDlg.DeviceNameCB.itemIndex>=0 then
        VoieSerie.DeviceName := ArduinoOscilloDlg.DeviceNameCB.Items[ArduinoOscilloDlg.DeviceNameCB.itemIndex];
     resolution := Tresolution(ArduinoOscilloDlg.ResolutionCB.itemIndex);
     nbrePointsArduino := strToInt(ArduinoOscilloDlg.nbreCB.items[ArduinoOscilloDlg.nbreCB.ItemIndex]);
     grapheA.ordreSinc := round(ArduinoOscilloDlg.sincSB.value);
     grapheA.avecSinc := ArduinoOscilloDlg.sincCB.IsChecked and (grapheA.ordreSinc>1);
     duree := PeriodeEch*NbrePointsArduino;
     dureeLabel.text := '  Dure = '+uniteTemps.formatValeurEtUnite(duree/1e6);
     grapheA.monde[mondeX].maxiStr := FloatToStrF(duree,ffFixed,6,1); // en microseconde
end;

begin
     TimerAcq.Enabled := false;
     VoieSerie.active := False;
     VoieSerie.clear;
     getSerialdevices;
     if SerialDevices.count=0 then begin
        showMessage('Pas de voie srie');
        exit;
     end;
     if ArduinoOscilloDlg=nil then Application.CreateForm(TArduinoOscilloDlg,ArduinoOscilloDlg);
     affecteIn;
     if ArduinoOscilloDlg.ShowModal=mrOK then affecteOut;
     activeVoie;
end;

procedure TArduinoOscilloForm.triggerTBChange(Sender: TObject);
begin
    seuilOK := false;
    if stopBtn.enabled then begin
       StopBtnClick(sender);
       StartBtnClick(sender);
    end;
end;

procedure TArduinoOscilloForm.TriggerData(aChaine : ansiString);
var i : integer;
    m : indiceMondeArduino;
begin
      liste.commaText := string(aChaine);
      if (NbrePointsCourant=0) and (liste.Count<(NbreVoies+1)) then  begin // premier point incomplet ?
          grapheA.monde[0].valeurNew[0] := 0;
          for i := 1 to NbreVoies do
             grapheA.monde[i].valeurNew[i] := 512;
          for i := 0 to liste.Count-1 do begin
             try
             grapheA.monde[i].valeurNew[i+NbreVoies+1-liste.count] := strToInt(liste[i]);
             except
             end;
          end;
          NbrePointsCourant := 1;
      end;
      if liste.Count<>(NbreVoies+1) then exit;
      if (NbrePointsCourant=0) and (liste[0]<>'0') then  begin // premier point perdu ?
          grapheA.monde[0].valeurNew[0] := 0;
          for i := 1 to liste.Count-1 do begin
            try
            grapheA.monde[i].valeurNew[0] := strToInt(liste[i]);
            except
            grapheA.monde[i].valeurNew[0] := 512;
            end;
          end;
          NbrePointsCourant := 1;
      end;
      for i := 0 to liste.Count-1 do begin
          try
          grapheA.monde[i].valeurNew[nbrePointsCourant] := strToInt(liste[i]);
          except
             if i=mondeX
                then grapheA.monde[i].valeurNew[nbrePointsCourant] := NbrePointsCourant*periodeEch
                else grapheA.monde[i].valeurNew[nbrePointsCourant] := 512;
          end;
      end;
      inc(nbrePointsCourant);
      if ((progressBar.Value+32)<NbrePointsCourant) then begin
           progressBar.Value := NbrePointsCourant;
           progressBar.Repaint;
      end;
      if nbrePointsCourant>=NbrePointsArduino then begin
            regressiBtn.Enabled := true;
            grapheA.nbrePoints := nbrePointsArduino;
            progressBar.Value := NbrePointsArduino;
            progressBar.Repaint;
            for i := 0 to pred(nbrePointsArduino) do begin
                grapheA.monde[0].valeur[i] := grapheA.monde[0].valeurNew[i];
                for m := mondeY to grapheA.mondeMax do
                    grapheA.monde[m].valeur[i] := grapheA.monde[m].valeurNew[i];
            end;
            paintBox.Repaint;
            if singleCB.isChecked
               then begin
                  timerAcq.Enabled := false;
                  startBtn.Enabled := True;
                  stopBtn.Enabled := false;
               end
               else StartBtnClick(nil); // Start ncessaire pour synchro
      end;
end;

function TArduinoOscilloForm.EnvoieCommande(aCommand : string) : boolean;
var i : integer;
begin
   if voieSerie.active then begin
       try
       cursor := crHourGlass;
       for i := 1 to aCommand.length do
           VoieSerie.WriteAnsiChar(AnsiChar(aCommand[i]));
       VoieSerie.WriteAnsiChar(#10); // LF
       sleep(pauseRS232); // pour que Arduino digre
       result := true;
       except
            result := false;
            voieSerie.Active := false;
            timerAcq.Enabled := false;
            StopBtn.enabled := false;
            StartBtn.enabled := false;
            toolsBtn.enabled := true;
            ShowMessage('Vrifier la voies srie');
       end;
       cursor := crDefault;
   end
   else result := false;
end;


function TArduinoOscilloForm.PeriodeEch : integer;
var chaine,intStr : string;
    i : integer;
    coeff : integer;
    carac : char;
begin
   chaine := FechList.items[FechList.ItemIndex];
   i := 1;
   intStr := '';
   coeff := 1;
   while i<=length(chaine) do begin
       carac := chaine[i];
       if charInSet(carac,['0'..'9'])
          then intStr := intStr+carac
          else if carac='k' then coeff := 1000;
       inc(i);
   end;
   result := strToInt(intStr)*coeff;
   result := round(1e6/result);
end;

end.
